// ============================================================================
//
// Unit:    ibeDALConnectionFB
// Inhalt:  Formular
// Aufgabe: DAL Connection fr Firebird Datenbank
//
//   (c) 2015 by Klaus Etscheid
// ============================================================================
unit ibeDALConnectionFB;

interface

uses
	SysUtils, Classes, DB,
	ibeTypes, ibeDALConnection, ibeDALQuery;

type

TibeDALConnectionFB = class(TibeDALConnection)
private
  Fq: TibeDALQuery;
protected
	procedure DoAfterInternalCreate(); override;
  function GetIsSupportedSequence(): Boolean; override;
  function GetIsSupportedTempTable(): Boolean; override;
  property q: TibeDALQuery read Fq;
public
	destructor Destroy; override;
  function RetrieveIdentityValueAfterInsert(const AName: string = ''): Variant; override;
  function RetrieveSequenceValue(const AName: string = ''): Variant; override;
  procedure RetrieveTableList(a_slTableNames: TStringList); override;
  function CVBoole(a_bValue: Boolean): string; override;
  function CVDate(a_dtDate: TDateTime): string; override;
  function CVDateTime(a_dtDate: TDateTime): string; override;
  function CVFloat(a_nValue: ibeFloat): string; override;
end;

implementation

procedure TibeDALConnectionFB.DoAfterInternalCreate(); 
begin
  inherited DoAfterInternalCreate();

  Fq := DAOQueryCreate();
end;

destructor TibeDALConnectionFB.Destroy; 
begin
  inherited Destroy();

  FreeAndNil(Fq);
end;

function TibeDALConnectionFB.RetrieveIdentityValueAfterInsert(const AName: string = ''): Variant;
begin
  q.SQL := 'select gen_id(' + AName + ', 0) as Last_ID from RDB$DATABASE';
  q.Open();
  
  Result := q.DataSet.FieldByName('Last_ID').Value;
end;

function TibeDALConnectionFB.RetrieveSequenceValue(const AName: string = ''): Variant; 
begin
  q.SQL := 'select next value for ' + AName + ' as New_ID from RDB$DATABASE';
  q.Open();
  
  Result := q.DataSet.FieldByName('New_ID').Value;
end;

procedure TibeDALConnectionFB.RetrieveTableList(a_slTableNames: TStringList);
begin
  q.Close;
  q.SQL := 'select rdb$relation_name as TableName' + #13#10
         + 'from rdb$relations' + #13#10
         + 'where rdb$view_blr is null' + #13#10
         + '  and (rdb$system_flag is null or rdb$system_flag = 0)';
  q.Open;

  while not q.DataSet.Eof do begin
    a_slTableNames.Add(Trim(q.DataSet.FieldByName('TableName').AsString));
    q.DataSet.Next;
  end;
end;

function TibeDALConnectionFB.CVBoole(a_bValue: Boolean): string; 
begin
  if a_bValue then Result := '1'
  else             Result := '0';
end;

function TibeDALConnectionFB.CVDate(a_dtDate: TDateTime): string; 
begin
	Result := FormatDateTime('yyyy-mm-dd', a_dtDate);
end;

function TibeDALConnectionFB.CVDateTime(a_dtDate: TDateTime): string; 
begin
	Result := FormatDateTime('yyyy-mm-dd hh:nn:ss', a_dtDate);
end;

function TibeDALConnectionFB.CVFloat(a_nValue: ibeFloat): string; 
var OrigDecimalSeperator, OrigThousandSeperator: Char;
begin
	OrigDecimalSeperator := FormatSettings.DecimalSeparator;
	OrigThousandSeperator := FormatSettings.ThousandSeparator;
	FormatSettings.DecimalSeparator := '.';
	FormatSettings.ThousandSeparator := ' ';
	try
		Result := FloatToStr(a_nValue);
	finally
		FormatSettings.DecimalSeparator := OrigDecimalSeperator;
		FormatSettings.ThousandSeparator := OrigThousandSeperator;
	end;
end;

function TibeDALConnectionFB.GetIsSupportedSequence(): Boolean; 
begin
  Result := True;
end;

function TibeDALConnectionFB.GetIsSupportedTempTable(): Boolean; 
begin
  Result := False;
end;

end.
