unit ibeDALConnectionManager;

interface

uses
	SysUtils, Classes, IniFiles,
	ibeException, ibeDALConnection;

type

TibeDALConnectionManager = class
private
	FDBConfig: TIniFile;
  FConnectionNames: TStringList;
	function GetCfgFileName: TFileName;
	procedure SetCfgFileName(aFileName: TFileName);
protected
	property DBConfig: TIniFile read FDBConfig;
public
  constructor Create; virtual;
	destructor Destroy; override;
	function RetriveConnection(aOwner: TComponent; const aConnectionName: string): TibeDALConnection;
	property CfgFileName: TFileName read GetCfgFileName write SetCfgFileName;
  property ConnectionNames: TStringList read FConnectionNames;
end;

EibeDALConnectionManager = class(EibeException);

implementation

uses
	ibeFuncCommon;

constructor TibeDALConnectionManager.Create;
begin
  inherited Create;

  FConnectionNames := TStringList.Create;
end;

destructor TibeDALConnectionManager.Destroy;
begin
	inherited Destroy;

  ibeFreeAndNil(FConnectionNames);
	ibeFreeAndNil(FDBConfig);
end;

function TibeDALConnectionManager.RetriveConnection(aOwner: TComponent; const aConnectionName: string): TibeDALConnection;
var sConnectionTypeID: string;
	connection: TibeDALConnection;
  sl: TStringList;
  i: Integer;
begin
	if DBConfig.SectionExists(aConnectionName) then
  begin
		sConnectionTypeID := DBConfig.ReadString(aConnectionName, 'ConnectionType', '[undefined]');

		connection := TibeDALConnectionFactory.CreateInstance(aOwner, sConnectionTypeID);
		try
      sl := TStringList.Create;
      try
  			DBConfig.ReadSection(aConnectionName, sl);
        for i := 0 to sl.Count - 1 do
        begin
          connection.Parameters.Values[sl[i]] := DBConfig.ReadString(aConnectionName, sl[i], '');
        end;
      finally
        ibeFreeAndNil(sl);
      end;

			connection.Setup;
		except
			on E: Exception do begin
				ibeFreeAndNil(connection);
				raise EibeDALConnectionManager.Create(Self, 'ConnectionManager.RetriveConnection(''' + aConnectionName + '''): es ist eine Ausnahme der Klasse ' + E.ClassName + ' aufgetreten. Text der Ausnahme: "' + e.Message + '"');
			end;
		end;
	end else
  begin
		raise EibeDALConnectionManager.Create(Self, 'ConnectionManager.RetriveConnection(''' + aConnectionName + '''): die angegebene Connection ist nicht konfiguriert');
	end;

	Result := connection;
end;

function TibeDALConnectionManager.GetCfgFileName: TFileName;
begin
	if Assigned(DBConfig) then begin
		Result := DBConfig.FileName;
	end else begin
		Result := '';
	end;
end;

procedure TibeDALConnectionManager.SetCfgFileName(aFileName: TFileName);
var sl: TStringList;
  i: Integer;
  LConnectionType: string;
begin
	FreeAndNil(FDBConfig);

	FDBConfig := TIniFile.Create(aFileName);

  // die im Config-File enthaltenen Connections in die Liste der ConnectionNames eintragen
  sl := TStringList.Create;
  try
    ConnectionNames.Clear;
    DBConfig.ReadSections(sl);
    for i := 0 to sl.Count - 1 do 
    begin
      LConnectionType := DBConfig.ReadString(sl[i], 'ConnectionType', '');
      if LConnectionType <> '' then
      begin
        ConnectionNames.Values[sl[i]] := DBConfig.ReadString(sl[i], 'Description', sl[i]);
      end;
    end;
  finally
    ibeFreeAndNil(sl);
  end;
end;

end.

