// ============================================================================
//
// Unit:    ibeDALQueryFB_IB
// Inhalt:  Formular
// Aufgabe: DAL Query fr Firebird Datenbank mit InterBase Express Komponenten
//
//   (c) 2015 by Klaus Etscheid
// ============================================================================
unit ibeDALQueryFB_IB;

interface

uses
  SysUtils, DB,
  IBX.IBDatabase, IBX.IBQuery, IBX.IBScript,
  ibeDALConnection, ibeDALQuery, ibeDALQueryFB;

type

TibeDALQueryFB_IB = class(TibeDALQueryFB)
private
  FScript: TIBScript;
  function GetQry: TIBQuery;
protected
  function CreateDataset(): TDataSet; override;
  procedure InternalExecute(); override;
  procedure InternalSetParameterValue(const AParameterName: string; const AParameterValue: Variant); override;
  function GetRecordCount(): Integer; override;
  function GetActive: Boolean; override;
  procedure SetActive(a_bActive: Boolean); override;
  Function GetSQL(): WideString; override;
  procedure SetSQL(const a_sSQL: WideString); override;
  property qry: TIBQuery read GetQry;
public
  constructor Create(); override;
  destructor Destroy(); override;
end;

implementation

constructor TibeDALQueryFB_IB.Create();
begin
  inherited Create();

  FScript := TIBScript.Create(nil);
end;

destructor TibeDALQueryFB_IB.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FScript);
end;

function TibeDALQueryFB_IB.CreateDataset(): TDataSet; 
begin
  Result := TIBQuery.Create(nil);
end;

function TibeDALQueryFB_IB.GetRecordCount(): Integer;
var
  LBookmark: TBookmark;
begin
  LBookmark := qry.GetBookmark();
  try
    // Recordcount kann erst ausgelesen werden, wenn alle Datenstze gefetcht sind.
    // deshalb erst die Bookmark vom derzeit aktuellen Datensatz holen, dann
    // auf den letzten Datensatz springen, und mittels Bookmark auf den anfangs
    // aktuellen Datensatz zurck positionieren
    qry.Last();
    qry.GotoBookmark(LBookmark);
    Result := qry.RecordCount;
  finally
    qry.FreeBookmark(LBookmark);
  end;
end;

function TibeDALQueryFB_IB.GetActive: Boolean;
begin
  Result := qry.Active;
end;

procedure TibeDALQueryFB_IB.SetActive(a_bActive: Boolean); 
begin
  qry.Active := a_bActive;
end;

procedure TibeDALQueryFB_IB.InternalExecute(); 
begin
//  qry.ExecSQL();
  if not Assigned(FScript.Database) then
  begin
    FScript.Database := TIBQuery(DataSet).Database;
  end;

  FScript.Script.Text := TIBQuery(DataSet).SQL.Text;
  FScript.ValidateScript();
  FScript.ExecuteScript();
end;

procedure TibeDALQueryFB_IB.InternalSetParameterValue(const AParameterName: string; const AParameterValue: Variant);
begin
  qry.ParamByName(AParameterName).Value := AParameterValue;
end;

Function TibeDALQueryFB_IB.GetSQL(): WideString;
begin
  Result := qry.SQL.Text;
end;

procedure TibeDALQueryFB_IB.SetSQL(const a_sSQL: WideString); 
begin
  qry.SQL.Text := a_sSQL;
end;

function TibeDALQueryFB_IB.GetQry(): TIBQuery;
begin
  if not (DataSet is TIBQuery) then begin
    raise EibeDALQuery.Create(Self, 'Interbase Query Dataset: das Dataset ist von der Klasse ' + DataSet.ClassName + ' und kann nicht als TIBQuery angesprochen werden');
  end;

  Result := TIBQuery(DataSet);
end;

end.
