// ============================================================================
//
// Unit:    ibeFCommon
// Inhalt:  Formulartemplate
// Aufgabe: Basis fr alle anderen Formulare. Bereitstellen rudimentrer Funktionen.
//          Das Formular enthlt ein Panel das die gesamte Formularflche einnimmt.
//          Aufgabe des Panels ist es die bei Embedded implementierten Formularen
//          nicht mehr ausgelste Formularereignisse OnActivate und OnDeactivate
//          zu simulieren. Dazu werden hier die OnEnter-/OnExit-Ereignisse vom
//          Panel verwendet, die dann die Ereignismethoden InternalActivateForm()
//          und InternalDeactivateForm() vom Formular auslsen.
//          In den OnEnter-/OnExit-Ereignissen des Panels werden auch die
//          Shortcuts der Actions aus alMain aktiviert bzw. deaktiviert.
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeFCommon;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, System.Actions, Vcl.ExtCtrls, Generics.Collections,
  ibeAppServiceProvider;

type

  TDeactivatedActionShortcut = class
  private
    FAction: TAction;
    FShortcut: TShortCut;
  protected
  public
    constructor Create(AAction: TAction);
    property Action: TAction read FAction;
    property Shortcut: TShortCut read FShortcut;
  end;

  TDeactivatedActionShortcutList = class(TObjectList<TDeactivatedActionShortcut>);

  TfrmCommon = class(TForm)
    alMain: TActionList;
    pnlFormMain: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure pnlFormMainEnter(Sender: TObject);
    procedure pnlFormMainExit(Sender: TObject);
  private
    FDeactivatedActionShortcuts: TDeactivatedActionShortcutList;
		function GetASP: TibeAppServiceProvider;
  protected
    class procedure InternalRegisterProgramFunction(); virtual;
    procedure InternalInit; virtual;
    procedure InternalActivateForm(); virtual;
    procedure InternalDeactivateForm(); virtual;
    function GetShortCaption: string; virtual;
	public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
    class procedure DoRegisterProgramFunction();
		property ASP: TibeAppServiceProvider read GetASP;
    property ShortCaption: string read GetShortCaption;
    procedure DisplayInContainer(a_ctlParent: TWinControl);
	end;

TfrmCommonClass = class of TfrmCommon;

implementation

{$R *.dfm}

constructor TfrmCommon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FDeactivatedActionShortcuts := TDeactivatedActionShortcutList.Create(True);
end;

destructor TfrmCommon.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FDeactivatedActionShortcuts);
end;

class procedure TfrmCommon.DoRegisterProgramFunction();
begin
  InternalRegisterProgramFunction();
end;

procedure TfrmCommon.DisplayInContainer(a_ctlParent: TWinControl);
begin
  BorderStyle := bsNone;
  Parent := a_ctlParent;
  Show;
  Align := alClient;
end;

procedure TfrmCommon.FormCreate(Sender: TObject);
begin
  InternalInit;
end;

procedure TfrmCommon.InternalInit;
begin
end;

class procedure TfrmCommon.InternalRegisterProgramFunction();
begin
end;

procedure TfrmCommon.InternalActivateForm();
begin
end;

procedure TfrmCommon.InternalDeactivateForm();
begin
end;

procedure TfrmCommon.pnlFormMainEnter(Sender: TObject);
var
  LDeactivatedActionShortcut: TDeactivatedActionShortcut;
begin
  for LDeactivatedActionShortcut in FDeactivatedActionShortcuts do
  begin
    LDeactivatedActionShortcut.Action.ShortCut := LDeactivatedActionShortcut.Shortcut;
  end;
  FDeactivatedActionShortcuts.Clear();

  InternalActivateForm();
end;

procedure TfrmCommon.pnlFormMainExit(Sender: TObject);
var
  i: Integer;
  LDeactivatedActionShortcut: TDeactivatedActionShortcut;
begin
  for i := 0 to alMain.ActionCount - 1 do
  begin
    if alMain.Actions[i].ShortCut <> 0 then
    begin
      LDeactivatedActionShortcut := TDeactivatedActionShortcut.Create(TAction(alMain.Actions[i]));
      FDeactivatedActionShortcuts.Add(LDeactivatedActionShortcut);
    end;
  end;

  InternalDeactivateForm();
end;

function TfrmCommon.GetASP: TibeAppServiceProvider;
begin
	Result := RetrieveASP;
end;

function TfrmCommon.GetShortCaption: string;
begin
  Result := Caption;
end;

// -+-+-+- TDeactivatedActionShortcut -+-+-+-

constructor TDeactivatedActionShortcut.Create(AAction: TAction);
begin
  inherited Create();

  FAction := AAction;
  FShortcut := AAction.ShortCut;

  AAction.ShortCut := 0;
end;

end.
