unit ibeFCommonContainer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, ActnList, Menus, DB, Generics.Collections,
  ibeFCommon, System.Actions;

type

TibeContainerForm = class
private
  FForm: TfrmCommon;
  FFormNeedsRefresh: Boolean;
  FContainer: TWinControl;
protected
public
  constructor Create(a_Form: TfrmCommon; a_Container: TWinControl);
  property Form: TfrmCommon read FForm write FForm;
  property FormNeedsRefresh: Boolean read FFormNeedsRefresh write FFormNeedsRefresh;
  property Container: TWinControl read FContainer write FContainer;
end;

TibeContainerFormList = TObjectList<TibeContainerForm>;

  TfrmCommonContainer = class(TfrmCommon)
    mnuPopPages: TPopupMenu;
    actPageClose: TAction;
    actPageMoveRight: TAction;
    actPageMoveLeft: TAction;
    Links1: TMenuItem;
    Rechts1: TMenuItem;
    Schlieen1: TMenuItem;
    mnuPopPageDelim1: TMenuItem;
    pnlTop: TPanel;
    pgcMain: TPageControl;
    procedure pgcMainChange(Sender: TObject);
    procedure pgcMainChanging(Sender: TObject; var AllowChange: Boolean);
    procedure actPageCloseExecute(Sender: TObject);
    procedure actPageMoveRightExecute(Sender: TObject);
    procedure actPageMoveLeftExecute(Sender: TObject);
  private
    FFormList: TibeContainerFormList;
    function DoActivateContainerForm(AContainerForm: TibeContainerForm): Boolean;
    function DoBeforeContainerFormChanged(AContainerForm: TibeContainerForm): Boolean;
    procedure DoAfterContainerFormChanged(AContainerForm: TibeContainerForm); virtual;
  protected
    property FormList: TibeContainerFormList read FFormList;
    function ActivateContainerForm(AContainerForm: TibeContainerForm): Boolean;
    function DeleteContainerForm(AContainerForm: TibeContainerForm): Boolean;
    procedure InternalAfterFormAdd(AFrm: TfrmCommon); virtual;
    function InternalBeforeContainerChanged(ACurrentForm: TfrmCommon): Boolean;
    procedure InternalAfterContainerChanged(ANewForm: TfrmCommon); virtual;
    function RetrieveContainerForm(AFrm: TfrmCommon): TibeContainerForm; overload;
    function RetrieveContainerForm(AContainer: TWinControl): TibeContainerForm; overload;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
    function AddForm(AFormClass: TfrmCommonClass; a_bActivateForm: Boolean): TfrmCommon;
    procedure ActivateForm(AForm: TfrmCommon);
    procedure ActivateContainer(AContainer: TWinControl);
  end;

implementation

uses
  ibeFuncCommon,
  ibeFCommonList;

{$R *.dfm}

constructor TfrmCommonContainer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFormList := TibeContainerFormList.Create;

  FormList.OwnsObjects := True;
end;

destructor TfrmCommonContainer.Destroy();
begin
  inherited Destroy();

  ibeFreeAndNil(FFormList);
end;

procedure TfrmCommonContainer.pgcMainChanging(Sender: TObject; var AllowChange: Boolean);
begin
  inherited;
  AllowChange := DoBeforeContainerFormChanged(RetrieveContainerForm(pgcMain.ActivePage));
end;

procedure TfrmCommonContainer.pgcMainChange(Sender: TObject);
begin
  inherited;
  DoActivateContainerForm(RetrieveContainerForm(pgcMain.ActivePage));
end;

function TfrmCommonContainer.AddForm(AFormClass: TfrmCommonClass; a_bActivateForm: Boolean): TfrmCommon;
var frm: TfrmCommon;
  ts: TTabSheet;
  cform: TibeContainerForm;
begin
  ts := TTabSheet.Create(pgcMain);
  ts.PageControl := pgcMain;

  frm := AFormClass.Create(Self);
  frm.DisplayInContainer(ts);

  cform := TibeContainerForm.Create(frm, ts);
  FormList.Add(cform);

  InternalAfterFormAdd(frm);

  ts.Caption := frm.ShortCaption;

  if a_bActivateForm then begin
    DoActivateContainerForm(cform);
  end;

  Result := frm;
end;

procedure TfrmCommonContainer.ActivateForm(AForm: TfrmCommon);
begin
  ActivateContainerForm(RetrieveContainerForm(AForm));
end;

procedure TfrmCommonContainer.ActivateContainer(AContainer: TWinControl);
begin
  ActivateContainerForm(RetrieveContainerForm(AContainer));
end;

function TfrmCommonContainer.ActivateContainerForm(AContainerForm: TibeContainerForm): Boolean;
var bOK: Boolean;
  cform: TibeContainerForm;
begin
  bOK := Assigned(AContainerForm);

  if bOK then begin
    cform := RetrieveContainerForm(pgcMain.ActivePage);
    bOK   := DoBeforeContainerFormChanged(cform);

    if bOK then begin
      bOK := DoActivateContainerForm(AContainerForm);
    end;
  end;

  Result := bOK;
end;

function TfrmCommonContainer.DeleteContainerForm(AContainerForm: TibeContainerForm): Boolean;
var
  LFormIndex: Integer;
  LOK: Boolean;
  LActiveContainerForm: TibeContainerForm;
begin
  // Container in der Formliste suchen
  LFormIndex := FormList.IndexOf(AContainerForm);
  // weiter, wenn der Container in der Liste verwaltet wird
  LOK := LFormIndex >= 0;

  if LOK then
  begin
    // prfen ob der zu lschende Container gerade ausgewhlt ist
    LActiveContainerForm := RetrieveContainerForm(pgcMain.ActivePage);

    if AContainerForm = LActiveContainerForm then
    begin
      // nur weitermachen, wenn der aktive Container gendert werden darf
      LOK := DoBeforeContainerFormChanged(AContainerForm)
    end;
  end;

  if LOK then
  begin
    // Formular und Tabsheet lschen
    AContainerForm.Form.Destroy();
    //TTabSheet(AContainerForm.Container).Visible := False;
    TTabSheet(AContainerForm.Container).Destroy();

    // Container lschen
    FormList.Delete(LFormIndex);
  end;

  Result := LOK;
end;

function TfrmCommonContainer.DoActivateContainerForm(AContainerForm: TibeContainerForm): Boolean;
var bOK: Boolean;
begin
  bOK := Assigned(AContainerForm);

  if bOK then begin
    if pgcMain.ActivePage <> AContainerForm.Container then begin
      pgcMain.ActivePage := TTabSheet(AContainerForm.Container);
    end;

    AContainerForm.Form.SetFocus;

    DoAfterContainerFormChanged(AContainerForm);
  end;

  Result := bOK;
end;

function TfrmCommonContainer.DoBeforeContainerFormChanged(AContainerForm: TibeContainerForm): Boolean;
var bOK: Boolean;
begin
  bOK   := Assigned(AContainerForm);

  if bOK then begin
    bOK := InternalBeforeContainerChanged(AContainerForm.Form);
  end;

  Result := bOK;
end;

procedure TfrmCommonContainer.DoAfterContainerFormChanged(AContainerForm: TibeContainerForm);
begin
  if AContainerForm.FormNeedsRefresh and (AContainerForm.Form is TibeFrmCommonList) then
  begin
    TibeFrmCommonList(AContainerForm.Form).ActionRefresh;

    AContainerForm.FormNeedsRefresh := False;
  end;

  InternalAfterContainerChanged(AContainerForm.Form);
end;

procedure TfrmCommonContainer.InternalAfterFormAdd(AFrm: TfrmCommon);
begin
end;

function TfrmCommonContainer.InternalBeforeContainerChanged(ACurrentForm: TfrmCommon): Boolean;
begin
  Result := True;
end;

procedure TfrmCommonContainer.InternalAfterContainerChanged(ANewForm: TfrmCommon);
begin
end;

function TfrmCommonContainer.RetrieveContainerForm(AFrm: TfrmCommon): TibeContainerForm;
var cform: TibeContainerForm;
begin
  cform := nil;

  for cform in FormList do begin
    if cform.Form = AFrm then begin
      break;
    end;
  end;

  Result := cform;
end;

function TfrmCommonContainer.RetrieveContainerForm(AContainer: TWinControl): TibeContainerForm;
var cform: TibeContainerForm;
begin
  cform := nil;

  for cform in FormList do begin
    if cform.Container = AContainer then begin
      break;
    end;
  end;

  Result := cform;
end;

procedure TfrmCommonContainer.actPageCloseExecute(Sender: TObject);
begin
  inherited;
  if Assigned(pgcMain.ActivePage) then begin
    pgcMain.ActivePage.Destroy;
  end;
end;

procedure TfrmCommonContainer.actPageMoveLeftExecute(Sender: TObject);
begin
  inherited;
  if Assigned(pgcMain.ActivePage) then begin
    if pgcMain.ActivePage.PageIndex > 0 then begin
      pgcMain.ActivePage.PageIndex := pgcMain.ActivePage.PageIndex - 1;
    end;
  end;
end;

procedure TfrmCommonContainer.actPageMoveRightExecute(Sender: TObject);
begin
  inherited;
  if Assigned(pgcMain.ActivePage) then begin
    if pgcMain.ActivePage.PageIndex < Pred(pgcMain.PageCount) then begin
      pgcMain.ActivePage.PageIndex := pgcMain.ActivePage.PageIndex + 1;
    end;
  end;
end;

constructor TibeContainerForm.Create(a_Form: TfrmCommon; a_Container: TWinControl);
begin
  FForm       := a_Form;
  FContainer  := a_Container;

  // ActionRefresh beim erstmaligen Aktivieren von Listenformularen automatisch aufrufen
  FormNeedsRefresh := True;
end;

end.
