unit ibeFCommonDOList;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ActnList, StdCtrls, ExtCtrls, DB, System.Actions,
  ibeFCommonDataList,
  ibeAppServiceProviderDB, ibeDALConnection, ibeDataObject;

type
  TibeFrmCommonDOList = class(TibeFrmCommonDataList)
  private
    FDOMain: TibeDataObject;
    FDoAddVisibleColumns: Boolean;
		function GetASP: TibeAppServiceProviderDB;
    function GetConnection: TibeDALConnection;
    function GetDOMainClass: TibeDataObjectClass;
    procedure SetDOMainClass(ADOClass: TibeDataObjectClass);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionRefresh(); override;
    procedure InternalActionDelete(); override;
    procedure DoBeforeRefresh(); override;
    property DOMain: TibeDataObject read FDOMain;
    property DOMainClass: TibeDataObjectClass read GetDOMainClass write SetDOMainClass;
    property DoAddVisibleColumns: Boolean read FDoAddVisibleColumns write FDoAddVisibleColumns;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
    property Connection: TibeDALConnection read GetConnection;
		property ASP: TibeAppServiceProviderDB read GetASP;
  end;

implementation

{$R *.dfm}

constructor TibeFrmCommonDOList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  DoAddVisibleColumns := True;
end;

destructor TibeFrmCommonDOList.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FDOMain);
end;

procedure TibeFrmCommonDOList.InternalActionInitGridMain();
var
  i: Integer;
begin
  inherited InternalActionInitGridMain();

  if DoAddVisibleColumns then
  begin
    for i := 0 to DOMain.FieldCount - 1 do
    begin
      if DOMain.FieldByIndex[i].VisibleInList then
      begin
        AddDataColumn(DOMain.FieldByIndex[i].FieldName, DOMain.FieldByIndex[i].Caption, DOMain.FieldByIndex[i].ColumnWidth);
      end;
    end;
  end;
end;

procedure TibeFrmCommonDOList.InternalActionRefresh();
begin
  inherited InternalActionRefresh();

  if Assigned(DOMain) then
  begin
    DOMain.Refresh();
    DOMain.DataCopyToDataset(DatMain);
  end;
end;

procedure TibeFrmCommonDOList.InternalActionDelete();
begin
  inherited InternalActionDelete();

  if Assigned(DOMain) then
  begin
    DOMain.Fetch(PKValue);
    DOMain.Delete();
  end;
end;

procedure TibeFrmCommonDOList.DoBeforeRefresh();
begin
  inherited DoBeforeRefresh();
  if Assigned(DOMain) then
  begin
    DOMain.FilterClear();
  end;
end;

function TibeFrmCommonDOList.GetDOMainClass: TibeDataObjectClass;
begin
  Result := TibeDataObjectClass(DOMain.ClassType);
end;

procedure TibeFrmCommonDOList.SetDOMainClass(ADOClass: TibeDataObjectClass);
begin
  if Assigned(DOMain) then
  begin
    FreeAndNil(FDOMain);
  end;

  FDOMain := ASP.DataObjectManager.CreateInstance(Connection, ADOClass);
  PKName := DOMain.PKName;
end;

function TibeFrmCommonDOList.GetConnection: TibeDALConnection;
begin
  Result := ASP.Connection;
end;

function TibeFrmCommonDOList.GetASP: TibeAppServiceProviderDB;
begin
	Result := TibeAppServiceProviderDB(inherited ASP);
end;

end.

