unit ibeFCommonDataList;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ActnList, StdCtrls, ExtCtrls, DB, VirtualTrees,
  Generics.Collections, System.Actions,
  ibeVirtualTreeview, ibeVirtualDataTreeview,
  ibeFCommonList,
  ibeVariant, ibeMemTable;

type

TibeDataListNodeData = record
  PKValue: Variant;
  CustomData: TObject;
end;
PibeDataListNodeData = ^TibeDataListNodeData;

  TibeFrmCommonDataList = class(TibeFrmCommonList)
    srcMain: TDataSource;
  private
    FDatMain: TibeMemTable;
    FPKName: string;
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
    procedure SetDatMain(ADat: TibeMemTable);
    function RetrieveNodeData(ANode: PVirtualNode): PibeDataListNodeData;
  protected
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
    function GetGrdMainClass(): TibeVirtualTreeviewClass; override;
    procedure InternalActionRefresh(); override;
    procedure InternalActionDisplay(); override;
    procedure DoAfterRefresh(); override;
    function AddDataColumn(const AFieldName: string; AWidth: Integer): TibeVirtualDataColumn; overload;
    function AddDataColumn(const AFieldName, ACaption: string; AWidth: Integer): TibeVirtualDataColumn; overload;
    procedure DoBeforeAddDataColumn(ADataColumn: TibeVirtualDataColumn; out AAddColumn: Boolean); virtual;
    property DatMain: TibeMemTable read FDatMain write SetDatMain;
    property PKName: string read FPKName write FPKName;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
  end;

implementation

{$R *.dfm}

// ====================== [ TibeFrmCommonDatasetList ] =========================

constructor TibeFrmCommonDataList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  grdMain.OnGetText := grdMainGetText;
  grdMain.OnFreeNode := grdMainFreeNode;
  grdMain.NodeDataSize := SizeOf(TibeDataListNodeData);

  FDatMain := TibeMemTable.Create(Self);
end;

destructor TibeFrmCommonDataList.Destroy();
begin
  inherited Destroy();
end;

procedure TibeFrmCommonDataList.InternalActionRefresh();
begin
  inherited InternalActionRefresh();

  if PKName = '' then
  begin
    raise Exception.Create(ClassName + '.InternalActionRefresh(): die Eigenschaft PKName ist nicht gesetzt, fortfahren nicht mglich');
  end;
end;

procedure TibeFrmCommonDataList.InternalActionDisplay();
var
  LNode: PVirtualNode;
  LData: PibeDataListNodeData;
begin
  inherited InternalActionDisplay();

  // fr das Fllen des Grids den Index entfernen, damit die Daten wie vom DO
  // sortiert sind
  DatMain.IndexFieldNames := '';
  try
    DatMain.First;
    while not DatMain.Eof do
    begin
      LNode := grdMain.AddChild(nil);
      LData := grdMain.GetNodeData(LNode);
      LData.PKValue := DatMain.FieldByName(PKName).Value;
      DatMain.Next();
    end;
  finally
    // fr den schnellen Zugriff auf die PK-Spalte den Index setzen
    DatMain.IndexFieldNames := PKName;
  end;
end;

procedure TibeFrmCommonDataList.DoAfterRefresh();
begin
  inherited DoAfterRefresh();

  TibeVirtualDataTreeview(grdMain).BindDataset(DatMain);
end;

function TibeFrmCommonDataList.AddDataColumn(const AFieldName: string; AWidth: Integer): TibeVirtualDataColumn;
begin
  Result := AddDataColumn(AFieldName, AFieldName, AWidth);
end;

function TibeFrmCommonDataList.AddDataColumn(const AFieldName, ACaption: string; AWidth: Integer): TibeVirtualDataColumn;
var
  LColumn: TibeVirtualDataColumn;
  LDoAddColumn: Boolean;
begin
  LColumn := TibeVirtualDataColumn(AddGridMainColumn(ACaption, AWidth));
  LColumn.FieldName := AFieldName;

  DoBeforeAddDataColumn(LColumn, LDoAddColumn);
  if not LDoAddColumn then
  begin
    FreeAndNil(LColumn);
  end;

  Result := LColumn;
end;

procedure TibeFrmCommonDataList.DoBeforeAddDataColumn(ADataColumn: TibeVirtualDataColumn; out AAddColumn: Boolean);
begin
  AAddColumn := True;
end;

function TibeFrmCommonDataList.GetPKValue(): Variant;
var
  LNodeData: PibeDataListNodeData;
begin
  LNodeData := RetrieveNodeData(grdMain.FocusedNode);
  
  if Assigned(LNodeData) then
  begin
    Result := LNodeData.PKValue;
  end else
  begin
    Result := Null;
  end;
end;

procedure TibeFrmCommonDataList.SetPKValue(APKValue: Variant);
var
  LFound: Boolean;
  LNode: PVirtualNode;
  LNodeData: PibeDataListNodeData;
  LOrigMoseCursor: TCursor;
begin
  LOrigMoseCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  try
    LFound := False;
    LNode := grdMain.GetFirst();

    if VarHasValue(APKValue) then
    begin
      while Assigned(LNode) and not LFound do
      begin
        LNodeData := RetrieveNodeData(LNode);
      
        if Assigned(LNodeData) then
        begin
          LFound := VarsAreEqual(APKValue, LNodeData.PKValue);
          if LFound then
          begin
            SelectNode(LNode);
          end;
        end;

        if not LFound then
        begin
          // wenn noch nicht gefunden, dann mit dem nchsten Node fortfahren
          LNode := grdMain.GetNext(LNode);
        end;
      end;
    end else
    begin
      // wird Null als PKValue bergeben, den derzeit selektierten Node deselektieren
      SelectNode(nil);
    end;
  finally
    Screen.Cursor := LOrigMoseCursor;
  end;
end;

function TibeFrmCommonDataList.RetrieveNodeData(ANode: PVirtualNode): PibeDataListNodeData;
begin
  Result := grdMain.GetNodeData(ANode);
end;

function TibeFrmCommonDataList.GetGrdMainClass: TibeVirtualTreeviewClass;
begin
  Result := TibeVirtualDataTreeview;
end;

procedure TibeFrmCommonDataList.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LData: PibeDataListNodeData;
begin
  if AColumn >= 0 then
  begin
    LData := grdMain.GetNodeData(ANode);
    if Assigned(LData) and DatMain.Active then
    begin
      if VarHasValue(LData.PKValue) then
      begin
        if DatMain.FindKey([LData.PKValue]) then
        begin
          ACellText := TibeVirtualDataColumn(grdMain.Header.Columns[AColumn]).DBFieldText;
        end;
      end;
    end;
  end;
end;

procedure TibeFrmCommonDataList.grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
var
  LNodeData: PibeDataListNodeData;
begin
  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      LNodeData.PKValue := Null;
    end;
  end;
end;

procedure TibeFrmCommonDataList.SetDatMain(ADat: TibeMemTable);
begin
  TibeVirtualDataTreeview(grdMain).BindDataset(nil);
  FreeAndNil(FDatMain);

  FDatMain := ADat;
  srcMain.DataSet := FDatMain;
end;

end.

