unit ibeVirtualDataTreeview;

interface

uses
  SysUtils, Classes, Variants, DB,
  VirtualTrees,
  ibeVirtualTreeview;

type

TibeVirtualDataColumn = class(TVirtualTreeColumn)
private
  FFieldName: string;
  FCaption: string;
  FAlignment: TAlignment;
  FWidth: Integer;
  FDBField: TField;
  function GetDBFieldText(): string;
  function GetDBFieldValue(): Variant;
protected
public
  procedure BindDBField(ADBFields: TFields);
  property FieldName: string read FFieldName write FFieldName;
  property DBField: TField read FDBField;
  property DBFieldText: string read GetDBFieldText;
  property DBFieldValue: Variant read GetDBFieldValue;
end;

TibeVirtualDataTreeview = class(TibeVirtualTreeview)
private
protected
  function GetColumnClass(): TVirtualTreeColumnClass; override;
public
  procedure BindDataset(ADataset: TDataSet);
end;

implementation

procedure TibeVirtualDataTreeview.BindDataset(ADataset: TDataSet);
var
  LIndex: TColumnIndex;
  LColumn: TibeVirtualDataColumn;
begin
  for LIndex := 0 to Header.Columns.Count - 1 do
  begin
    LColumn := TibeVirtualDataColumn(Header.Columns[LIndex]);
    
    if Assigned(ADataset) then
    begin
      LColumn.BindDBField(ADataset.Fields);
    end else
    begin
      LColumn.BindDBField(nil);
    end;
  end;
end;

function TibeVirtualDataTreeview.GetColumnClass(): TVirtualTreeColumnClass;
begin
  Result := TibeVirtualDataColumn;
end;

procedure TibeVirtualDataColumn.BindDBField(ADBFields: TFields);
begin
  if Assigned(ADBFields) then
  begin
    FDBField := ADBFields.FindField(FieldName);

    if Assigned(DBField) then
    begin
      if Text = FieldName then
      begin
        Text := DBField.DisplayLabel;
      end;
      Alignment := DBField.Alignment;
    end else
    begin
      raise Exception.Create('TDataColumn.BindDBField(): das Feld "' + FieldName + '" ist in der Datenmenge nicht enthalten');
    end
  end else
  begin
    FDBField := nil;
  end;
end;

function TibeVirtualDataColumn.GetDBFieldText(): string;
begin
  if Assigned(DBField) then
  begin
    Result := DBField.DisplayText;
  end else
  begin
    Result := '';
  end;
end;

function TibeVirtualDataColumn.GetDBFieldValue(): Variant;
begin
  if Assigned(DBField) then
  begin
    Result := DBField.Value;
  end else
  begin
    Result := Null;
  end;
end;

end.
