// ============================================================================
//
// Unit:    ibeAppServiceProviderDB
// Inhalt:  Formular
// Aufgabe: Klassenbibliothek - Bereitstellen Anwendungsweit verfgbarer
//          Informationen und Services fr Datenbankanwendungen
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeAppServiceProviderDB;

interface

uses
	SysUtils, Classes,
  ibeString, ibeFileUtil,
	ibeAppServiceProvider, 
  ibeDALConnectionManager, ibeDALConnection, ibeDataObjectManager;

type

TibeAppServiceProviderDB = class(TibeAppServiceProvider)
private
  FDataObjectManager: TibeDataObjectManager;
	FConnectionManager: TibeDALConnectionManager;
	FConnection: TibeDALConnection;
  FConnectionName: string;
	FOwner: TComponent;
  FDOCfgPath, FDOTraceFileName: TFileName;
  FDOTraceEnabled: Boolean;
protected
  procedure SetAppExeName(const AAppExeName: TFileName); override;
	procedure SetCfgPath(const path: TFileName); override;
public
	constructor Create; override;
	destructor Destroy; override;
	procedure InitMainConnection;
  property DataObjectManager: TibeDataObjectManager read FDataObjectManager;
	property ConnectionManager: TibeDALConnectionManager read FConnectionManager;
  property ConnectionName: string read FConnectionName write FConnectionName;
	property Connection: TibeDALConnection read FConnection;
	property Owner: TComponent read FOwner write FOwner;
  property DOCfgPath: TFileName read FDOCfgPath write FDOCfgPath;
  property DOTraceFileName: TFileName read FDOTraceFileName write FDOTraceFileName;
  property DOTraceEnabled: Boolean read FDOTraceEnabled write FDOTraceEnabled;
end;

function RetrieveASP: TibeAppServiceProviderDB;

implementation

uses
	ibeFuncCommon;

constructor TibeAppServiceProviderDB.Create;
begin
	inherited Create;

  FDataObjectManager := TibeDataObjectManager.Create();
	FConnectionManager := TibeDALConnectionManager.Create;
  DOTraceEnabled     := False;
end;

destructor TibeAppServiceProviderDB.Destroy;
begin
	inherited Destroy;

  FreeAndNil(FDataObjectManager);
	FreeAndNil(FConnectionManager);
  FreeAndNil(FConnection);
end;

procedure TibeAppServiceProviderDB.InitMainConnection;
begin
	FConnection := ConnectionManager.RetriveConnection(Owner, ConnectionName);
end;

procedure TibeAppServiceProviderDB.SetAppExeName(const AAppExeName: TFileName);
begin
  inherited SetAppExeName(AAppExeName);

  FDOTraceFileName := AddBackslash(GetTempPath()) + 'IBKE\DOTrace\';
  CreatePath(DOTraceFileName);
  FDOTraceFileName := FDOTraceFileName + AppName + '-Trace.sql';
end;

procedure TibeAppServiceProviderDB.SetCfgPath(const path: TFileName);
begin
  inherited SetCfgPath(path);

  FConnectionName := AppConfig.ReadString('App-Standard', 'ConnectionMain', 'Main');
  ConnectionManager.CfgFileName := AppConfig.FileName;
  DOCfgPath := AddBackslash(path) + 'DOD';
end;

function RetrieveASP: TibeAppServiceProviderDB;
begin
  Result := TibeAppServiceProviderDB(ibeAppServiceProvider.RetrieveASP)
end;

end.

