unit ibeModelAction;

interface

uses
  SysUtils,
  ibeDALConnection;

type

TibeModelAction = class
private
  FConnection: TibeDALConnection;
protected
  property Connection: TibeDALConnection read FConnection;
  procedure InternalExecute(); virtual;
  procedure DoBeforeExecute(); virtual;
  procedure DoAfterExecute(); virtual;
public
  constructor Create(AConnection: TibeDALConnection); virtual;
  procedure Execute();
end;

implementation

constructor TibeModelAction.Create(AConnection: TibeDALConnection);
begin
  inherited Create();

  FConnection := AConnection;
end;

procedure TibeModelAction.Execute();
begin
  Connection.TranStart();
  try
    DoBeforeExecute();

    InternalExecute();

    DoAfterExecute();

    Connection.TranCommit();
  except
    on E: Exception do
    begin
      Connection.TranRollback(E);

      raise;
    end;
  end;
end;

procedure TibeModelAction.InternalExecute();
begin
end;

procedure TibeModelAction.DoBeforeExecute();
begin
end;

procedure TibeModelAction.DoAfterExecute();
begin
end;

end.

