// ============================================================================
//
// Unit:    ibeString
// Inhalt:  Formular
// Aufgabe: Funktionsbibliothek
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeString;

interface

Uses
	SysUtils, Classes, Math, Windows,
	ibeCType, ibeTypes;

Function WordCount( Const s: String; Const WordDelims: TibeSetOfChars ): Integer;
Function WordPosition( Const n: Integer; Const s: String; Const WordDelims: TibeSetOfChars ): Integer;

Function ExtractWord( n: Integer; Const s: String; Const WordDelims: TibeSetOfChars ): String; Overload;
Function ExtractWord( n: Integer; Const s: String; Const WordDelims: TibeSetOfChars; Const Default: String ): String; Overload;
Function ExtractWordPos( n: Integer; Const s: String; Const WordDelims: TibeSetOfChars; Var Pos: Integer ): String;
Function ExtractDelimited( n: Integer; Const s: String; Const Delims: TibeSetOfChars ): String;

function ExtractSubstr( Const s: String; Var Pos: Integer; Const Delims: TibeSetOfChars ): String;

Function RemoveLastWord( Const str: String; Const Delims: TibeSetOfChars ): String;

{ Ersetzen von Teilstrings in einem String.																			}
{ Replace ersetzt das erste Vorkommen von SearchPattern durch RStr in str       }
{ ReplaceAll ersetzt alle Vorkommen von SearchPattern durch RStr in str         }
{ ReplaceStrings ersetzt alle SearchStrings durch NewStrings gleichen Indexes   }
Function ReplacePattern( Const str, SearchPattern, NewPattern: String): String;
Function ReplaceAllPattern( Const str, SearchPattern, NewPattern: String): String;
Function ReplaceStrings( Const str: String; Const SearchStrings, NewStrings: Array Of String ): String;

Function AddCharacterOpt(const s: WideString; c: WideChar): WideString;
Function AddBackslash(const FileName: TFileName): TFileName;
Function DelCharacterOpt(const s: WideString; c: WideChar): WideString;
Function DelBackslash( Const FileName: TFileName): TFileName;

{ Setzt s in Quotes. In s bereits vorhandene Quotes werden verdoppelt }
Function SetStringInQuotes( Const s: String; Quote: Char = '''' ): String;
Function RemoveQuotesInString( Const s: String; Quote: Char = '''' ): String;

{ TestAllowedChars() liefert True, wenn alle Zeichen von s in AllowedChars einthalten sind }
Function TestAllowedChars( Const s: String; AllowedChars: TibeSetOfChars ): Boolean;
{ TestOneAllowedCharExists() liefert True, sobald mindestens eines der AllowedChars in s enthalten ist }
Function TestOneAllowedCharExists( Const s: String; AllowedChars: TibeSetOfChars ): Boolean;

Function FilterChars( Const s: String; chr: Char ): String;
Function FilterString( Const s, Filter: String ): String;
Function FilterUnallowedChars( Const s: String; AllowedChars: TibeSetOfChars ): String;

Function S1DelimS2( Const s1, s2, delim: String ): String;
Function StrBoxing( Const s, Prefix, Postfix: String ): String;

{ CompareStartStr, CompareEndStr:                                               }
{ Vergleichen unter Bercksichtigung der Gro-/Kleinschreibung die ersten/letzen}
{ Maxlen Zeichen der beiden Strings.                                            }
{ Erbebnise:	< 0 => ASubText < AText                                           }
{							> 0 => ASubText > AText                                           }
{							= 0 => ASubText = AText                                           }
Function CompareStartStr( Const ASubText, AText: String; MaxLen: Integer ): Integer;
Function CompareEndStr( Const ASubText, AText: String; MaxLen: Integer ): Integer;

{ Wandelt i in String und fllt ggf. fhrende Nullen bis Digits auf }
Function IntToStr0( Value: Integer; Digits: Integer = 0 ): String;

{ Methoden fr TibeSetOfBytes }
Function SetOfBytes2String( bytes: TibeSetOfBytes; Delimeter: Char = ';' ): String;
Function String2SetOfBytes( Const bytes: String; Delimeter: Char = ';' ): TibeSetOfBytes;

{ **************************************************************************** }
{ *                                                                          * }
{ * Zerlegt einen String an ausgewhlten Zeichen in Teilstrings und fllt    * }
{ * damit eine Stringliste.                                                  * }
{ * Funktionsergebnis: Anzahl gefundener Teilstrings, immer >= 1.            * }
{ *                                                                          * }
{ **************************************************************************** }

Function SplitStringOnDelimChars( Const s: String; DelimChars: TibeSetOfChars; sl: TStringList ): Integer;

{ **************************************************************************** }
{ *                                                                          * }
{ * Umwandlung von normalisierten Strings in Datentypen und Datentypen in    * }
{ * normalisierte Stringreprsentation                                       * }
{ *                                                                          * }
{ **************************************************************************** }

Function nDateTimeToStr( dt: TDateTime ): String;
Function nDateToStr( dt: TDateTime ): String;
Function nTimeToStr( dt: TDateTime ): String;
Function nFloatToStr( f: ibeFloat ): String;
Function nBooleanToStr( b: Boolean ): String;

Function nStrToDateTime( dt: String ): TDateTime;
Function nStrToDate( dt: String ): TDateTime;
Function nStrToTime( dt: String ): TDateTime;
Function nStrToFloat( f: String ): ibeFloat;
Function nStrToBoolean( b: String ): Boolean;

Function IfThen( AValue: Boolean; Const ATrue, AFalse: WideChar ): WideChar; Overload;
Function IfThen( AValue: Boolean; Const ATrue, AFalse: WideString ): WideString; Overload;

Function RandomFromSetOfChars( ACharSet: TibeSetOfChars ): Char;
Function GenerateRandomString( Length: Integer; AllowedChars: TibeSetOfChars ): String;

function ibeRetrieveGUID: string;
function ibeRetrieveUID: string;

implementation

uses DateUtils, StrUtils;

Function WordCount( Const s: String; Const WordDelims: TibeSetOfChars ): Integer;
Var sLen, i: Cardinal;
Begin
	Result := 0;
	i := 1;
	sLen := Length(S);
	While i <= sLen Do Begin
		While (i <= sLen) And CharInSet(s[i], WordDelims) Do Inc(i);
		If i <= sLen Then Inc(Result);
		While (i <= sLen) And not CharInSet(s[i], WordDelims) Do Inc(i);
	End;
End;

Function WordPosition( Const n: Integer; Const s: String; Const WordDelims: TibeSetOfChars ): Integer;
Var Count, i: Integer;
begin
	Count := 0;
	i := 1;
	Result := 0;
	While (i <= Length(s)) And (Count <> n) Do Begin
	{ Delimiter berspringen }
		While (i <= Length(s)) And CharInSet(s[i], WordDelims) Do Inc(i);
		{ Wenn hier nicht das Stringende berschritten wurde, dann ist hier ein Wortanfang }
		If i <= Length(s) then Inc(Count);
		{ Wenn noch nicht das n-te Wort erreicht ist, auf das Ende des aktuellen Worts positionieren }
		If Count <> n Then While (i <= Length(s)) And Not CharInSet(s[i], WordDelims) Do Inc(i)
		Else							 Result := i;
	End;
End;

Function ExtractWord( n: Integer; Const s: String; Const WordDelims: TibeSetOfChars ): String;
Var i, Len: Integer;
Begin
	Len := 0;
	i := WordPosition( n, s, WordDelims);
	If i <> 0 Then Begin
	{ Bis zum Ende des aktuellen Worts laufen }
		While (i <= Length(s)) And not CharInSet(s[i], WordDelims) Do Begin
		{ Das n-te Zeichen an Result anhngen }
			Inc(Len);
			SetLength( Result, Len );
			Result[Len] := s[i];
			Inc(i);
		End;
	End;
	SetLength( Result, Len );
End;

Function ExtractWord( n: Integer; Const s: String; Const WordDelims: TibeSetOfChars; Const Default: String ): String; Overload;
Begin
	Result := ExtractWord( n, s, WordDelims );
	If Result = '' Then Result := Default;
End;

Function ExtractWordPos( n: Integer; Const S: String; Const WordDelims: TibeSetOfChars; Var Pos: Integer ): String;
Var i, Len: Integer;
Begin
	Len := 0;
	i := WordPosition( n, s, WordDelims );
	Pos := i;
	If i <> 0 Then Begin
	{ Bis zum Ende des aktuellen Worts laufen }
		While (i <= Length(s)) And not CharInSet(s[i], WordDelims) Do Begin
		{ Das n-te Zeichen an Result anhngen }
			Inc(Len);
			SetLength( Result, Len );
			Result[Len] := s[i];
			Inc(i);
		End;
	End;
	SetLength( Result, Len );
End;

Function ExtractDelimited( n: Integer; Const s: String; Const Delims: TibeSetOfChars ): String;
Var CurWord, i, Len, sLen: Integer;
Begin
	CurWord := 0;
	i := 1;
	Len := 0;
	sLen := Length(s);
	SetLength( Result, 0 );
	While (i <= sLen) And (CurWord <> n) Do Begin
		If CharInSet(s[i], Delims) Then Begin
			Inc(CurWord)
		End Else Begin
			If CurWord = n - 1 Then Begin
				Inc(Len);
				SetLength( Result, Len );
				Result[Len] := s[i];
			End;
		End;
		Inc(i);
	End;
End;

Function ExtractSubstr( Const s: String; Var Pos: Integer; Const Delims: TibeSetOfChars ): String;
Var i: Integer;
Begin
	i := Pos;
	While (i <= Length(s)) And Not CharInSet(s[i], Delims) Do Inc(i);
	Result := Copy( s, Pos, i - Pos );
	If (i <= Length(s)) And CharInSet(s[i], Delims) Then Inc(i);
	Pos := i;
End;

Function RemoveLastWord( Const str: String; Const Delims: TibeSetOfChars ): String;
Var delim, s: String;
	c: Char;
	LastPos: Integer;
Begin
	delim	:= '';
	s			:= str;

	For c := #0 To High( Char ) Do Begin
		If CharInSet(c, Delims) Then delim := delim + c;
	End;

	LastPos	:= LastDelimiter( delim, s );
	While (LastPos > 0) And (LastPos = Length( s )) Do Begin
	{ Der bergebene String endet mit einem der Delimiter-Zeichen. Jetzt den String }
	{ um ein Zeichen verkrzen und erneut nach dem letzten Vorkommen suchen }
		SetLength( s, Pred( LastPos ) );
		LastPos	:= LastDelimiter( delim, s );
	End;;

	If LastPos > 0 Then Result := Copy( s, 1, LastPos )
	Else								Result := s;
End;

Function ReplacePattern( Const str, SearchPattern, NewPattern: String): String;
Var i: Integer;
Begin
	i := Pos( SearchPattern, str );
	If i > 0 Then Begin
		Result := Copy( str, 1, i - 1 ) + NewPattern;
		Result := Result + Copy( str, i + Length(searchPattern), Length(str) );
	End Else Begin
		Result := str;
	End;
End;

Function ReplaceAllPattern( Const str, SearchPattern, NewPattern: String): String;
Begin
	Result := str;
	While Pos( SearchPattern, result ) > 0 Do Begin
		Result := ReplacePattern( Result, SearchPattern, NewPattern );
	End;
End;

Function ReplaceStrings( Const str: String; Const SearchStrings, NewStrings: Array Of String ): String;
Var i, Count: Integer;
Begin
	Result := str;
	Count := Min( High(SearchStrings), High(NewStrings) );
	For i := 0 To Count Do Begin
		Result := ReplaceAllPattern( Result, SearchStrings[i], NewStrings[i] );
	End;
End;

Function AddCharacterOpt(const s: WideString; c: WideChar): WideString;
Var len: Integer;
Begin
	Result := s;
	len := Length( Result );
	If (len > 0) And (Result[len] <> c) Then Begin
		Inc( len );
		SetLength( Result, len );
		Result[len] := c;
	End;
End;

Function AddBackslash(const FileName: TFileName): TFileName;
Begin
	Result := AddCharacterOpt(FileName, '\');
End;

Function DelCharacterOpt(const s: WideString; c: WideChar): WideString;
Var len: Integer;
Begin
	len := Length( s );
	If (len > 0) And (s[len] = c) Then Result :=  Copy( s, 1, Pred(len) )
	Else															 Result := s;
End;

Function DelBackslash(const FileName: TFileName): TFileName;
Begin
	Result := DelCharacterOpt(FileName, '\');
End;

Function SetStringInQuotes( Const s: String; Quote: Char = '''' ): String;
Begin
	Result := AnsiQuotedStr( s, Quote );
End;

Function RemoveQuotesInString( Const s: String; Quote: Char = '''' ): String;
Var s2: PChar;
Begin
	s2 := PChar(s);
	Result := AnsiExtractQuotedStr( s2, Quote );
End;

Function TestAllowedChars( Const s: String; AllowedChars: TibeSetOfChars ): Boolean;
Var i: Integer;
Begin
	Result := True;
	For i := 1 To Length(s) Do Begin
		Result	:= CharInSet(s[i], AllowedChars);
		If Not Result Then Break;
	End;
End;

Function TestOneAllowedCharExists( Const s: String; AllowedChars: TibeSetOfChars ): Boolean;
Var i: Integer;
Begin
	Result := False;
	For i := 1 To Length(s) Do Begin
		Result	:= CharInSet(s[i], AllowedChars);
		If Result Then Break;
	End;
End;

Function FilterChars( Const s: String; chr: Char ): String;
Var i: Integer;
Begin
	Result := s;
	For i := Length(Result) DownTo 1 Do Begin
		If Result[i] = chr Then Delete( Result, i, 1 );
	End;
End;

Function FilterString( Const s, Filter: String ): String;
Var i, len: Integer;
Begin
	len := Length(Filter);
	Result := s;
	For i := 1 To len Do Begin
		Result := FilterChars( Result, Filter[i] );
	End;
End;

Function FilterUnallowedChars( Const s: String; AllowedChars: TibeSetOfChars ): String;
Var i: Integer;
Begin
	Result := s;
	For i := Length(Result) DownTo 1 Do Begin
		If not CharInSet(Result[i], AllowedChars) Then Delete( Result, i, 1 );
	End;
End;

Function S1DelimS2( Const s1, s2, delim: String ): String;
Begin
	If			s1 = '' Then	Result := s2
	Else If	s2 = '' Then	Result := s1
	Else									Result := s1 + delim + s2;
End;

Function StrBoxing( Const s, Prefix, Postfix: String ): String;
Begin
	If Trim( s ) <> '' Then Result	:= Prefix + s + Postfix
	Else										Result	:= '';
End;

Function CompareStartStr( Const ASubText, AText: String; MaxLen: Integer ): Integer;
Var s: String;
Begin
	s				:= Copy( AText, 1, Min( MaxLen, Length( ASubText ) ) );
	Result	:= AnsiCompareStr( s, ASubText );
End;

Function CompareEndStr( Const ASubText, AText: String; MaxLen: Integer ): Integer;
Var s: String;
Begin
	s				:= RightStr( AText, Min( MaxLen, Length( ASubText ) ) );
	Result	:= AnsiCompareStr( s, ASubText );
End;

Function IntToStr0( Value: Integer; Digits: Integer = 0 ): String;
Begin
	Result := Format( '%*.*d', [Digits, Digits, Value] );
End;

Function SetOfBytes2String( bytes: TibeSetOfBytes; Delimeter: Char ): String;
Var b: Byte;
Begin
	Result := '';
	For b := Low(Byte) To High(Byte) Do Begin
		If b In Bytes Then Begin
			If Result <> '' Then Result := Result + Delimeter;
			Result := Result + IntToStr( b );
		End;
	End;
End;

Function String2SetOfBytes( Const bytes: String; Delimeter: Char ): TibeSetOfBytes;
Var i, wc: Integer;
	s: String;
Begin
	Result	:= [];
	wc			:= WordCount( bytes, [Delimeter] );
	For i := 1 To wc Do Begin
		s	:= ExtractWord( i, bytes, [Delimeter] );
		Result := Result + [StrToInt( s )];
	End;
End;

Function SplitStringOnDelimChars( Const s: String; DelimChars: TibeSetOfChars; sl: TStringList ): Integer;
Var i: Integer;
Begin
	sl.Clear;
	Result := WordCount( s, DelimChars );
	For i := 1 To Result Do Begin
		sl.Add( ExtractWord( i, s, DelimChars ) );
	End;
End;

Function nDateTimeToStr( dt: TDateTime ): String;
Begin
	Result := nDateToStr( dt ) + 'T' + nTimeToStr( dt );
End;

Function nDateToStr( dt: TDateTime ): String;
Begin
	Result := FormatDateTime( 'yyyy-mm-dd', dt );
End;

Function nTimeToStr( dt: TDateTime ): String;
Var h, m, s, ms: Word;
Begin
	DecodeTime( dt, h, m, s, ms );
	Result := Format( '%2.2d:%2.2d:%2.2d', [h, m, s] );
	If ms > 0 Then Result := Result + Format( '.%3.3d', [ms] );
End;

Function nFloatToStr( f: ibeFloat ): String;
Var OrigDecimalSeparator, OrigThousandSeparator: Char;
Begin
	OrigDecimalSeparator	:= FormatSettings.DecimalSeparator;
	OrigThousandSeparator	:= FormatSettings.ThousandSeparator;
	FormatSettings.ThousandSeparator			:= ' ';
	FormatSettings.DecimalSeparator			:= '.';

	Result :=	Format( '%.20e', [f] );

	FormatSettings.ThousandSeparator	:= OrigThousandSeparator;
	FormatSettings.DecimalSeparator	:= OrigDecimalSeparator;
End;

Function nBooleanToStr( b: Boolean ): String;
Begin
	If b Then	Result := 'true'
	Else			Result := 'false';
End;

Function nStrToDateTime( dt: String ): TDateTime;
Var s: String;
Begin
	Result := 0.0;
	s := ExtractWord( 1, dt, ['T', 't', ' '] );
	If s <> '' Then Result := Result + NStrToDate( s );

	s := ExtractWord( 2, dt, ['T', 't', ' '] );
	If s <> '' Then Result := Result + NStrToTime( s );
End;

Function nStrToDate( dt: String ): TDateTime;
Var d, m, y: Word;
Begin
	y := StrToIntDef( ExtractWord( 1, dt, ['-'] ), 0 );
	m := StrToIntDef( ExtractWord( 2, dt, ['-'] ), 0 );
	d := StrToIntDef( ExtractWord( 3, dt, ['-'] ), 0 );

	Result := EncodeDate( y, m, d );
End;

Function nStrToTime( dt: String ): TDateTime;
Var h, m, s, ms: Word;
	str: ibeWideString;
Begin
	str := ExtractWord( 1, dt, ['.'] );
	h := StrToIntDef( ExtractWord( 1, str, [':'] ), 0 );
	m := StrToIntDef( ExtractWord( 2, str, [':'] ), 0 );
	s := StrToIntDef( ExtractWord( 3, str, [':'] ), 0 );

	ms := StrToIntDef( ExtractWord( 2, dt, ['.'] ), 0 );

	Result := EncodeTime( h, m, s, ms );
End;

Function nStrToFloat( f: String ): ibeFloat;
Var OrigDecimalSeparator, OrigThousandSeparator: Char;
Begin
	OrigDecimalSeparator	:= FormatSettings.DecimalSeparator;
	OrigThousandSeparator	:= FormatSettings.ThousandSeparator;
	FormatSettings.ThousandSeparator			:= ' ';
	FormatSettings.DecimalSeparator			:= '.';

	Result :=	StrToFloat( f );

	FormatSettings.ThousandSeparator	:= OrigThousandSeparator;
	FormatSettings.DecimalSeparator	:= OrigDecimalSeparator;
End;

Function nStrToBoolean( b: String ): Boolean;
Begin
	If 			SameText( b, 'true' )  Or SameText( b, '1' ) Then Result := True
	Else If SameText( b, 'false' ) Or SameText( b, '0' ) Then Result := False
	Else Raise Exception.Create( 'Text ''' + b + ''' kann nicht als Boolean interpretiert werden.' );
End;

Function IfThen( AValue: Boolean; Const ATrue, AFalse: WideChar ): WideChar;
Begin
	If AValue Then Result := ATrue
	Else					 Result := AFalse;
End;

Function IfThen( AValue: Boolean; Const ATrue, AFalse: WideString ): WideString;
Begin
	If AValue Then Result := ATrue
	Else					 Result := AFalse;
End;

Function RandomFromSetOfChars( ACharSet: TibeSetOfChars ): Char;
Var Done: Boolean;
Begin
	Done		:= False;
	Repeat
		Result := Char(Random( 256 ));
		If CharInSet(Result, ACharSet) Then Done := True;
	Until Done;
End;

Function GenerateRandomString( Length: Integer; AllowedChars: TibeSetOfChars ): String;
Var i: Integer;
Begin
	Result := '';
	For i := 1 To Length Do Begin
		Result := Result + RandomFromSetOfChars( AllowedChars );
	End;
End;

function ibeRetrieveGUID: string;
var NewGUID: TGUID;
begin
	CreateGUID(NewGUID);
	Result := GUIDToString(NewGUID);
end;

function ibeRetrieveUID: string;
var NewGUID: TGUID;
begin
	CreateGUID(NewGUID);
	Result := GUIDToString(NewGUID);

	Result := FilterUnallowedChars(Result, ibeCharsXDigit);
end;

end.

