unit modMainCommon;

interface

uses
  SysUtils, Classes, System.ImageList, Vcl.ImgList, Vcl.Controls,
  ibeAppServiceProviderDB;

type
  TdmMainCommon = class(TDataModule)
    ilHot: TImageList;
    ilDisabled: TImageList;
    ilEnabled: TImageList;
  private
		FAutoConnect: Boolean;
		FASP: TibeAppServiceProviderDB;
  protected
    procedure DoBeforeOpenConnectionMain(); virtual;
		function InternalExecute(): Boolean; virtual;
    procedure RetrieveApplicationExeName(out AExeName: TFileName); virtual; abstract;
		property AutoConnect: Boolean read FAutoConnect write FAutoConnect;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
		function ActionStartup(): Boolean;
		property ASP: TibeAppServiceProviderDB read FASP;
  end;

implementation

uses
  ibeAppServiceProvider;

{$R *.dfm}

constructor TdmMainCommon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

	AutoConnect	:= True;
end;

destructor TdmMainCommon.Destroy(); 
begin
  inherited Destroy();

  FASP := nil;
  StoreASP(FASP);
end;

function TdmMainCommon.ActionStartup(): Boolean;
var 
  LOK: Boolean;
  LExeName: TFileName;
begin
	LOK := True;

	if LOK then begin
    RetrieveApplicationExeName(LExeName);

		FASP := TibeAppServiceProviderDB.Create();
    StoreASP(ASP);

    ASP.AppExeName := LExeName;
    ASP.Owner := Self;
	end;

	if LOK then begin
		ASP.InitMainConnection();
  end;

	if LOK and AutoConnect then begin
    DoBeforeOpenConnectionMain();
    
    ASP.Connection.Open();
	end;

	if LOK then begin
		LOK := InternalExecute();
	end;

	Result := LOK;
end;

procedure TdmMainCommon.DoBeforeOpenConnectionMain(); 
begin
end;

function TdmMainCommon.InternalExecute(): Boolean; 
begin
  Result := True;
end;

end.
