unit FDTBDODNavgator;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Menus, System.Actions,
  Vcl.ActnList, Vcl.StdCtrls, Vcl.ExtCtrls, System.ImageList, Vcl.ImgList, Data.DB,
  Vcl.ToolWin, Vcl.ComCtrls, Generics.Collections, Generics.Defaults, Math,
  XMLDoc, XMLIntf,
  VirtualTrees, ibeVirtualTreeview,
  ibeVariant, ibeAppServiceProviderDB, ibeFileUtil, ibeDataObjectDefinition,
  ibeFileHash, ibeOSProcedures, ibeString, ibeXMLDocument,
  libFCommonList, ibeFCommonEdit,
  FDTBDODViewFieldList;

type

TDODMitDateiname = class(TibeDataObjectDefinition)
private
  FFileName: TFileName;
  FSHA256Hash: string;
  FID: Integer;
  FDateAccessed: TDateTime;
  function GetDOClassName(): string;
protected
  constructor Create();
  procedure InitDefinition(const ADOClassName: string; const ADefinitionFileName: TFileName);
  procedure Touch();
  property DOClassName: string read GetDOClassName;
  property FileName: TFileName read FFileName write FFileName;
  property SHA256Hash: string read FSHA256Hash;
  Property ID: Integer read FID write FID;
  property DateAccessed: TDateTime read FDateAccessed write FDateAccessed;
end;

type

TDODList = class(TObjectList<TDODMitDateiname>);

  TfrmDTBDODNavgator = class(TlibFrmCommonList)
    spNavigatorWidth: TSplitter;
    pnlViewFieldHost: TPanel;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    dlgOpenDOD: TOpenDialog;
    pnlTop: TPanel;
    Label1: TLabel;
    txtDODPath: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure actNewExecute(Sender: TObject);
  private
    FLastNodeID: Integer;
    FFrmFiewFields: TfrmDTBDODViewFieldList;
    FDODList: TDODList;
    LSelectedNode: PVirtualNode;
    procedure AddDODToNavigator(const ADODName, AViewName: string; ADOD: TDODMitDateiname; AParentNode: PVirtualNode);
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure grdMainBeforeCellPaint(ASender: TBaseVirtualTree; ATargetCanvas: TCanvas; ANode: PVirtualNode; AColumn: TColumnIndex; ACellPaintMode: TVTCellPaintMode; ACellRect: TRect; var AContentRect: TRect);
    procedure grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
    procedure grdMainMouseUp(ASender: TObject; AButton: TMouseButton; AShift: TShiftState; AX, AY: Integer);
    procedure grdMainKeyDown(ASender: TObject; var AKey: Word; AShift: TShiftState);
    procedure LoadDODList(const ADODListFileName: TFileName);
    procedure SaveDODList(const ADODListFileName: TFileName);
    function RetrieveNextNodeID(): Integer;
    function RetrieveDODPath(): TFileName;
    function RetrieveDODListFileName(): TFileName;
  protected
    procedure SelectNode(ANode: PVirtualNode);
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionRefresh(); override;
    procedure InternalActionDisplay(); override;
    procedure InternalActionDelete(); override;
    function RetieveNodeData(ANode: PVirtualNode): Pointer;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
    function RetrieveEditClass(): TibeFrmCommonEditClass; override;
    property FrmFiewFields: TfrmDTBDODViewFieldList read FFrmFiewFields;
    property DODList: TDODList read FDODList;
  public
    procedure OpenDOD(const AFileName: TFileName);
  end;

implementation

uses
  modMain;

type

TDODNodeData = record
  Name: string;
  ViewName: string;
  DOD: TDODMitDateiname;
  function GetNodeID(): string;
end;
PDODNodeData = ^TDODNodeData;

{$R *.dfm}

procedure TfrmDTBDODNavgator.FormCreate(Sender: TObject);
var
  LDODListFileName: TFileName;
  LDODMitDateinameComparer: TComparison<TDODMitDateiname>;
begin
  inherited;
  grdMain.OnGetText := grdMainGetText;
  grdMain.OnBeforeCellPaint := grdMainBeforeCellPaint;
  grdMain.OnFreeNode := grdMainFreeNode;
  grdMain.OnKeyDown := grdMainKeyDown;
  grdMain.OnMouseUp := grdMainMouseUp;

  grdMain.NodeDataSize := SizeOf(TDODNodeData);
  grdMain.DisplayAsTree := True;

  FFrmFiewFields := TfrmDTBDODViewFieldList.Create(Self);
  FrmFiewFields.DisplayInContainer(pnlViewFieldHost);
  FrmFiewFields.Show();

  LDODMitDateinameComparer := function(const Left, Right: TDODMitDateiname): Integer
                              begin
                                Result := Sign(Right.DateAccessed - Left.DateAccessed);
                              end;

  FDODList := TDODList.Create(TComparer<TDODMitDateiname>.Construct(LDODMitDateinameComparer));

  FLastNodeID := -1;
  txtDODPath.Text := RetrieveDODPath();
  LDODListFileName := RetrieveDODListFileName();
  if FileExists(LDODListFileName) then
  begin
    LoadDODList(LDODListFileName);
    ActionRefresh();
    SelectNode(grdMain.GetFirst());
  end;
end;

procedure TfrmDTBDODNavgator.FormDestroy(Sender: TObject);
var
  LDODListFileName: TFileName;
begin
  inherited;

  LDODListFileName := RetrieveDODListFileName();
  SaveDODList(LDODListFileName);

  FreeAndNil(FDODList);
end;

procedure TfrmDTBDODNavgator.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();
  // alternative Darstellung der Tree-Buttons auswhlen
  grdMain.ButtonStyle := bsTriangle;

  AddGridMainColumn('Name', 200);
  AddGridMainColumn('View', 100);
  AddGridMainColumn('Access', 100);
end;

procedure TfrmDTBDODNavgator.InternalActionRefresh();
begin
  inherited InternalActionRefresh();
end;

procedure TfrmDTBDODNavgator.InternalActionDisplay();
var
  LDOD: TDODMitDateiname;
begin
  DODList.Sort();

  for LDOD in DODList do
  begin
    AddDODToNavigator(LDOD.DOClassName, '', LDOD, nil);
  end;
end;

procedure TfrmDTBDODNavgator.InternalActionDelete();
var
  i: Integer;
  LDOD: TDODMitDateiname;
  LData: PDODNodeData;
begin
  inherited InternalActionDelete();

  LData := RetieveNodeData(grdMain.FocusedNode);
  if Assigned(LData) then
  begin
    for LDOD in DODList do
    begin
      if SameText(LData.DOD.FileName, LDOD.FileName) then
      begin
        DODList.Remove(LDOD);
//        ActionRefresh();
        Break;
      end;
    end;
  end;
end;

procedure TfrmDTBDODNavgator.SelectNode(ANode: PVirtualNode);
var
  LData: PDODNodeData;
  LSHA256Hash: string;
  LSize: Cardinal;
begin
  Screen.Cursor := crHourGlass;
  try
    inherited SelectNode(ANode);

    LData := RetieveNodeData(ANode);
    if Assigned(LData) then
    begin
      if FileExists(LData.DOD.FileName) then
      begin
        LSHA256Hash := CalcFileDCPHash(LData.DOD.FileName, dcpSHA256, LSize);
        if not SameText(LSHA256Hash, LData.DOD.SHA256Hash) then
        begin
          // die DOD-Datei wurde verndert. Das DOD neu einlesen;
          LData.DOD.InitDefinition(LData.DOD.DOClassName, LData.DOD.FileName);
        end;

        if LData.ViewName = '' then
        begin
          LData.DOD.SelectActiveView(LData.DOD.MainViewName);
        end else
        begin
          LData.DOD.SelectActiveView(LData.ViewName);
        end;
        FrmFiewFields.DODView := LData.DOD.ActiveView;
        FrmFiewFields.ActionRefresh();

        LData.DOD.Touch();

        LSelectedNode := ANode;
        grdMain.Repaint();
      end else
      begin
        MessageDlg('Die Definitionsdatei "' + LData.DOD.FileName + '" existiert nicht mehr und kann daher nicht angezeigt werden. Der zugehrige Eintrag im Navigator wird gelscht.', mtInformation, [mbOK], 0);
        ActionDelete();
        // das zuletzt ausgewhlte DOD wieder zum aktuellen machen
        LData := LSelectedNode.GetData();
        PKValue := LData.GetNodeID();
      end;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmDTBDODNavgator.OpenDOD(const AFileName: TFileName);
var
  LDODID: Integer;
  LDOD: TDODMitDateiname;
  LFound: Boolean;
  LDODName: TFileName;
begin
  Screen.Cursor := crHourGlass;
  try
    LDODName := ExtractFileJustName(AFileName);
    LFound := False;

    for LDOD in DODList do
    begin
      if SameText(AFileName, LDOD.FileName) then
      begin
        LFound := True;
        PKValue := IntToStr(LDOD.ID) + '|';
        Break;
      end;
    end;

    if LFound then
    begin
    end else
    begin
      LDOD := TDODMitDateiname.Create();
      try
        LDOD.ID := RetrieveNextNodeID();
        LDOD.FileName := AFileName;
        //LDOD.InitDefinition(LDODName, AFileName);
        DODList.Add(LDOD);
      except
        FreeAndNil(LDOD);
        raise;
      end;

      ActionRefresh();

      PKValue := IntToStr(LDOD.ID) + '|';
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmDTBDODNavgator.actNewExecute(Sender: TObject);
begin
  inherited;
  dlgOpenDOD.InitialDir := RetrieveDODPath();
  dlgOpenDOD.FileName := '';

  if dlgOpenDOD.Execute then
  begin
    OpenDOD(dlgOpenDOD.FileName);
  end;
end;

procedure TfrmDTBDODNavgator.AddDODToNavigator(const ADODName, AViewName: string; ADOD: TDODMitDateiname; AParentNode: PVirtualNode);
var
  i: Integer;
  LMainViewName: string;
  LNode: PVirtualNode;
  LData: PDODNodeData;
begin
  LNode := grdMain.AddChild(AParentNode);
  LData := RetieveNodeData(LNode);

  LData.DOD := ADOD;
  LData.Name := ADODName;
  LData.ViewName := AViewName;

  if (AViewName = '') and (ADOD.ViewCount > 1) then
  begin
    LMainViewName := ADOD.ActiveView.Name;

    for i := 0 to ADOD.ViewCount - 1 do
    begin
      if not SameText(LMainViewName, ADOD.View[i].Name) then
      begin
        AddDODToNavigator(ADODName, ADOD.View[i].Name, ADOD, LNode);
      end;
    end;
  end;
end;

procedure TfrmDTBDODNavgator.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LData: PDODNodeData;
begin
  ACellText := '';

  LData := RetieveNodeData(ANode);

  case AColumn of
    0:  ACellText := LData.Name;
    1:  ACellText := LData.ViewName;
    2:  ACellText := DateTimeToStr(LData.DOD.DateAccessed);
  end;
end;

procedure TfrmDTBDODNavgator.grdMainBeforeCellPaint(ASender: TBaseVirtualTree; ATargetCanvas: TCanvas; ANode: PVirtualNode; AColumn: TColumnIndex; ACellPaintMode: TVTCellPaintMode; ACellRect: TRect; var AContentRect: TRect);
begin
  if ANode = LSelectedNode then
  begin
    ATargetCanvas.Brush.Color := grdMain.Colors.UnfocusedSelectionColor;

    ATargetCanvas.FillRect(ACellRect);
  end;
end;

procedure TfrmDTBDODNavgator.grdMainMouseUp(ASender: TObject; AButton: TMouseButton; AShift: TShiftState; AX, AY: Integer);
begin
  if AButton = mbLeft then
  begin
    SelectNode(grdMain.FocusedNode);
  end;
end;

procedure TfrmDTBDODNavgator.grdMainKeyDown(ASender: TObject; var AKey: Word; AShift: TShiftState);
begin
  if AKey = VK_RETURN then
  begin
    SelectNode(grdMain.FocusedNode);
    AKey := 0;
  end;
end;

procedure TfrmDTBDODNavgator.grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
var
  LData: PDODNodeData;
begin
  LData := RetieveNodeData(ANode);

  LData.Name := '';
  LData.ViewName := '';
end;

function TfrmDTBDODNavgator.RetieveNodeData(ANode: PVirtualNode): Pointer;
begin
  Result := grdMain.GetNodeData(ANode);
end;

procedure TfrmDTBDODNavgator.LoadDODList(const ADODListFileName: TFileName);
var
  LDODFileName: TFileName;
  LDateAccessed: TDateTime;
  LDOD: TDODMitDateiname;
  LXMLDoc: TibeXMLDocument;
  LXMLElement: TibeXMLElement;
  LXMLNode: IXMLNode;
begin
  LXMLDoc := TibeXMLDocument.Create();
  try
    LXMLDoc.LoadFromFile(ADODListFileName);

    LXMLElement := TibeXMLElement.Create();
    try
      LXMLNode := LXMLElement.GetNextChildNode('DODFile', LXMLDoc.DocumentElement);
      while Assigned(LXMLNode) do
      begin
        LDODFileName := LXMLElement.GetAttributeTypedValue('FileName', '', LXMLNode);

        if FileExists(LDODFileName) then
        begin
          LDateAccessed := LXMLElement.GetAttributeTypedValue('DateAccessed', Now(), LXMLNode);

          LDOD := TDODMitDateiname.Create();
          LDOD.FileName := LDODFileName;
          LDOD.DateAccessed := LDateAccessed;
          LDOD.ID := RetrieveNextNodeID();

          DODList.Add(LDOD);
        end;

        LXMLNode := LXMLElement.GetNextChildNode('DODFile', LXMLDoc.DocumentElement);
      end;
    finally
      FreeAndNil(LXMLElement);
    end;
  finally
    FreeAndNil(LXMLDoc);
  end;
end;

procedure TfrmDTBDODNavgator.SaveDODList(const ADODListFileName: TFileName);
var
  LDOD: TDODMitDateiname;
  LXMLDoc: TibeXMLDocument;
  LXMLElement: TibeXMLElement;
  LXMLNode: IXMLNode;
begin
  LXMLDoc := TibeXMLDocument.Create();
  try
    LXMLDoc.CreateDocument('DODFileList');

    LXMLElement := TibeXMLElement.Create();
    try
      for LDOD in DODList do
      begin
        LXMLNode := LXMLElement.CreateElement('DODFile', LXMLDoc.DocumentElement);

        LXMLElement.CreateAttribute('FileName', LDOD.FileName, LXMLNode);
        LXMLElement.CreateAttribute('DateAccessed', LDOD.DateAccessed, LXMLNode);
      end;
    finally
      FreeAndNil(LXMLElement);
    end;

    CreatePath(ExtractFilePath(ADODListFileName));
    LXMLDoc.SaveToFile(ADODListFileName);
  finally
    FreeAndNil(LXMLDoc);
  end;
end;

function TfrmDTBDODNavgator.RetrieveNextNodeID(): Integer;
begin
  Inc(FLastNodeID);

  Result := FLastNodeID;
end;

function TfrmDTBDODNavgator.GetPKValue(): Variant;
var
  LData: PDODNodeData;
begin
  LData := RetieveNodeData(grdMain.FocusedNode);

  if Assigned(LData) then
  begin
    Result := LData.GetNodeID();
  end;
end;

procedure TfrmDTBDODNavgator.SetPKValue(APKValue: Variant);
var
  LFound: Boolean;
  LNode: PVirtualNode;
  LData: PDODNodeData;
begin
  LFound := False;
  LNode := grdMain.GetFirst();
  while Assigned(LNode) and not LFound do
  begin
    LData := RetieveNodeData(LNode);
    if Assigned(LData) then
    begin
      LFound := SameText(APKValue, LData.GetNodeID());

      if LFound then
      begin
        SelectNode(LNode);
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

function TfrmDTBDODNavgator.RetrieveEditClass(): TibeFrmCommonEditClass;
begin
  Result := nil;
end;

function TfrmDTBDODNavgator.RetrieveDODPath(): TFileName;
begin
  Result := TibeAppServiceProviderDB(ASP).DOCfgPath;
end;

function TfrmDTBDODNavgator.RetrieveDODListFileName(): TFileName;
var
  LAppDataPath: TFileName;
  LDODLstPath: TFileName;
  LConnectionDescription: string;
begin
  LConnectionDescription := TibeAppServiceProviderDB(ASP).Connection.Parameters.Values['Description'];
  LConnectionDescription :=  GetValidFilename( Trim(LConnectionDescription));
  if LConnectionDescription <> '' then
  begin
    LAppDataPath := GetUserAppDataRoaming();
    LDODLstPath := AddBackslash(LAppDataPath) + 'IBKE\' + LConnectionDescription;

    Result := LDODLstPath +  '\DODRecentFileList.xml';
  end else
  begin
    Result := '';
  end;
end;

constructor TDODMitDateiname.Create();
begin
  inherited Create();

  Touch();
end;

procedure TDODMitDateiname.InitDefinition(const ADOClassName: string; const ADefinitionFileName: TFileName);
var
  LSize: Cardinal;
begin
  inherited InitDefinition(ADOClassName, ADefinitionFileName);

  FFileName := ADefinitionFileName;
  FSHA256Hash := CalcFileDCPHash(ADefinitionFileName, dcpSHA256, LSize);
end;

function TDODMitDateiname.GetDOClassName(): string;
begin
  if inherited DOClassName <> '' then
  begin
    Result := inherited DOClassName;
  end else
  begin
    Result := ExtractFileJustName(FileName);
  end;
end;

procedure TDODMitDateiname.Touch();
begin
  FDateAccessed := Now();
end;

function TDODNodeData.GetNodeID(): string;
begin
  Result := IntToStr(DOD.ID) + '|' + ViewName;
end;

end.

