unit FDTBDODViewFieldList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Menus, System.Actions,
  Vcl.ActnList, Vcl.StdCtrls, Vcl.ExtCtrls, System.ImageList, Vcl.ImgList, Data.DB,
  Vcl.ToolWin, Vcl.ComCtrls, Clipbrd,
  VirtualTrees,
  ibeOSProcedures,
  ibeDataObjectTypes, ibeDataObjectDefinition,
  libFCommonList;

type
  TfrmDTBDODViewFieldList = class(TlibFrmCommonList)
    tbSimpleCopy: TToolButton;
    mnuPopSimpleCopy: TPopupMenu;
    actCopyFieldNames: TAction;
    FeldnamenKopieren1: TMenuItem;
    mnuPopMainDelimCopy: TMenuItem;
    FeldnamenKopieren2: TMenuItem;
    actSelectAll: TAction;
    actSelectAllTblAlias: TAction;
    actSelectInvert: TAction;
    actSelectClear: TAction;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    Allesauswhlen1: TMenuItem;
    AllemitTabellenaliasauswhlen1: TMenuItem;
    Auswahlumkehren1: TMenuItem;
    Auswahlaufheben1: TMenuItem;
    N1: TMenuItem;
    actCodeGenerateFielsAssignment: TAction;
    CodefrFeldwertZuweisung1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure actCopyFieldNamesExecute(Sender: TObject);
    procedure actSelectAllExecute(Sender: TObject);
    procedure actSelectAllTblAliasExecute(Sender: TObject);
    procedure actSelectInvertExecute(Sender: TObject);
    procedure actSelectClearExecute(Sender: TObject);
    procedure actCodeGenerateFielsAssignmentExecute(Sender: TObject);
    procedure tbSimpleCopyClick(Sender: TObject);
  private
    FDODView: TibeDODefView;
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure RetrieveMarkedFieldNames(AFieldNames: TStrings);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionDisplay(); override;
    function RetieveNodeData(ANode: PVirtualNode): Pointer;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
  public
    property DODView: TibeDODefView read FDODView write FDODView;
  end;

implementation

uses
  modMain,
  FDTBGenerateCodeAssignFieldsValues;

type

TNodeData = record
  ViewField: TibeDODefViewField;
end;
PNodeData = ^TNodeData;

{$R *.dfm}

procedure TfrmDTBDODViewFieldList.actSelectAllExecute(Sender: TObject);
begin
  inherited;
  grdMain.SelectAll(False);
end;

procedure TfrmDTBDODViewFieldList.actSelectAllTblAliasExecute(Sender: TObject);
var
  LTblAlias: string;
  LDoAppend: Boolean;
  LNode: PVirtualNode;
  LData: PNodeData;
begin
  inherited;
  LDoAppend := QueryKeyPressedStatus(kcShift);
  LData := grdMain.FocusedNode.GetData();
  LTblAlias := LData.ViewField.TblAlias;

  LNode := grdMain.GetFirst();
  while Assigned(LNode) do
  begin
    LData := LNode.GetData();
    grdMain.Selected[LNode] := SameText(LTblAlias, LData.ViewField.TblAlias) or (LDoAppend and grdMain.Selected[LNode]);
    LNode := grdMain.GetNext(LNode);
  end;
end;

procedure TfrmDTBDODViewFieldList.actSelectClearExecute(Sender: TObject);
var
  LNode: PVirtualNode;
begin
  inherited;
  LNode := grdMain.GetFirst();
  while Assigned(LNode) do
  begin
    grdMain.Selected[LNode] := False;
    LNode := grdMain.GetNext(LNode);
  end;
end;

procedure TfrmDTBDODViewFieldList.actSelectInvertExecute(Sender: TObject);
var
  LNode: PVirtualNode;
begin
  inherited;
  LNode := grdMain.GetFirst();
  while Assigned(LNode) do
  begin
    grdMain.Selected[LNode] := not grdMain.Selected[LNode];
    LNode := grdMain.GetNext(LNode);
  end;
end;

procedure TfrmDTBDODViewFieldList.FormCreate(Sender: TObject);
begin
  inherited;
  grdMain.OnGetText := grdMainGetText;
  grdMain.NodeDataSize := SizeOf(TNodeData);
  grdMain.TreeOptions.SelectionOptions := grdMain.TreeOptions.SelectionOptions + [toMultiSelect];
end;

procedure TfrmDTBDODViewFieldList.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();

  AddGridMainColumn('Feldname', 200);
  AddGridMainColumn('Datentyp', 100);
  AddGridMainColumn('Datentyp (Tabelle)', 100);
  AddGridMainColumn('SQL Ausdruck', 100);
  AddGridMainColumn('Beschriftung', 200);
end;

procedure TfrmDTBDODViewFieldList.InternalActionDisplay();
var
  i: Integer;
  LNode: PVirtualNode;
  LData: PNodeData;
begin
  inherited InternalActionDisplay();

  for i := 0 to DODView.FieldCount - 1 do
  begin
    LNode := grdMain.AddChild(nil);
    LData := grdMain.GetNodeData(LNode);
    LData.ViewField := DODView.Field[i];
  end;
end;

procedure TfrmDTBDODViewFieldList.actCopyFieldNamesExecute(Sender: TObject);
var
  LStrList: TStringList;
begin
  inherited;
  LStrList := TStringList.Create();
  try
    RetrieveMarkedFieldNames(LStrList);

    if LStrList.Count = 1 then
    begin
      // wenn nur ein Feldname kopiert wird, dann diesen ohne abschlieende
      // Leerzeile in die Zwischenablage legen
      Clipboard.AsText := LStrList[0];
    end else
    begin
      Clipboard.AsText := LStrList.Text;
    end;
  finally
    FreeAndNil(LStrList);
  end;
end;

procedure TfrmDTBDODViewFieldList.actCodeGenerateFielsAssignmentExecute(Sender: TObject);
var
  LFieldName: string;
  LFieldNames: TStringList;
  LFrmGenerateCode: TfrmDTBGenerateCodeAssignFieldsValues;
begin
  inherited;
  LFieldNames := TStringList.Create();
  try
    RetrieveMarkedFieldNames(LFieldNames);

    LFrmGenerateCode := TfrmDTBGenerateCodeAssignFieldsValues.Create(Self);
    try
      for LFieldName in LFieldNames do
      begin
        LFrmGenerateCode.AddFieldName(LFieldName);
      end;
      LFrmGenerateCode.ShowModal();
    finally
      FreeAndNil(LFrmGenerateCode);
    end;
  finally
    FreeAndNil(LFieldNames);
  end;
end;

procedure TfrmDTBDODViewFieldList.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LData: PNodeData;
begin
  LData := ASender.GetNodeData(ANode);

  if Assigned(LData) then
  begin
    case AColumn of
      0: ACellText := LData.ViewField.FieldName;
      1: ACellText := DODefDataTypeIDArray[LData.ViewField.DataType];
      2: ACellText := DODefDataTypeIDArray[LData.ViewField.TblDataType];
      3: ACellText := LData.ViewField.SQLExpressionText;
      4: ACellText := LData.ViewField.Caption;
    end;
  end;
end;

procedure TfrmDTBDODViewFieldList.RetrieveMarkedFieldNames(AFieldNames: TStrings);
var
  LNode: PVirtualNode;
  LData: PNodeData;
begin
  LNode := grdMain.GetFirst();
  while Assigned(LNode) do
  begin
    if grdMain.Selected[LNode] then
    begin
      LData := grdMain.GetNodeData(LNode);
      AFieldNames.Add(LData.ViewField.FieldName);
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

function TfrmDTBDODViewFieldList.RetieveNodeData(ANode: PVirtualNode): Pointer;
begin
  Result := grdMain.GetNodeData(ANode);
end;

function TfrmDTBDODViewFieldList.GetPKValue(): Variant;
var
  LData: PNodeData;
begin
  LData := grdMain.GetNodeData(grdMain.FocusedNode);

  if Assigned(LData) and Assigned(LData.ViewField) then
  begin
    Result := LData.ViewField.FieldName;
  end;
end;

procedure TfrmDTBDODViewFieldList.SetPKValue(APKValue: Variant);
begin
end;

procedure TfrmDTBDODViewFieldList.tbSimpleCopyClick(Sender: TObject);
var
  LPopupPos: TPoint;
begin
  inherited;
  LPopupPos.X := tbSimpleCopy.Left;
  LPopupPos.Y := tbSimpleCopy.Height;
  LPopupPos := tbMain.ClientToScreen(LPopupPos);

  tbSimpleCopy.DropdownMenu.Popup(LPopupPos.X, LPopupPos.Y);
end;

end.

