unit FDTBGenerateCodeAssignFieldsValues;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, ibeFCommon, System.Actions,
  Vcl.ActnList, Vcl.ExtCtrls, Vcl.StdCtrls, clipbrd;

type
  TfrmDTBGenerateCodeAssignFieldsValues = class(TfrmCommon)
    txtCode: TMemo;
    pnlTop: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    txtDODestName: TEdit;
    Label3: TLabel;
    txtDOSourceName: TEdit;
    cmdCopyCode: TButton;
    actCopyCode: TAction;
    actCloseForm: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure actCopyCodeExecute(Sender: TObject);
    procedure actCloseFormExecute(Sender: TObject);
  private
    FFieldNames: TStringList;
    procedure txtDONameChange(Sender: TObject);
    procedure GenerateCode(const ADODestName, ADOSrcName: string; ACodeLines: TStrings);
  public
    procedure AddFieldName(const AFieldName: string);
  end;

implementation

{$R *.dfm}

procedure TfrmDTBGenerateCodeAssignFieldsValues.FormCreate(Sender: TObject);
begin
  inherited;
  FFieldNames := TStringList.Create();

  txtDODestName.Text := 'LdoDest';
  txtDOSourceName.Text := 'LdoSrc';

  txtDODestName.OnChange := txtDONameChange;
  txtDOSourceName.OnChange := txtDONameChange;
end;

procedure TfrmDTBGenerateCodeAssignFieldsValues.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FFieldNames);
end;

procedure TfrmDTBGenerateCodeAssignFieldsValues.FormShow(Sender: TObject);
begin
  inherited;

  GenerateCode(txtDODestName.Text, txtDOSourceName.Text, txtCode.Lines);
end;

procedure TfrmDTBGenerateCodeAssignFieldsValues.GenerateCode(const ADODestName, ADOSrcName: string; ACodeLines: TStrings);
  // Beginn lokale Funktionen
  function BuildFieldCode(const ADOName, AFieldName: string): string;
  var
    LFieldCode: string;
  begin
    LFieldCode := '';

    if ADOName <> '' then
    begin
      LFieldCode := LFieldCode + ADOName + '.';
    end;
    LFieldCode := LFieldCode + 'Field[''' + AFieldName + '''].Value';

    Result := LFieldCode;
  end;
  // Ende lokale Funktionen
var
  LFieldName: string;
  LDestFieldCode, LSrcFieldCode: string;
  LRow: string;
begin
  ACodeLines.Clear();

  for LFieldName in FFieldNames do
  begin
    LDestFieldCode := BuildFieldCode(ADODestName, LFieldName);
    LSrcFieldCode := BuildFieldCode(ADOSrcName, LFieldName);
    LRow := Format('  %s := %s;', [LDestFieldCode, LSrcFieldCode]);

    ACodeLines.Add(LRow)
  end;
end;

procedure TfrmDTBGenerateCodeAssignFieldsValues.actCloseFormExecute(
  Sender: TObject);
begin
  inherited;
  Close();
end;

procedure TfrmDTBGenerateCodeAssignFieldsValues.actCopyCodeExecute(Sender: TObject);
begin
  inherited;
  Clipboard.AsText := txtCode.Text;
end;

procedure TfrmDTBGenerateCodeAssignFieldsValues.AddFieldName(const AFieldName: string);
begin
  FFieldNames.Add(AFieldName);
end;

procedure TfrmDTBGenerateCodeAssignFieldsValues.txtDONameChange(
  Sender: TObject);
begin
  inherited;
  GenerateCode(txtDODestName.Text, txtDOSourceName.Text, txtCode.Lines);
end;

end.

