// ============================================================================
//
// Unit:    DODLibEditClasses
// Inhalt:  Klassendefinitionen fr die Verwendungn in den Bearbeitungsformularen
//          vom DataObjectDefinition Editor
// Aufgabe: Funktionsbibliothek
//
//   (c) by Klaus Etscheid since 2017
// ============================================================================
unit DODLibEditClasses;

interface

uses
  System.SysUtils, System.Classes, Generics.Collections,
  ibeFuncCommon,
  ibeDataObjectTypes, ibeDataObjectDefinition;

type

TDODObserverSenderOfEvent = (dosDOD, dosTable, dosTableField, dosView, dosViewField);
TDODObserverTypeOfEvent = (doeBeginBatchUpdate, doeEndBatchUpdate, doeObjectAdd, doeObjectUpdate, doeObjectDelete);

TDODObserverEvent = procedure(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer) of object;

TDODObserver = class
private
  FSender: TDODObserverSenderOfEvent;
  FEvent: TDODObserverTypeOfEvent;
  FObjectID: Integer;
  FNotifyProc: TDODObserverEvent;
protected
public
  constructor Create(ASender: TDODObserverSenderOfEvent; AEvent: TDODObserverTypeOfEvent; AObjectID: Integer; ANotifyProc: TDODObserverEvent);
  property Sender: TDODObserverSenderOfEvent read FSender;
  property Event: TDODObserverTypeOfEvent read FEvent;
  property ObjectID: Integer read FObjectID;
  property NotifyProc: TDODObserverEvent read FNotifyProc;
end;

TDODObserverList = class(TObjectList<TDODObserver>)
private
protected
public
end;

type

// Liefert eine Basisklasse fr die Verwendung in den Editor-Formularen.
// In den Ableitungen macht ein Property mit Namen "Data" ein Objekt vom Typ der
// bearbeiteten DOD-Klasse die Daten aus dem DOD-Objekt public
TDODLibEditCommon = class
private
  FID: Integer;
  FOnNotifyObserver: TDODObserverEvent;
protected
  procedure NotifyObserver(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent);
public
  constructor Create();
  procedure Assign(ADODEditObject: TDODLibEditCommon);
  property ID: Integer read FID write FID;
  property OnNotifyObserver: TDODObserverEvent read FOnNotifyObserver write FOnNotifyObserver;
end;

TDODLibEditCommonList<T: TDODLibEditCommon> = class(TObjectList<T>)
private
  FOnNotifyObserver: TDODObserverEvent;
  function GetIndexByID(AID: Integer): Integer;
  function GetItemByID(AID: Integer): TDODLibEditCommon;
protected
  procedure NotifyObserver(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
public
  constructor Create(AOwnsObjects: Boolean = True);
  destructor Destroy(); override;
  function Add(AItem: TDODLibEditCommon): Integer;
  procedure Delete(Index: Integer);
  property IndexByID[AID: Integer]: Integer read GetIndexByID;
  property ItemByID[AID: Integer]: TDODLibEditCommon read GetItemByID;
  property OnNotifyObserver: TDODObserverEvent read FOnNotifyObserver write FOnNotifyObserver;
end;

TDODLibEditTable = class(TDODLibEditCommon)
private
  FData: TibeDODefTable;
protected
public
  constructor Create();
  destructor Destroy(); override;
  procedure Assign(ADODEditObject: TDODLibEditCommon);
  property Data: TibeDODefTable read FData;
end;

TDODLibEditTableList = class(TDODLibEditCommonList<TDODLibEditTable>);

TDODLibEditTableField = class(TDODLibEditCommon)
private
  FData: TibeDODefTableField;
protected
public
  constructor Create();
  destructor Destroy(); override;
  procedure Assign(ADODEditObject: TDODLibEditCommon);
  property Data: TibeDODefTableField read FData;
end;

TDODLibEditTableFieldList = class(TDODLibEditCommonList<TDODLibEditTableField>)
private
protected
public
  function FindFieldByTableAlias(const ATableAlias, AFieldName: string): TDODLibEditTableField;
  procedure AssignToDODTableFieldList(ADODTableFieldList: TibeDODefTableFieldList);
end;

TDODLibEditViewField = class(TDODLibEditCommon)
private
  FData: TibeDODefViewField;
protected
public
  constructor Create();
  destructor Destroy(); override;
  procedure Assign(ADODViewField: TDODLibEditViewField; ATblFieldList: TibeDODefTableFieldList);
  property Data: TibeDODefViewField read FData;
end;

TDODLibEditViewFieldList = class(TDODLibEditCommonList<TDODLibEditViewField>)
private
protected
public
  procedure AssignToDODViewFieldList(ADODViewFieldList: TibeDODefViewFieldList);
end;

TDODLibEditViewOrderByField = class(TDODLibEditCommon)
private
  FData: TibeDODefViewOrderByField;
protected
public
  constructor Create();
  destructor Destroy(); override;
  procedure Assign(ADODViewOrderByField: TDODLibEditViewOrderByField);
  property Data: TibeDODefViewOrderByField read FData;
end;

TDODLibEditViewOrderByFieldList = class(TDODLibEditCommonList<TDODLibEditViewField>)
private
protected
public
end;

TDODLibEditViewGroupByField = class(TDODLibEditCommon)
private
  FData: TibeDODefViewGroupByField;
protected
public
  constructor Create();
  destructor Destroy(); override;
  procedure Assign(ADODViewGroupByField: TDODLibEditViewGroupByField);
  property Data: TibeDODefViewGroupByField read FData;
end;

TDODLibEditViewGroupByFieldList = class(TDODLibEditCommonList<TDODLibEditViewGroupByField>)
private
protected
public
end;

TDODLibEditView = class(TDODLibEditCommon)
private
  FFields: TDODLibEditViewFieldList;
  FOrderByFields: TDODLibEditViewOrderByFieldList;
  FGroupByFields: TDODLibEditViewGroupByFieldList;
  FName, FDescription, FViewPKName: string;
protected
public
  constructor Create();
  destructor Destroy(); override;
  procedure Assign(ADODView: TDODLibEditView; ATblFieldList: TibeDODefTableFieldList); overload;
  procedure Assign(ADOView: TibeDODefView; ATblFieldList: TibeDODefTableFieldList); overload;
  procedure AssignToActiveView(ADOD: TibeDataObjectDefinition);
  property Name: string read FName write FName;
  property Description: string read FDescription write FDescription;
  property ViewPKName: string read FViewPKName write FViewPKName;
  property Fields: TDODLibEditViewFieldList read FFields;
  property OrderByFields: TDODLibEditViewOrderByFieldList read FOrderByFields;
  property GroupByFields: TDODLibEditViewGroupByFieldList read FGroupByFields;
end;

TDODLibEditViewList = class(TDODLibEditCommonList<TDODLibEditView>)
private
protected
public
end;

TDODLibDataObjectDefinition = class
private
  FDODFileName: TFileName;
  FMainTableAlias, FMainTablePKName: string;
  FMainTableKKValueMode: TibeDOPKValueMode;
  FMainTablePKSequenceName: string;
  FMainViewID: Integer;
  FTables: TDODLibEditTableList;
  FTableFields: TDODLibEditTableFieldList;
  FViews: TDODLibEditViewList;
  FObservers: TDODObserverList;
  function GetMainViewName(): string;
protected
public
  constructor Create();
  destructor Destroy(); override;
  procedure New();
  procedure Clear();
  procedure LoadFromDODFile(const ADODFileName: TFileName);
  procedure LoadFromDODStream(AStream: TStream);
  procedure SaveToDODFile(); overload;
  procedure SaveToDODFile(const ADODFileName: TFileName); overload;
  procedure SaveToDODStream(AStream: TStream);
  procedure RegisterObserver(ASender: TDODObserverSenderOfEvent; AEvent: TDODObserverTypeOfEvent; AObjectID: Integer; ANotifyProc: TDODObserverEvent);
  procedure UnRegisterObserver(ANotifyProc: TDODObserverEvent); overload;
  procedure UnRegisterObserver(ASender: TDODObserverSenderOfEvent; AEvent: TDODObserverTypeOfEvent; AObjectID: Integer); overload;
  procedure NotifyObservers(ASender: TDODObserverSenderOfEvent; AEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
  property Observers: TDODObserverList read FObservers;
  property DODFileName: TFileName read FDODFileName;
  property MainTableAlias: string read FMainTableAlias write FMainTableAlias;
  property MainTablePKName: string read FMainTablePKName write FMainTablePKName;
  property MainTableKKValueMode: TibeDOPKValueMode read FMainTableKKValueMode write FMainTableKKValueMode;
  property MainTablePKSequenceName: string read FMainTablePKSequenceName write FMainTablePKSequenceName;
  property MainViewID: Integer read FMainViewID write FMainViewID;
  property MainViewName: string read GetMainViewName;
  property Tables: TDODLibEditTableList read FTables;
  property TableFields: TDODLibEditTableFieldList read FTableFields;
  property Views: TDODLibEditViewList read FViews;
end;

implementation

var M_NextID: Integer = 0;

function GenerateNextID(): Integer;
begin
  Result := M_NextID;
  Inc(M_NextID);
end;

constructor TDODLibDataObjectDefinition.Create();
begin
  inherited Create();

  FTables := TDODLibEditTableList.Create();
  FTableFields := TDODLibEditTableFieldList.Create();
  FViews := TDODLibEditViewList.Create();
  FObservers := TDODObserverList.Create();

  Tables.OnNotifyObserver := NotifyObservers;
  TableFields.OnNotifyObserver := NotifyObservers;
  Views.OnNotifyObserver := NotifyObservers;
end;

destructor TDODLibDataObjectDefinition.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FObservers);
  FreeAndNil(FViews);
  FreeAndNil(FTableFields);
  FreeAndNil(FTables);
end;

procedure TDODLibDataObjectDefinition.New();
var
  LView: TDODLibEditView;
begin
  Clear();

  LView := TDODLibEditView.Create();
  LView.Name := 'Main';
  LView.Description := 'Standardansicht';

  Views.Add(LView);

  MainViewID := LView.ID;
end;

procedure TDODLibDataObjectDefinition.Clear();
begin
  FDODFileName := '';

  MainTableAlias := '';
  MainTablePKName := '';
  MainTableKKValueMode := pkmNone;
  MainTablePKSequenceName := '';

  Views.Clear();
  TableFields.Clear();
  Tables.Clear();
end;

procedure TDODLibDataObjectDefinition.LoadFromDODFile(const ADODFileName: TFileName);
var
  LStream: TFileStream;
begin
  LStream := TFileStream.Create(ADODFileName, fmOpenRead);
  try
    LoadFromDODStream(LStream);

    // abschlieend den Dateinamen merken
    FDODFileName := ADODFileName;
  finally
    FreeAndNil(LStream);
  end;
end;

procedure TDODLibDataObjectDefinition.LoadFromDODStream(AStream: TStream);
var
  i: Integer;
  LDOD: TibeDataObjectDefinition;
  LTblFieldList: TibeDODefTableFieldList;
  LTable: TDODLibEditTable;
  LTableField: TDODLibEditTableField;
  LView: TDODLibEditView;
begin
  Clear();

  LDOD := TibeDataObjectDefinition.Create();
  try
    // die DOD-Datei laden
    LDOD.InitDefinition('Generic-DOD', AStream);

    MainViewID := -1;

    FMainTableAlias := LDOD.MainTableAlias;
    FMainTablePKName := LDOD.MainTablePKName;
    FMainTableKKValueMode := LDOD.MainTableKKValueMode;
    FMainTablePKSequenceName := LDOD.MainTablePKSequenceName;

    // die Tabellen aus dem DOD einlesen
    for i := 0 to LDOD.TableCount - 1 do
    begin
      LTable := TDODLibEditTable.Create();
      try
        LTable.Data.Assign(LDOD.Table[i]);
        Tables.Add(LTable);
      except
        // im Ausnahmefall das Tabellenobjekt freigeben und die Exception erneut auslsen
        FreeAndNil(LTable);
        raise;
      end;
    end;

    // die Tabellenfelder aus dem DOD einlesen
    for i := 0 to LDOD.FieldCount - 1 do
    begin
      LTableField := TDODLibEditTableField.Create();
      try
        LTableField.Data.Assign(LDOD.Field[i]);
        TableFields.Add(LTableField);
      except
        // im Ausnahmefall das Feldobjekt freigeben und die Exception erneut auslsen
        FreeAndNil(LTableField);
        raise;
      end;
    end;

    LTblFieldList := TibeDODefTableFieldList.Create(False);
    try
      TableFields.AssignToDODTableFieldList(LTblFieldList);
      for i := 0 to LDOD.ViewCount - 1 do
      begin
        LView := TDODLibEditView.Create();
        try
          LView.Assign(LDOD.View[i], LTblFieldList);
          Views.Add(LView);

          if SameText(LDOD.ActiveView.Name, LView.Name) then
          begin
            MainViewID := LView.ID;
          end;
        except
          FreeAndNil(LView);
          raise;
        end;
      end;
    finally
      FreeAndNil(LTblFieldList);
    end;
  finally
    FreeAndNil(LDOD);
  end;
end;

procedure TDODLibDataObjectDefinition.SaveToDODFile();
begin
  SaveToDODFile(DODFileName);
end;

procedure TDODLibDataObjectDefinition.SaveToDODFile(const ADODFileName: TFileName);
var
  LOpenMode: Integer;
  LStream: TFileStream;
begin
  if FileExists(ADODFileName) then
  begin
    LOpenMode := fmOpenWrite;
  end else
  begin
    LOpenMode := fmCreate;
  end;

  LStream := TFileStream.Create(ADODFileName, LOpenMode or fmExclusive);
  try
    SaveToDODStream(LStream);

    // abschlieend den Dateinamen merken
    FDODFileName := ADODFileName;
  finally
    FreeAndNil(LStream);
  end;
end;

procedure TDODLibDataObjectDefinition.SaveToDODStream(AStream: TStream);
var
  i, LIndex: Integer;
  LDOD: TibeDataObjectDefinition;
begin
  LDOD := TibeDataObjectDefinition.Create();
  try
    LDOD.MainTableAlias := MainTableAlias;
    LDOD.MainTablePKName := MainTablePKName;
    LDOD.MainTableKKValueMode := MainTableKKValueMode;
    LDOD.MainTablePKSequenceName := MainTablePKSequenceName;
    LDOD.MainViewName := MainViewName;

    // Tabellendefinitionen in das DOD bertragen
    for i := 0 to Tables.Count - 1 do
    begin
      LIndex := LDOD.AddTable();
      LDOD.Table[LIndex].Assign(TDODLibEditTable(Tables[i]).Data);
    end;

    // Tabellenfelder in das DOD bertragen
    for i := 0 to TableFields.Count - 1 do
    begin
      LIndex := LDOD.AddField();
      LDOD.Field[LIndex].Assign(TDODLibEditTableField(TableFields[i]).Data);
    end;

    // Viewdefinitionen in das DOD bertragen
    for i := 0 to Views.Count - 1 do
    begin
      LDOD.AddView(TDODLibEditView(Views[i]).Name, True);
      TDODLibEditView(Views[i]).AssignToActiveView(LDOD);
    end;

    LDOD.SaveToStream(AStream);
  finally
    FreeAndNil(LDOD);
  end;
end;

function TDODLibDataObjectDefinition.GetMainViewName(): string;
var
  LView: TDODLibEditView;
begin
  for LView in Views do
  begin
    if LView.ID = MainViewID then
    begin
      Result := LView.Name;
      Break;
    end;
  end;
end;

procedure TDODLibDataObjectDefinition.RegisterObserver(ASender: TDODObserverSenderOfEvent; AEvent: TDODObserverTypeOfEvent; AObjectID: Integer; ANotifyProc: TDODObserverEvent);
begin
  Observers.Add(TDODObserver.Create(ASender, AEvent, AObjectID, ANotifyProc));
end;

procedure TDODLibDataObjectDefinition.UnRegisterObserver(ANotifyProc: TDODObserverEvent);
var
  i: Integer;
begin
  for i := Pred(Observers.Count) downto 0 do
  begin
    if Addr(Observers[i].NotifyProc) = Addr(ANotifyProc) then
    begin
      Observers.Delete(i);
    end;
  end;
end;

procedure TDODLibDataObjectDefinition.UnRegisterObserver(ASender: TDODObserverSenderOfEvent; AEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
var
  i: Integer;
  LObserver: TDODObserver;
begin
  for i := Pred(Observers.Count) downto 0 do
  begin
    LObserver := Observers[i];
    if (LObserver.Sender = ASender) and (LObserver.Event = AEvent) and (LObserver.ObjectID = AObjectID) then
    begin
      Observers.Delete(i);
    end;
  end;
end;

procedure TDODLibDataObjectDefinition.NotifyObservers(ASender: TDODObserverSenderOfEvent; AEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
var
  LObserver: TDODObserver;
begin
  for LObserver in Observers do
  begin
    if (LObserver.Sender = ASender) and (LObserver.Event = AEvent) then
    begin
      if (AObjectID < 0) or (LObserver.ObjectID < 0) then
      begin
        // ObjectID < 0 beschreibt Broadcast-Adressen
        LObserver.NotifyProc(ASender, AEvent, AObjectID);
      end else if LObserver.ObjectID = AObjectID then
      begin
        // ObjectID's >= 0 sind reale Adressen
        LObserver.NotifyProc(ASender, AEvent, AObjectID);
      end;
    end;
  end;
end;

constructor TDODLibEditCommonList<T>.Create(AOwnsObjects: Boolean);
begin
  inherited Create(AOwnsObjects);
end;

destructor TDODLibEditCommonList<T>.Destroy();
begin
  inherited Destroy();
end;

function TDODLibEditCommonList<T>.Add(AItem: TDODLibEditCommon): Integer;
begin
  AItem.OnNotifyObserver := OnNotifyObserver;
  Result := inherited Add(AItem);

  NotifyObserver(dosViewField, doeObjectAdd, AItem.ID);
end;

procedure TDODLibEditCommonList<T>.Delete(Index: Integer);
begin
  NotifyObserver(dosViewField, doeObjectDelete, Self[Index].ID);

  inherited Delete(Index);
end;

function TDODLibEditCommonList<T>.GetIndexByID(AID: Integer): Integer;
var
  i: Integer;
begin
  Result := -1;

  for i := 0 to Count - 1 do
  begin
    if Self[i].ID = AID then
    begin
      Result := i;
      Break;
    end;
  end;
end;

function TDODLibEditCommonList<T>.GetItemByID(AID: Integer): TDODLibEditCommon;
var
  LItem: TDODLibEditCommon;
begin
  Result := nil;

  for LItem in Self do
  begin
    if LItem.ID = AID then
    begin
      Result := LItem;
      Break;
    end;
  end;
end;

procedure TDODLibEditCommonList<T>.NotifyObserver(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
begin
  if Assigned(OnNotifyObserver) then
  begin
    OnNotifyObserver(ASenderOfEvent, ATypeOfEvent, AObjectID);
  end;
end;

constructor TDODLibEditTable.Create();
begin
  inherited Create();

  FData := TibeDODefTable.Create();
end;

destructor TDODLibEditTable.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FData);
end;

procedure TDODLibEditTable.Assign(ADODEditObject: TDODLibEditCommon);
begin
  Data.Assign(TDODLibEditTable(ADODEditObject).Data);

  inherited Assign(ADODEditObject);
end;

constructor TDODLibEditTableField.Create();
begin
  inherited Create();

  FData := TibeDODefTableField.Create();
end;

destructor TDODLibEditTableField.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FData);
end;

procedure TDODLibEditTableField.Assign(ADODEditObject: TDODLibEditCommon);
begin
  Data.Assign(TDODLibEditTableField(ADODEditObject).Data);

  inherited Assign(ADODEditObject);
end;

function TDODLibEditTableFieldList.FindFieldByTableAlias(const ATableAlias, AFieldName: string): TDODLibEditTableField;
var
  i: Integer;
  LField: TDODLibEditTableField;
begin
  Result := nil;

  for i := 0 to Pred(Count) do
  begin
    LField := TDODLibEditTableField(Self[i]);

    if SameText(ATableAlias, LField.Data.TblAlias) and SameText(AFieldName, LField.Data.Name) then
    begin
      Result := LField;
      break;
    end;
  end;
end;

procedure TDODLibEditTableFieldList.AssignToDODTableFieldList(ADODTableFieldList: TibeDODefTableFieldList);
var
  LTableField: TDODLibEditTableField;
begin
  for LTableField in Self do
  begin
    ADODTableFieldList.Add(LTableField.Data);
  end;
end;

constructor TDODLibEditViewField.Create();
begin
  inherited Create();

  FData := TibeDODefViewField.Create();
end;

destructor TDODLibEditViewField.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FData);
end;

procedure TDODLibEditViewField.Assign(ADODViewField: TDODLibEditViewField; ATblFieldList: TibeDODefTableFieldList);
begin
  Data.Assign(ADODViewField.Data, ATblFieldList);

  inherited Assign(ADODViewField);
end;

procedure TDODLibEditViewFieldList.AssignToDODViewFieldList(ADODViewFieldList: TibeDODefViewFieldList);
var
  LViewField: TDODLibEditViewField;
begin
  for LViewField in Self do
  begin
    ADODViewFieldList.Add(LViewField.Data);
  end;
end;

constructor TDODLibEditViewOrderByField.Create();
begin
  inherited Create();

  FData := TibeDODefViewOrderByField.Create();
end;

destructor TDODLibEditViewOrderByField.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FData);
end;

procedure TDODLibEditViewOrderByField.Assign(ADODViewOrderByField: TDODLibEditViewOrderByField);
begin
  Data.Assign(ADODViewOrderByField.Data);

  inherited Assign(ADODViewOrderByField);
end;

constructor TDODLibEditViewGroupByField.Create();
begin
  inherited Create();

  FData := TibeDODefViewGroupByField.Create();
end;

destructor TDODLibEditViewGroupByField.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FData);
end;

procedure TDODLibEditViewGroupByField.Assign(ADODViewGroupByField: TDODLibEditViewGroupByField);
begin
  Data.Assign(ADODViewGroupByField.Data);

  inherited Assign(ADODViewGroupByField);
end;

constructor TDODLibEditView.Create();
begin
  inherited Create();

  FFields := TDODLibEditViewFieldList.Create();
  FOrderByFields := TDODLibEditViewOrderByFieldList.Create();
  FGroupByFields := TDODLibEditViewGroupByFieldList.Create();
end;

destructor TDODLibEditView.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FGroupByFields);
  FreeAndNil(FOrderByFields);
  FreeAndNil(FFields);
end;

procedure TDODLibEditView.Assign(ADODView: TDODLibEditView; ATblFieldList: TibeDODefTableFieldList);
var
  i: Integer;
  LField: TDODLibEditViewField;
  LOrderField: TDODLibEditViewOrderByField;
  LGroupField: TDODLibEditViewGroupByField;
begin
  inherited Assign(ADODView);

  Name := ADODView.Name;
  Description := ADODView.Description;
  ViewPKName := ADODView.ViewPKName;

  NotifyObserver(dosView, doeObjectUpdate);

  Fields.Clear();
  for i := 0 to Pred(ADODView.Fields.Count) do
  begin
    LField := TDODLibEditViewField.Create();
    try
      LField.Assign(ADODView.Fields[i], ATblFieldList);
      Fields.Add(LField);
    except
      FreeAndNil(LField);
      raise;
    end;
  end;

  OrderByFields.Clear();
  for i := 0 to Pred(ADODView.OrderByFields.Count) do
  begin
    LOrderField := TDODLibEditViewOrderByField.Create();
    try
      LOrderField.Assign(TDODLibEditViewOrderByField(ADODView.OrderByFields[i]));
      OrderByFields.Add(LOrderField);
    except
      FreeAndNil(LOrderField);
      raise;
    end;
  end;

  GroupByFields.Clear();
  for i := 0 to Pred(ADODView.GroupByFields.Count) do
  begin
    LGroupField := TDODLibEditViewGroupByField.Create();
    try
      LGroupField.Assign(ADODView.GroupByFields[i]);
      GroupByFields.Add(LGroupField);
    except
      FreeAndNil(LGroupField);
      raise;
    end;
  end;
end;

procedure TDODLibEditView.Assign(ADOView: TibeDODefView; ATblFieldList: TibeDODefTableFieldList);
var
  i: Integer;
  LField: TDODLibEditViewField;
  LOrderField: TDODLibEditViewOrderByField;
  LGroupField: TDODLibEditViewGroupByField;
begin
  Name := ADOView.Name;
  Description := ADOView.Description;
  ViewPKName := ADOView.ViewPKName;

  NotifyObserver(dosView, doeObjectUpdate);

  Fields.Clear();
  for i := 0 to Pred(ADOView.FieldCount) do
  begin
    LField := TDODLibEditViewField.Create();
    try
      LField.Data.Assign(ADOView.Field[i], ATblFieldList);
      Fields.Add(LField);
    except
      FreeAndNil(LField);
      raise;
    end;
  end;

  OrderByFields.Clear();
  for i := 0 to Pred(ADOView.OrderByFieldCount) do
  begin
    LOrderField := TDODLibEditViewOrderByField.Create();
    try
      LOrderField.Data.Assign(ADOView.OrderByField[i]);
      OrderByFields.Add(LOrderField);
    except
      FreeAndNil(LOrderField);
      raise;
    end;
  end;

  GroupByFields.Clear();
  for i := 0 to Pred(ADOView.GroupByFieldCount) do
  begin
    LGroupField := TDODLibEditViewGroupByField.Create();
    try
      LGroupField.Data.Assign(ADOView.GroupByField[i]);
      GroupByFields.Add(LGroupField);
    except
      FreeAndNil(LGroupField);
      raise;
    end;
  end;
end;

procedure TDODLibEditView.AssignToActiveView(ADOD: TibeDataObjectDefinition);
var
  i: Integer;
  LField: TibeDODefViewField;
  LOrderField: TibeDODefViewOrderByField;
  LGroupField: TibeDODefViewGroupByField;
begin
  ADOD.ActiveView.Name := Name;
  ADOD.ActiveView.Description := Description;
  ADOD.ActiveView.ViewPKName := ViewPKName;

  for i := 0 to Pred(Fields.Count) do
  begin
    LField := TDODLibEditViewField(Fields[i]).Data;
    //ADOD.AddViewField(LField.FieldName, LField.Caption, LField.AggFunc);
    ADOD.AddViewField(LField);
  end;

  for i := 0 to Pred(OrderByFields.Count) do
  begin
    LOrderField := TDODLibEditViewOrderByField(OrderByFields[i]).Data;
    ADOD.AddViewOrderByField(LOrderField.TblFieldAlias, LOrderField.Direction);
  end;

  for i := 0 to Pred(GroupByFields.Count) do
  begin
    LGroupField := TDODLibEditViewGroupByField(GroupByFields[i]).Data;
    ADOD.AddViewGroupByField(LGroupField.TblFieldAlias);
  end;
end;

constructor TDODLibEditCommon.Create();
begin
  inherited Create();

  ID := GenerateNextID();
end;

procedure TDODLibEditCommon.Assign(ADODEditObject: TDODLibEditCommon);
begin
  ID := ADODEditObject.ID;
end;

procedure TDODLibEditCommon.NotifyObserver(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent);
begin
  if Assigned(OnNotifyObserver) then
  begin
    OnNotifyObserver(ASenderOfEvent, ATypeOfEvent, ID);
  end;
end;

constructor TDODObserver.Create(ASender: TDODObserverSenderOfEvent; AEvent: TDODObserverTypeOfEvent; AObjectID: Integer; ANotifyProc: TDODObserverEvent);
begin
  inherited Create();

  FSender := ASender;
  FEvent := AEvent;
  FObjectID := AObjectID;
  FNotifyProc := ANotifyProc;
end;

initialization

  M_NextID := 0;

end.

