unit FDODEditor;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.Actions, Vcl.ActnList,
  Vcl.Menus, Vcl.ComCtrls, Vcl.ExtCtrls, Vcl.StdCtrls, System.UITypes,
  ibeFCommonContainer, ibeFCommon, ibeFCommonEdit,
  ibeFileUtil, ibeDataObjectDefinition,
  DODLibEditClasses,
  libFCommonEditHost,
  FDODPersistanceEdit,
  FDODViewFieldList;

type
  TfrmDODEditor = class(TfrmCommonContainer)
    actEditView: TAction;
    ViewBearbeiten1: TMenuItem;
    txtMainViewName: TEdit;
    actNewView: TAction;
    actDeleteView: TAction;
    mnuPopMainDelimViewFct: TMenuItem;
    NeueView1: TMenuItem;
    ViewLschen1: TMenuItem;
    Button1: TButton;
    actEditDOD: TAction;
    Label1: TLabel;
    procedure actEditViewExecute(Sender: TObject);
    procedure mnuPopPagesPopup(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure actDeleteViewExecute(Sender: TObject);
    procedure actNewViewExecute(Sender: TObject);
    procedure actEditDODExecute(Sender: TObject);
  private
    FDODMain: TDODLibDataObjectDefinition;
    FFrmPersistance: TfrmDODPersistanceEdit;
    FDODIsLoaded: Boolean;
    procedure UnloadDODMain();
    procedure LoadFromDODMain();
    procedure OnMainViewChanged(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
    procedure OnNewView(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
  protected
    property DODMain: TDODLibDataObjectDefinition read FDODMain;
    property FrmPersistance: TfrmDODPersistanceEdit read FFrmPersistance;
    procedure InternalAfterContainerChanged(ANewForm: TfrmCommon); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
    procedure NewDOD();
    procedure LoadFromFile(const ADODFileName: TFileName);
    procedure SaveToFile(const ADODFileName: TFileName);
    function Validate(out AMessage: string): Boolean;
    property DODIsLoaded: Boolean read FDODIsLoaded;
  end;

implementation

uses
  FDODDataEdit, FDODViewEdit;

{$R *.dfm}

constructor TfrmDODEditor.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FDODMain := TDODLibDataObjectDefinition.Create();
  FDODIsLoaded := False;
end;

procedure TfrmDODEditor.FormCreate(Sender: TObject);
begin
  inherited;

  FFrmPersistance := TfrmDODPersistanceEdit(AddForm(TfrmDODPersistanceEdit, False));
  FrmPersistance.DODMain := DODMain;
end;

procedure TfrmDODEditor.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FFrmPersistance);
end;

destructor TfrmDODEditor.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FDODMain);
end;

procedure TfrmDODEditor.InternalAfterContainerChanged(ANewForm: TfrmCommon);
begin
  inherited InternalAfterContainerChanged(ANewForm);

  if ANewForm = FrmPersistance then
  begin
    FrmPersistance.ActionRefresh();
  end;
end;

procedure TfrmDODEditor.NewDOD();
begin
  UnloadDODMain();

  DODMain.New();

  LoadFromDODMain();
end;

procedure TfrmDODEditor.LoadFromFile(const ADODFileName: TFileName);
begin
  UnloadDODMain();

  DODMain.LoadFromDODFile(ADODFileName);

  LoadFromDODMain();
end;

procedure TfrmDODEditor.UnloadDODMain();
var
  i: Integer;
begin
  FDODIsLoaded := False;

  DODMain.UnRegisterObserver(OnMainViewChanged);
  DODMain.UnRegisterObserver(OnNewView);

  for i := FormList.Count - 1 downto 0 do
  begin
    if FormList[i].Form is TfrmDODViewFieldList then
    begin
      DeleteContainerForm(FormList[i]);
    end;
  end;
end;

procedure TfrmDODEditor.LoadFromDODMain();
var
  i: Integer;
  LViewForm: TfrmDODViewFieldList;
  LContainerForm: TibeContainerForm;
begin
  for i := 0 to DODMain.Views.Count - 1 do
  begin
    pgcMain.Tag := pgcMain.Tag + 1;
    LViewForm := TfrmDODViewFieldList(AddForm(TfrmDODViewFieldList, False));
    LViewForm.Caption := 'View #' + IntToStr(pgcMain.Tag) + ' ' + TDODLibEditView(DODMain.Views[i]).Name;
    LViewForm.DODMain := DODMain;
    LViewForm.DODView := TDODLibEditView(DODMain.Views[i]);

    LContainerForm := RetrieveContainerForm(LViewForm);
    TTabSheet(LContainerForm.Container).Caption := LViewForm.Caption;
  end;

  ActivateForm(FFrmPersistance);

  txtMainViewName.Text := DODMain.MainViewName;

  DODMain.RegisterObserver(dosDOD, doeObjectUpdate, -1, OnMainViewChanged);
  DODMain.RegisterObserver(dosView, doeObjectUpdate, DODMain.MainViewID, OnMainViewChanged);
  DODMain.RegisterObserver(dosView, doeObjectAdd, -1, OnNewView);

  FDODIsLoaded := True;
end;

procedure TfrmDODEditor.SaveToFile(const ADODFileName: TFileName);
var
  LMemoryStream: TMemoryStream;
  LFileStream: TFileStream;
  LOpenMode: Integer;
begin
  //DODMain.SaveToDODFile(ADODFileName);
  // 03.10.2017 Bugfix beim berschreiben mit einer verkleinerten DOD-Datei: wenn
  // die resultierende XML-Datei verkleinert wird, wird beim direkten Schreiben
  // in einen Filestream mit DODMain.SaveToDODFile() am Ende der Datei ein Rest
  // der ursprnglichen XML-Struktur angehngt. Das lie sich auch mit dem explititen
  // Lschen der Datei vor dem Streamen oder dem Zurcksetzen von Size nach dem
  // Erzeugen des Filestreams nicht beheben.
  // Deshalb hier das DOD erst in einen Memorystream schreiben und anschlieend
  // den Memorystream in einen Filestream kopieren.
  LMemoryStream := TMemoryStream.Create();
  try
    DODMain.SaveToDODStream(LMemoryStream);

    if FileExists(ADODFileName) then
    begin
      LOpenMode := fmOpenWrite;
    end else
    begin
      LOpenMode := fmCreate;
    end;

    LFileStream := TFileStream.Create(ADODFileName, LOpenMode or fmExclusive);
    try
      LFileStream.Size := 0;
      LFileStream.Position := 0;

      LMemoryStream.SaveToStream(LFileStream);
    finally
      FreeAndNil(LFileStream);
    end;
  finally
    FreeAndNil(LMemoryStream);
  end;
end;

function TfrmDODEditor.Validate(out AMessage: string): Boolean;
var
  LDODStream: TStream;
  LDOD: TibeDataObjectDefinition;
begin
  Result := False;

  // Zum Validieren wird DODMain abgespeichert und in ein neu erzeutes DOD-Objekt
  // geladen. Die Validierung liefert True wenn der Ladevorgang ohne Fehler ausgefhrt
  // wurde.
  LDODStream := TMemoryStream.Create();
  try
    DODMain.SaveToDODStream(LDODStream);
    try
      LDOD := TibeDataObjectDefinition.Create();
      try
        LDOD.InitDefinition('DODTemp', LDODStream);
        Result := True;
      except
        on E: Exception do
        begin
          AMessage := 'Eine Exception der Klasse ' + E.ClassName + ' ist aufgetreten. Text der Fehlermeldung: ' + E.Message;
        end;
      end;
    finally
      FreeAndNil(LDOD);
    end;
  finally
    FreeAndNil(LDODStream);
  end;
end;

procedure TfrmDODEditor.mnuPopPagesPopup(Sender: TObject);
var
  LActiveForm: TibeContainerForm;
begin
  inherited;
  LActiveForm := RetrieveContainerForm(pgcMain.ActivePage);
  actEditView.Enabled := Assigned(LActiveForm) and (LActiveForm.Form is TfrmDODViewFieldList);
  actDeleteView.Enabled := actEditView.Enabled;
end;

procedure TfrmDODEditor.actEditDODExecute(Sender: TObject);
var
  LFrmEditHost: TlibFrmCommonEditHost;
  LFrmCommonEdit: TibeFrmCommonEdit;
  LFrmDODEdit: TfrmDODDataEdit;
begin
  inherited;
  LFrmEditHost := TlibFrmCommonEditHost.Create(Self);
  try
    LFrmEditHost.CreateEditForm(TfrmDODDataEdit, LFrmCommonEdit);
    LFrmDODEdit := TfrmDODDataEdit(LFrmCommonEdit);

    try
      LFrmDODEdit.DODMain := DODMain;

      LFrmEditHost.ActionEdit();
    finally
      FreeAndNil(LFrmDODEdit);
    end;
  finally
    FreeAndNil(LFrmEditHost);
  end;
end;

procedure TfrmDODEditor.actNewViewExecute(Sender: TObject);
var
  LFrmEditHost: TlibFrmCommonEditHost;
  LFrmCommonEdit: TibeFrmCommonEdit;
  LFrmViewEdit: TfrmDODViewEdit;
begin
  inherited;
  LFrmEditHost := TlibFrmCommonEditHost.Create(Self);
  try
    //LFrmEditHost.PKValue := TfrmDODViewFieldList(LActiveForm.Form).DODView.ID;
    LFrmEditHost.CreateEditForm(TfrmDODViewEdit, LFrmCommonEdit);
    LFrmViewEdit := TfrmDODViewEdit(LFrmCommonEdit);

    try
      LFrmViewEdit.DODMain := DODMain;

      LFrmEditHost.ActionNew();
    finally
      FreeAndNil(LFrmViewEdit);
    end;
  finally
    FreeAndNil(LFrmEditHost);
  end;
end;

procedure TfrmDODEditor.actEditViewExecute(Sender: TObject);
var
  LFrmEditHost: TlibFrmCommonEditHost;
  LFrmCommonEdit: TibeFrmCommonEdit;
  LFrmViewEdit: TfrmDODViewEdit;
  LActiveForm: TibeContainerForm;
begin
  inherited;
  LActiveForm := RetrieveContainerForm(pgcMain.ActivePage);

  if Assigned(LActiveForm) and (LActiveForm.Form is TfrmDODViewFieldList) then
  begin
    LFrmEditHost := TlibFrmCommonEditHost.Create(Self);
    try
      LFrmEditHost.PKValue := TfrmDODViewFieldList(LActiveForm.Form).DODView.ID;
      LFrmEditHost.CreateEditForm(TfrmDODViewEdit, LFrmCommonEdit);
      LFrmViewEdit := TfrmDODViewEdit(LFrmCommonEdit);

      try
        LFrmViewEdit.DODMain := DODMain;

        LFrmEditHost.ActionEdit();
      finally
        FreeAndNil(LFrmViewEdit);
      end;
    finally
      FreeAndNil(LFrmEditHost);
    end;
  end;
end;

procedure TfrmDODEditor.actDeleteViewExecute(Sender: TObject);
var
  LIndex: Integer;
  LMsg: string;
  LActiveForm: TibeContainerForm;
begin
  inherited;
  LActiveForm := RetrieveContainerForm(pgcMain.ActivePage);

  if Assigned(LActiveForm) and (LActiveForm.Form is TfrmDODViewFieldList) then
  begin
    LIndex := DODMain.Views.IndexOf(TfrmDODViewFieldList(LActiveForm.Form).DODView);
    if LIndex > -1 then
    begin
      LMsg := 'Wollen Sie die View "' + DODMain.Views[LIndex].Name + '" wirklich lschen?';
      if MessageDlg(LMsg, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      begin
        DODMain.Views.Delete(LIndex);
        DeleteContainerForm(LActiveForm);
      end;
    end;
  end;
end;

procedure TfrmDODEditor.OnMainViewChanged(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
begin
  txtMainViewName.Text := DODMain.MainViewName;
end;

procedure TfrmDODEditor.OnNewView(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
var
  LView: TDODLibEditView;
  LViewForm: TfrmDODViewFieldList;
  LContainerForm: TibeContainerForm;
begin
  LView := nil;

  for LView in DODMain.Views do
  begin
    if LView.ID = AObjectID then
    begin
      Break;
    end;
  end;

  if Assigned(LView) then
  begin
    pgcMain.Tag := pgcMain.Tag + 1;
    LViewForm := TfrmDODViewFieldList(AddForm(TfrmDODViewFieldList, False));
    LViewForm.Caption := 'View #' + IntToStr(pgcMain.Tag) + ' ' + LView.Name;
    LViewForm.DODMain := DODMain;
    LViewForm.DODView := LView;

    LContainerForm := RetrieveContainerForm(LViewForm);
    TTabSheet(LContainerForm.Container).Caption := LViewForm.Caption;

    ActivateForm(LViewForm);
  end;
end;

end.

