unit FDODTableEdit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.Actions,
  Vcl.ActnList, Vcl.StdCtrls, Vcl.ExtCtrls,
  ibeFCommonEdit,
  ibeDataObjectDefinition, ibeDataObjectTypes,
  FDODLibCommonEdit,
  DODLibEditClasses,
  FDODTableNativeFieldList;

type
  TfrmDODTableEdit = class(TfrmDODLibCommonEdit)
    spTableHeight: TSplitter;
    pnlNativeFields: TPanel;
    pnlTable: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    txtAlias: TEdit;
    cboJoinType: TComboBox;
    cboTableName: TComboBox;
    txtFieldPrefix: TEdit;
    txtJoinCondition: TEdit;
    txtDescription: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure txtAliasExit(Sender: TObject);
    procedure cboTableNameExit(Sender: TObject);
    procedure txtFieldPrefixExit(Sender: TObject);
  private
    FFrmNativeFields: TfrmDODTableNativeFieldList;
    FDODTable: TDODLibEditTable;
    procedure DoRefreshNativeFields();
  protected
    property FrmNativeFields: TfrmDODTableNativeFieldList read FFrmNativeFields;
    property DODTable: TDODLibEditTable read FDODTable;
    procedure DataLoad(); override;
    procedure DataDisplay(); override;
    procedure DataStore(); override;
    procedure DataSave(); override;
  public
  end;

implementation

{$R *.dfm}

procedure TfrmDODTableEdit.FormCreate(Sender: TObject);
var
  LJoinType: TibeDOTableJoinType;
  LStringList: TStringList;
begin
  inherited;

  FDODTable := TDODLibEditTable.Create();

  cboTableName.Items.Clear();
  LStringList := TStringList.Create();
  try
    ASP.Connection.RetrieveTableList(LStringList);
    LStringList.Sort();
    cboTableName.Items.Assign(LStringList);
  finally
    FreeAndNil(LStringList);
  end;

  cboJoinType.Items.Clear();
  for LJoinType := Low(TibeDOTableJoinType) to High(TibeDOTableJoinType) do
  begin
    cboJoinType.Items.Add(TableJoinTypeArray[LJoinType]);
  end;

  FFrmNativeFields := TfrmDODTableNativeFieldList.Create(Self);
  FrmNativeFields.DisplayInContainer(pnlNativeFields);
  FrmNativeFields.Show();
end;

procedure TfrmDODTableEdit.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FDODTable);
end;

procedure TfrmDODTableEdit.txtAliasExit(Sender: TObject);
begin
  inherited;

  if (txtAlias.Text <> '') and (txtFieldPrefix.Text = '') then
  begin
    // wenn das Alias-Feld gefllt und noch kein Feldprfix angegeben ist,
    // dann den Feldprfix mit dem Aliasnamen in Grobuchstaben fllen
    txtFieldPrefix.Text := UpperCase(txtAlias.Text);
  end;

  if txtAlias.Text <> FrmNativeFields.TableAlias then
  begin
    DoRefreshNativeFields();
  end;
end;

procedure TfrmDODTableEdit.txtFieldPrefixExit(Sender: TObject);
begin
  inherited;
  if txtFieldPrefix.Text <> FrmNativeFields.TableFieldPrefix then
  begin
    DoRefreshNativeFields();
  end;
end;

procedure TfrmDODTableEdit.DataLoad();
var
  LTable: TDODLibEditTable;
begin
  if EditMode in [emBrowse, emEdit] then
  begin
    LTable := TDODLibEditTable(DODMain.Tables.ItemByID[PKValue]);

    if Assigned(LTable) then
    begin
      DODTable.Assign(LTable);
    end;
  end;

  FrmNativeFields.DODMain := DODMain;
end;

procedure TfrmDODTableEdit.cboTableNameExit(Sender: TObject);
begin
  inherited;
  if cboTableName.Text <> FrmNativeFields.TableName then
  begin
    DoRefreshNativeFields();
  end;
end;

procedure TfrmDODTableEdit.DataDisplay();
begin
  txtAlias.Text := DODTable.Data.Alias;
  cboTableName.Text := DODTable.Data.Name;
  txtFieldPrefix.Text := DODTable.Data.FieldPrefix;
  cboJoinType.Text := DODTable.Data.JoinType;
  txtJoinCondition.Text := DODTable.Data.JoinCondition;
  txtDescription.Text := DODTable.Data.Description;

  // nachdem die Controls der Tabellendaten gefllt sind kann die Feldliste
  // aktualisiert werden
  DoRefreshNativeFields();
end;

procedure TfrmDODTableEdit.DataStore();
begin
  DODTable.Data.Alias := txtAlias.Text;
  DODTable.Data.Name := cboTableName.Text;
  DODTable.Data.FieldPrefix := txtFieldPrefix.Text;
  DODTable.Data.JoinType := cboJoinType.Text;
  DODTable.Data.JoinCondition := txtJoinCondition.Text;
  DODTable.Data.Description := txtDescription.Text;

  FrmNativeFields.TableName := cboTableName.Text;
  FrmNativeFields.TableAlias := txtAlias.Text;
  FrmNativeFields.TableFieldPrefix := txtFieldPrefix.Text;
end;

procedure TfrmDODTableEdit.DataSave();
var
  LTable: TDODLibEditTable;
begin
  PKValue := DODTable.ID;

  if EditMode = emEdit then
  begin
    // mit der ID aus der Edit-Form nach der Tabelle im DOD suchen. Der gefunden
    // Tabelle den Inhalt der Tabelle aus der Edit-Form zuweisen
    LTable := TDODLibEditTable(DODMain.Tables.ItemByID[DODTable.ID]);

    if Assigned(LTable) then
    begin
      LTable.Assign(DODTable);
    end;
  end else if EditMode = emNew then
  begin
    // ein neues Tabellenobjekt erzeugen und in das DOD einfgen
    LTable := TDODLibEditTable.Create();
    LTable.Assign(DODTable);
    DODMain.Tables.Add(LTable);
  end;

  FrmNativeFields.ActionSave();
end;

procedure TfrmDODTableEdit.DoRefreshNativeFields();
begin
  FrmNativeFields.TableName := cboTableName.Text;
  FrmNativeFields.TableAlias := txtAlias.Text;
  FrmNativeFields.TableFieldPrefix := txtFieldPrefix.Text;
  FrmNativeFields.ActionRefresh();
end;

end.

