unit FDODTableFieldEdit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.Actions, Vcl.ActnList,
  Vcl.StdCtrls,
  ibeFCommonEdit, FDODLibCommonEdit,
  ibeDataObjectDefinition, ibeDataObjectTypes, ibeDALConnection,
  DODLibEditClasses, Vcl.ExtCtrls;

type
  TfrmDODTableFieldEdit = class(TfrmDODLibCommonEdit)
    Label1: TLabel;
    cboDataType: TComboBox;
    Label2: TLabel;
    cboTable: TComboBox;
    optFieldTypeField: TRadioButton;
    Label3: TLabel;
    cboTableField: TComboBox;
    optFieldTypeExpression: TRadioButton;
    txtExpression: TMemo;
    lblExpression: TLabel;
    Label6: TLabel;
    txtAlias: TEdit;
    Label7: TLabel;
    txtCaption: TEdit;
    Label8: TLabel;
    txtWidth: TEdit;
    Label9: TLabel;
    txtDisplayOptions: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure optFieldTypeFieldClick(Sender: TObject);
    procedure optFieldTypeExpressionClick(Sender: TObject);
    procedure cboTableSelect(Sender: TObject);
    procedure cboTableFieldSelect(Sender: TObject);
  private
    FDODTableField: TDODLibEditTableField;
    FTableFieldList: TibeNativeDatabaseFieldList;
    FTableFieldPrefix: string;
    procedure UpdateCboTableFields(const ATableName: string);
  protected
    property DODTableField: TDODLibEditTableField read FDODTableField;
    procedure DataLoad(); override;
    procedure DataDisplay(); override;
    procedure DataStore(); override;
    procedure DataSave(); override;
  public
  end;

implementation

{$R *.dfm}

procedure TfrmDODTableFieldEdit.FormCreate(Sender: TObject);
var
  LDODDataType: TibeDODefDataType;
begin
  inherited;
  FDODTableField := TDODLibEditTableField.Create();
  FTableFieldList := TibeNativeDatabaseFieldList.Create();

  cboDataType.Clear();
  for LDODDataType := Low(TibeDODefDataType) to High(TibeDODefDataType) do
  begin
    cboDataType.Items.Add(DODefDataTypeIDArray[LDODDataType]);
  end;
end;

procedure TfrmDODTableFieldEdit.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FTableFieldList);
  FreeAndNil(FDODTableField);
end;

procedure TfrmDODTableFieldEdit.DataLoad();
var
  i: Integer;
  LField: TDODLibEditTableField;
begin
  if EditMode in [emBrowse, emEdit] then
  begin
    LField := TDODLibEditTableField(DODMain.TableFields.ItemByID[PKValue]);

    if Assigned(LField) then
    begin
      DODTableField.Assign(LField);
    end;
  end;

  cboTable.Clear();
  for i := 0 to Pred(DODMain.Tables.Count) do
  begin
    cboTable.Items.AddObject(TDODLibEditTable(DODMain.Tables[i]).Data.Alias, DODMain.Tables[i]);
  end;
end;

procedure TfrmDODTableFieldEdit.DataDisplay();
var
  LTableName: string;
begin
  cboTable.ItemIndex := cboTable.Items.IndexOf(DODTableField.Data.TblAlias);
//  txtFieldName.Text := DODTableField.Data.Name;
  txtAlias.Text := DODTableField.Data.Alias;
  txtExpression.Text := DODTableField.Data.Expression;
  txtCaption.Text := DODTableField.Data.Caption;
  txtWidth.Text := IntToStr(DODTableField.Data.Width);
  txtDisplayOptions.Text := DODTableField.Data.DisplayOption;
  cboDataType.ItemIndex := cboDataType.Items.IndexOf(DODefDataTypeIDArray[DODTableField.Data.DataType]);

  if cboTable.ItemIndex >= 0 then
  begin
    LTableName := TDODLibEditTable(cboTable.Items.Objects[cboTable.ItemIndex]).Data.Name;
    UpdateCboTableFields(LTableName);
    cboTableField.ItemIndex := cboTableField.Items.IndexOf(DODTableField.Data.Name);
  end;

  optFieldTypeExpression.Checked := DODTableField.Data.IsExpression;
  optFieldTypeField.Checked := not optFieldTypeExpression.Checked;
end;

procedure TfrmDODTableFieldEdit.DataStore();
begin
  DODTableField.Data.TblAlias := cboTable.Text;
  //DODTableField.Data.Name := txtFieldName.Text;
  DODTableField.Data.Alias := txtAlias.Text;
  DODTableField.Data.Caption := txtCaption.Text;
  DODTableField.Data.Width := StrToInt(txtWidth.Text);
  DODTableField.Data.DisplayOption := txtDisplayOptions.Text;
  DODTableField.Data.DataType :=  DODefDataTypeFromString(cboDataType.Text);
  if optFieldTypeExpression.Checked then
  begin
    DODTableField.Data.Expression := txtExpression.Text;
  end else
  begin
    DODTableField.Data.Name := cboTableField.Text;
  end;
end;

procedure TfrmDODTableFieldEdit.DataSave();
var
  LField: TDODLibEditTableField;
begin
  PKValue := DODTableField.ID;

  if EditMode = emEdit then
  begin
    // mit der ID aus der Edit-Form nach dem Tabellenfeld im DOD suchen. Dem
    // gefunden Tabellenfeld den Inhalt des Felds aus der Edit-Form zuweisen
    LField := TDODLibEditTableField(DODMain.TableFields.ItemByID[DODTableField.ID]);

    if Assigned(LField) then
    begin
      LField.Assign(DODTableField);
    end;
  end else if EditMode = emNew then
  begin
    // ein neues Feldobjekt erzeugen und in das DOD einfgen
    LField := TDODLibEditTableField.Create();
    LField.Assign(DODTableField);
    DODMain.TableFields.Add(LField);
  end;
end;

procedure TfrmDODTableFieldEdit.UpdateCboTableFields(const ATableName: string);
var
  LField: TibeNativeDatabaseField;
begin
  inherited;
  cboTableField.Clear();
  FTableFieldList.Clear();

  if cboTable.ItemIndex <> -1 then
  begin
    ASP.Connection.RetrieveTableFieldList(ATableName, FTableFieldList);
  end;

  for LField in FTableFieldList do
  begin
    cboTableField.Items.AddObject(LField.FieldName, LField);
  end;
end;

procedure TfrmDODTableFieldEdit.cboTableFieldSelect(Sender: TObject);
var
  LField: TibeNativeDatabaseField;
begin
  inherited;
  txtAlias.Text := FTableFieldPrefix + '_' + cboTableField.Text;

  if cboTableField.Items.Count > 0 then
  begin
    LField := TibeNativeDatabaseField(cboTableField.Items.Objects[cboTableField.ItemIndex]);

    if DODefDataTypeFromDBFieldType(LField.DataType) = dodtString then
    begin
      txtWidth.Text := IntToStr(LField.Width);
    end;
  end;
end;

procedure TfrmDODTableFieldEdit.cboTableSelect(Sender: TObject);
var
  LTableName: string;
  LField: TibeNativeDatabaseField;
begin
  inherited;

  FTableFieldPrefix := TDODLibEditTable(cboTable.Items.Objects[cboTable.ItemIndex]).Data.FieldPrefix;
  LTableName := TDODLibEditTable(cboTable.Items.Objects[cboTable.ItemIndex]).Data.Name;
  UpdateCboTableFields(LTableName);

  if cboTableField.Items.Count > 0 then
  begin
    cboTableField.ItemIndex := 0;
    txtAlias.Text := FTableFieldPrefix + '_' + cboTableField.Text;
    LField := TibeNativeDatabaseField(cboTableField.Items.Objects[cboTableField.ItemIndex]);
    if DODefDataTypeFromDBFieldType(LField.DataType) = dodtString then
    begin
      txtWidth.Text := IntToStr(LField.Width);
    end;
  end;
end;

procedure TfrmDODTableFieldEdit.optFieldTypeExpressionClick(Sender: TObject);
begin
  inherited;
  lblExpression.Enabled := optFieldTypeExpression.Checked;
  txtExpression.Enabled := optFieldTypeExpression.Checked;
end;

procedure TfrmDODTableFieldEdit.optFieldTypeFieldClick(Sender: TObject);
begin
  inherited;
  lblExpression.Enabled := optFieldTypeExpression.Checked;
  txtExpression.Enabled := optFieldTypeExpression.Checked;
end;

end.

