unit FDODTableFieldList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, Vcl.Menus, System.StrUtils,
  System.Actions, Vcl.ActnList, Vcl.StdCtrls, Vcl.ExtCtrls, System.ImageList,
  Vcl.ImgList, Vcl.ToolWin, Vcl.ComCtrls,
  VirtualTrees, ibeVirtualTreeview,
  ibeDataObjectTypes,
  ibeFCommonEdit,
  DODLibEditClasses,
  FDODLibCommonList;


type

TDODTableFieldListNodeData = record
  TableField: TDODLibEditTableField;
  ViewField: TDODLibEditViewField;    // dieses Feld wird in Ableitungen der Liste
                                      // fr die Zuordnung von View- und Tabellenfeld
                                      // verwendet
end;
PDODTableFieldListNodeData = ^TDODTableFieldListNodeData;

  TfrmDODTableFieldList = class(TfrmDODLibCommonList)
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    procedure FormCreate(Sender: TObject);
  private
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
    procedure OnEndUpdateNotification(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionDisplay(); override;
    procedure InternalActionDelete(); override;
    procedure InitNodeData(ANode: PVirtualNode; ANodeData: PDODTableFieldListNodeData); virtual;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
    procedure SetDODMain(ADOD: TDODLibDataObjectDefinition); override;
    function RetieveNodeData(ANode: PVirtualNode): PDODTableFieldListNodeData;
    function RetieveNodeTableField(ANode: PVirtualNode): TDODLibEditTableField;
    function RetrieveEditClass(): TibeFrmCommonEditClass; override;
  public
  end;

implementation

uses
  FDODTableFieldEdit;

{$R *.dfm}

procedure TfrmDODTableFieldList.FormCreate(Sender: TObject);
begin
  inherited;

  grdMain.OnGetText := grdMainGetText;
  grdMain.OnFreeNode := grdMainFreeNode;
  grdMain.NodeDataSize := SizeOf(TDODTableFieldListNodeData);
end;

procedure TfrmDODTableFieldList.InternalActionDisplay();
var
  i: Integer;
  LNode: PVirtualNode;
  LData: PDODTableFieldListNodeData;
begin
  inherited InternalActionDisplay();

  for i := 0 to DODMain.TableFields.Count - 1 do
  begin
    LNode := grdMain.AddChild(nil);
    LData := grdMain.GetNodeData(LNode);
    LData.TableField := TDODLibEditTableField.Create();
    LData.TableField.Assign(DODMain.TableFields[i]);

    InitNodeData(LNode, LData);
  end;
end;

procedure TfrmDODTableFieldList.InitNodeData(ANode: PVirtualNode; ANodeData: PDODTableFieldListNodeData);
begin
end;

procedure TfrmDODTableFieldList.InternalActionDelete();
var
  LIndex: Integer;
  LField: TDODLibEditTableField;
begin
  inherited InternalActionDelete();

  LField := RetieveNodeTableField(grdMain.FocusedNode);

  if Assigned(LField) then
  begin
    LIndex := DODMain.TableFields.IndexByID[LField.ID];

    if LIndex >= 0 then
    begin
      DODMain.TableFields.Delete(LIndex);
    end;
  end;
end;

procedure TfrmDODTableFieldList.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();

  AddGridMainColumn('Feldalias', 200);
  AddGridMainColumn('Tabellenalias', 100);
  AddGridMainColumn('Feldname', 150);
  AddGridMainColumn('Beschriftung', 150);
  AddGridMainColumn('Zeichen', 50);
  AddGridMainColumn('Anz.Optionen', 75);
  AddGridMainColumn('Datentyp', 75);
  AddGridMainColumn('Expression', 150);
end;

procedure TfrmDODTableFieldList.OnEndUpdateNotification(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
begin
  ActionRefresh();
end;

procedure TfrmDODTableFieldList.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LField: TDODLibEditTableField;
begin
  ACellText := '';

  LField := RetieveNodeTableField(ANode);

  if Assigned(LField) then
  begin
    case AColumn of
      0: ACellText := LField.Data.Alias;
      1: ACellText := LField.Data.TblAlias;
      2: ACellText := LField.Data.Name;
      3: ACellText := LField.Data.Caption;
      4: ACellText := IfThen(LField.Data.DataType in [dodtString], IntToStr(LField.Data.Width), '');
      5: ACellText := LField.Data.DisplayOption;
      6: ACellText := DODefDataTypeIDArray[LField.Data.DataType];
      7: ACellText := LField.Data.Expression;
    end;
  end;
end;

procedure TfrmDODTableFieldList.grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
var
  LNodeData: PDODTableFieldListNodeData;
begin
  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      FreeAndNil(LNodeData.TableField);
    end;
  end;
end;

function TfrmDODTableFieldList.GetPKValue(): Variant;
var
  LField: TDODLibEditTableField;
begin
  Result := Null;

  LField := RetieveNodeTableField(grdMain.FocusedNode);

  if Assigned(LField) then
  begin
    Result := LField.ID;
  end;
end;

procedure TfrmDODTableFieldList.SetPKValue(APKValue: Variant);
var
  LNode: PVirtualNode;
  LNodeData: PDODTableFieldListNodeData;
begin
  LNode := grdMain.GetFirst();

  while Assigned(LNode) do
  begin
    LNodeData := grdMain.GetNodeData(LNode);

    if Assigned(LNodeData) then
    begin
      if APKValue = LNodeData.TableField.ID then
      begin
        SelectNode(LNode);
        Break;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

procedure TfrmDODTableFieldList.SetDODMain(ADOD: TDODLibDataObjectDefinition);
begin
  inherited SetDODMain(ADOD);

  if Assigned(ADOD) then
  begin
    ADOD.RegisterObserver(dosTableField, doeEndBatchUpdate, -1, OnEndUpdateNotification);
  end;
end;

function TfrmDODTableFieldList.RetieveNodeData(ANode: PVirtualNode): PDODTableFieldListNodeData;
begin
  Result := nil;

  if Assigned(ANode) then
  begin
    Result := grdMain.GetNodeData(ANode);
  end;
end;

function TfrmDODTableFieldList.RetieveNodeTableField(ANode: PVirtualNode): TDODLibEditTableField;
var
  LNodeData: PDODTableFieldListNodeData;
begin
  Result := nil;

  LNodeData := RetieveNodeData(ANode);

  if Assigned(LNodeData) then
  begin
    Result := LNodeData.TableField;
  end;
end;

function TfrmDODTableFieldList.RetrieveEditClass(): TibeFrmCommonEditClass;
begin
  Result := TfrmDODTableFieldEdit;
end;

end.

