unit FDODTableList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Menus,
  System.Actions, Vcl.ActnList, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.ComCtrls,
  System.ImageList, Vcl.ImgList, Vcl.ToolWin,
  VirtualTrees, ibeVirtualTreeview,
  ibeFCommonEdit,
  DODLibEditClasses,
  FDODLibCommonList;

type
  TfrmDODTableList = class(TfrmDODLibCommonList)
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    procedure FormCreate(Sender: TObject);
  private
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionDisplay(); override;
    procedure InternalActionDelete(); override;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
    function RetieveNodeTable(ANode: PVirtualNode): TDODLibEditTable;
    function RetrieveEditClass(): TibeFrmCommonEditClass; override;
  public
  end;

implementation

uses
  FDODTableEdit;

type

TDODTableListNodeData = record
  Table: TDODLibEditTable;
end;
PDODTableListNodeData = ^TDODTableListNodeData;

{$R *.dfm}

procedure TfrmDODTableList.FormCreate(Sender: TObject);
begin
  inherited;

  grdMain.OnGetText := grdMainGetText;
  grdMain.OnFreeNode := grdMainFreeNode;
  grdMain.NodeDataSize := SizeOf(TDODTableListNodeData);
end;

procedure TfrmDODTableList.InternalActionDisplay();
var
  i: Integer;
  LNode: PVirtualNode;
  LData: PDODTableListNodeData;
begin
  inherited InternalActionDisplay();

  for i := 0 to DODMain.Tables.Count - 1 do
  begin
    LNode := grdMain.AddChild(nil);
    LData := grdMain.GetNodeData(LNode);
    LData.Table := TDODLibEditTable.Create();
    LData.Table.Assign(DODMain.Tables[i]);
  end;
end;

procedure TfrmDODTableList.InternalActionDelete();
var
  LIndex: Integer;
  LTable: TDODLibEditTable;
begin
  inherited InternalActionDelete();

  LTable := RetieveNodeTable(grdMain.FocusedNode);

  if Assigned(LTable) then
  begin
    LIndex := DODMain.Tables.IndexByID[LTable.ID];

    if LIndex >= 0 then
    begin
      DODMain.Tables.Delete(LIndex);
    end;
  end;
end;

procedure TfrmDODTableList.InternalActionInitGridMain();
begin
  AddGridMainColumn('Alias', 150);
  AddGridMainColumn('Name', 200);
  AddGridMainColumn('Join-Typ', 150);
  AddGridMainColumn('Join-Bedingung', 250);
  AddGridMainColumn('Beschreibung', 250);
end;

procedure TfrmDODTableList.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LTable: TDODLibEditTable;
begin
  ACellText := '';

  LTable := RetieveNodeTable(ANode);

  if Assigned(LTable) then
  begin
    case AColumn of
      0:  ACellText := LTable.Data.Alias;
      1:  ACellText := LTable.Data.Name;
      2:  ACellText := LTable.Data.JoinType;
      3:  ACellText := LTable.Data.JoinCondition;
      4:  ACellText := LTable.Data.Description;
    end;
  end;
end;

procedure TfrmDODTableList.grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
var
  LNodeData: PDODTableListNodeData;
begin
  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      FreeAndNil(LNodeData.Table);
    end;
  end;
end;

function TfrmDODTableList.GetPKValue(): Variant;
var
  LTable: TDODLibEditTable;
begin
  Result := Null;

  LTable := RetieveNodeTable(grdMain.FocusedNode);

  if Assigned(LTable) then
  begin
    Result := LTable.ID;
  end;
end;

procedure TfrmDODTableList.SetPKValue(APKValue: Variant);
var
  LNode: PVirtualNode;
  LNodeData: PDODTableListNodeData;
begin
  LNode := grdMain.GetFirst();

  while Assigned(LNode) do
  begin
    LNodeData := grdMain.GetNodeData(LNode);

    if Assigned(LNodeData) then
    begin
      if APKValue = LNodeData.Table.ID then
      begin
        SelectNode(LNode);
        Break;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

function TfrmDODTableList.RetieveNodeTable(ANode: PVirtualNode): TDODLibEditTable;
var
  LNodeData: PDODTableListNodeData;
begin
  Result := nil;

  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      Result := LNodeData.Table;
    end;
  end;
end;

function TfrmDODTableList.RetrieveEditClass(): TibeFrmCommonEditClass;
begin
  Result := TfrmDODTableEdit;
end;

end.

