unit FDODTableNativeFieldList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.ImageList, System.StrUtils,
  System.UITypes, Vcl.ImgList, Vcl.Menus, System.Actions, Vcl.ActnList, Vcl.StdCtrls,
  Vcl.ToolWin, Vcl.ComCtrls, Vcl.ExtCtrls, Data.DB, Generics.Collections,
  VirtualTrees, ibeVirtualTreeview,
  ibeFCommonEdit,
  FDODLibCommonList,
  ibeDALConnection, ibeDataObjectTypes,
  DODLibEditClasses;

type

TDatabaseField = class(TibeNativeDatabaseField)
private
  FDODTableField: TDODLibEditTableField;
protected
public
  property DODTableField: TDODLibEditTableField read FDODTableField write FDODTableField;
end;

TDatabaseFieldList = class(TObjectList<TDatabaseField>);

  TfrmDODTableNativeFieldList = class(TfrmDODLibCommonList)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FTableFields: TDatabaseFieldList;
    FTableName, FTableAlias, FTableFieldPrefix: string;
    procedure DoSaveDeleteObsoleteReferences();
    procedure DoSaveCreateNewTableFields();
    procedure DoSaveUpdateExistingTableFields();
    procedure DoSaveDeleteUncheckedTableFields();
    procedure UpdateTableFieldProperties(ATableField: TDODLibEditTableField; AField: TDatabaseField);
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure grdMainDrawText(ASender: TBaseVirtualTree; ATargetCanvas: TCanvas; ANode: PVirtualNode; AColumn: TColumnIndex; const AText: string; const ACellRect: TRect; var ADefaultDraw: Boolean);
  protected
    property TableFields: TDatabaseFieldList read FTableFields;
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionRefresh(); override;
    procedure InternalActionDisplay(); override;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
    function RetrieveEditClass(): TibeFrmCommonEditClass; override;
    function RetieveNodeField(ANode: PVirtualNode): TDatabaseField;
  public
    property TableName: string read FTableName write FTableName;
    property TableAlias: string read FTableAlias write FTableAlias;
    property TableFieldPrefix: string read FTableFieldPrefix write FTableFieldPrefix;
    procedure ActionSave();
  end;

type

TDODTableFieldNodeData = record
  Field: TDatabaseField;
end;
PDODTableFieldNodeData = ^TDODTableFieldNodeData;

implementation

{$R *.dfm}

procedure TfrmDODTableNativeFieldList.FormCreate(Sender: TObject);
begin
  inherited;

  FTableFields := TDatabaseFieldList.Create();

  grdMain.OnGetText := grdMainGetText;
  grdMain.OnDrawText := grdMainDrawText;
end;

procedure TfrmDODTableNativeFieldList.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FTableFields);
end;

procedure TfrmDODTableNativeFieldList.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();

  // Spalten definieren
  AddGridMainColumn('Feldname', 150);
  AddGridMainColumn('Datentype', 150);
  AddGridMainColumn('Zeichen', 150);

  // Checkboxen fr die Nodes einrichten
  grdMain.TreeOptions.MiscOptions := grdMain.TreeOptions.MiscOptions + [toCheckSupport];
end;

procedure TfrmDODTableNativeFieldList.InternalActionRefresh();
var
  LNativeFields: TibeNativeDatabaseFieldList;
  LNativeField: TibeNativeDatabaseField;
  LField: TDatabaseField;
begin
  inherited InternalActionRefresh();

  TableFields.Clear();

  if TableName <> '' then
  begin
    LNativeFields := TibeNativeDatabaseFieldList.Create();
    try
      ASP.Connection.RetrieveTableFieldList(TableName, LNativeFields);

      for LNativeField in LNativeFields do
      begin
        LField := TDatabaseField.Create();
        try
          LField.Assign(LNativeField);
          LField.DODTableField := DODMain.TableFields.FindFieldByTableAlias(TableAlias, LField.FieldName);

          TableFields.Add(LField);
        except
          FreeAndNil(LField);
          raise;
        end;
      end;
    finally
      FreeAndNil(LNativeFields);
    end;
  end;
end;

procedure TfrmDODTableNativeFieldList.InternalActionDisplay();
var
  LField: TDatabaseField;
  LNode: PVirtualNode;
  LData: PDODTableFieldNodeData;
begin
  inherited InternalActionDisplay();

  for LField in TableFields do
  begin
    LNode := grdMain.AddChild(nil);
    LNode.CheckType := ctCheckBox;
    LData := grdMain.GetNodeData(LNode);
    LData.Field := LField;

    if Assigned(LField.DODTableField) then
    begin
      LNode.CheckState := csCheckedNormal;
    end;
  end;
end;

procedure TfrmDODTableNativeFieldList.ActionSave();
begin
  DODMain.NotifyObservers(dosTableField, doeBeginBatchUpdate, -1);
  try
    DoSaveDeleteObsoleteReferences();
    DoSaveCreateNewTableFields();
    DoSaveUpdateExistingTableFields();
    DoSaveDeleteUncheckedTableFields();
  finally
    DODMain.NotifyObservers(dosTableField, doeEndBatchUpdate, -1);
  end;
end;

/// Durchsucht das DOD nach Tabellenfeldern die in den Feldern der hier
/// untersuchten Tabelle nicht mehr vorkommen.
procedure TfrmDODTableNativeFieldList.DoSaveDeleteObsoleteReferences();
var
  i: Integer;
  LFieldFound: Boolean;
  LNode: PVirtualNode;
  LField: TDatabaseField;
  LTableField: TDODLibEditTableField;
begin
  // Schleife ber alle Tabellenfelder vom DOD
  for i := Pred(DODMain.TableFields.Count) downto 0 do
  begin
    LTableField := TDODLibEditTableField(DODMain.TableFields[i]);

    // Tabellenfelder die eine Expression bilden sind nicht immer einem nativen
    // Feld der Tabelle zugeordnet und drfen daher nicht gelscht werden
    if not LTableField.Data.IsExpression then
    begin
      // Bei passendem Tabellenalias das Tabellenfeld vom DOD in den Feldern der
      // hier gelisteten Tabelle suchen
      if SameText(TableAlias, LTableField.Data.TblAlias) then
      begin
        LFieldFound := False;
        LNode := grdMain.GetFirst();
        while Assigned(LNode) and not LFieldFound do
        begin
          LField := RetieveNodeField(LNode);

          if Assigned(LField) then
          begin
            if Assigned(LField.DODTableField) then
            begin
              LFieldFound := SameText(LField.DODTableField.Data.Name, LTableField.Data.Name);
            end;
          end;

          LNode := grdMain.GetNext(LNode);
        end;

        // Das Tabellenfeld im DOD lschen, wenn es in der Feldliste der hier
        // gelisteten Tabelle nicht vorkommt
        if not LFieldFound then
        begin
          DODMain.TableFields.Delete(i);
        end;
      end;
    end;
  end;
end;

procedure TfrmDODTableNativeFieldList.DoSaveCreateNewTableFields();
var
  LNode: PVirtualNode;
  LField: TDatabaseField;
  LTableField: TDODLibEditTableField;
begin
  LNode := grdMain.GetFirst();
  while Assigned(LNode) do
  begin
    if LNode.CheckState in [csCheckedNormal] then
    begin
      LField := RetieveNodeField(LNode);

      if Assigned(LField) then
      begin
        if not Assigned(LField.DODTableField) then
        begin
          LTableField := TDODLibEditTableField.Create();
          UpdateTableFieldProperties(LTableField, LField);

          DODMain.TableFields.Add(LTableField);
        end;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

procedure TfrmDODTableNativeFieldList.DoSaveUpdateExistingTableFields();
var
  LNode: PVirtualNode;
  LField: TDatabaseField;
begin
  LNode := grdMain.GetFirst();
  while Assigned(LNode) do
  begin
    if LNode.CheckState in [csCheckedNormal] then
    begin
      LField := RetieveNodeField(LNode);

      if Assigned(LField) then
      begin
        if Assigned(LField.DODTableField) then
        begin
          UpdateTableFieldProperties(LField.DODTableField, LField);
        end;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

procedure TfrmDODTableNativeFieldList.DoSaveDeleteUncheckedTableFields();
var
  i: Integer;
  LNode: PVirtualNode;
  LField: TDatabaseField;
begin
  LNode := grdMain.GetFirst();
  while Assigned(LNode) do
  begin
    // die nicht selektierten Nodes weiter untersuchen
    if LNode.CheckState in [csUncheckedNormal] then
    begin
      LField := RetieveNodeField(LNode);

      if Assigned(LField) then
      begin
        // wenn dem Node ein Tabellenfeld aus dem DOD zugeordnet ist, dann muss
        // das Tabellenfeld im DOD gelscht werden
        if Assigned(LField.DODTableField) then
        begin
          // das zum Tabellenfeld der Liste gehrende Tabellenfeld im DOD suchen
          // und dort lschen
          for i := Pred(DODMain.TableFields.Count) downto 0 do
          begin
            if LField.DODTableField.ID = DODMain.TableFields[i].ID then
            begin
              DODMain.TableFields.Delete(i);
              Break;
            end;
          end;
        end;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

procedure TfrmDODTableNativeFieldList.UpdateTableFieldProperties(ATableField: TDODLibEditTableField; AField: TDatabaseField);
begin
  ATableField.Data.TblAlias := TableAlias;
  ATableField.Data.Name := AField.FieldName;
  ATableField.Data.Alias := TableFieldPrefix + '_' + AField.FieldName;
  ATableField.Data.DataType := DODefDataTypeFromDBFieldType(AField.DataType);
  ATableField.Data.Width := AField.Width;

  if ATableField.Data.DataType = dodtBoolean then
  begin
    // fr Boolsche Felder Default Anzeigeoptionen einstellen
    ATableField.Data.DisplayOption := 'x;-';
  end;
end;

procedure TfrmDODTableNativeFieldList.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LField: TDatabaseField;
begin
  ACellText := '';

  LField := RetieveNodeField(ANode);

  if Assigned(LField) then
  begin
    case AColumn of
      0: ACellText := LField.FieldName;
      1: ACellText := FieldTypeNames[LField.DataType];
      2: ACellText := IfThen(LField.DataType in [ftString, ftFixedWideChar, ftWideMemo], IntToStr(LField.Width), '');
    end;
  end;
end;

procedure TfrmDODTableNativeFieldList.grdMainDrawText(ASender: TBaseVirtualTree; ATargetCanvas: TCanvas; ANode: PVirtualNode; AColumn: TColumnIndex; const AText: string; const ACellRect: TRect; var ADefaultDraw: Boolean);
var
  LField: TDatabaseField;
begin
  LField := RetieveNodeField(ANode);

  if Assigned(LField) then
  begin
    if Assigned(LField.DODTableField) then
    begin
      // Falls das Feld bereits im DOD referenziert wird die Darstellung hervorheben
      ATargetCanvas.Font.Style := ATargetCanvas.Font.Style + [fsBold];

      if not (ANode.CheckState in [csCheckedNormal, csCheckedPressed]) then
      begin
        if (ANode <> ASender.FocusedNode) and not (vsSelected in ANode.States) then
        begin
          // in normaler Darstellung Textfarbe auf Grau ndern
          ATargetCanvas.Font.Color := clGrayText;
        end;
        ATargetCanvas.Font.Style := ATargetCanvas.Font.Style + [fsItalic, fsStrikeOut];
      end;
    end else
    begin
      // neu selektierte Felder hervorheben
      if ANode.CheckState in [csCheckedNormal, csCheckedPressed] then
      begin
        if (ANode <> ASender.FocusedNode) and not (vsSelected in ANode.States) then
        begin
          // in normaler Darstellung Textfarbe auf dunkelblau und fett ndern
          ATargetCanvas.Font.Color := clBlue;
          ATargetCanvas.Font.Style := ATargetCanvas.Font.Style + [fsBold];
        end;
      end;
    end;
  end;
end;

function TfrmDODTableNativeFieldList.RetieveNodeField(ANode: PVirtualNode): TDatabaseField;
var
  LNodeData: PDODTableFieldNodeData;
begin
  Result := nil;

  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      Result := LNodeData.Field;
    end;
  end;
end;

function TfrmDODTableNativeFieldList.GetPKValue(): Variant;
var
  LField: TDatabaseField;
begin
  Result := Null;

  LField := RetieveNodeField(grdMain.FocusedNode);

  if Assigned(LField) then
  begin
    Result := LField.FieldName;
  end;
end;

procedure TfrmDODTableNativeFieldList.SetPKValue(APKValue: Variant);
var
  LNode: PVirtualNode;
  LField: TDatabaseField;
begin
  LNode := grdMain.GetFirst();

  while Assigned(LNode) do
  begin
    LField := RetieveNodeField(LNode);

    if Assigned(LField) then
    begin
      if APKValue = LField.FieldName then
      begin
        SelectNode(LNode);
        Break;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

function TfrmDODTableNativeFieldList.RetrieveEditClass(): TibeFrmCommonEditClass;
begin
  Result := nil;
end;

end.

