unit FDODViewEdit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.Actions, Vcl.ActnList,
  Vcl.ExtCtrls, Vcl.StdCtrls,
  ibeFCommonEdit,
  ibeDataObjectDefinition, ibeDataObjectTypes,
  FDODLibCommonEdit,
  DODLibEditClasses,
  FDODViewEditTableFieldList;

type
  TfrmDODViewEdit = class(TfrmDODLibCommonEdit)
    spViewHeight: TSplitter;
    pnlDODFields: TPanel;
    pnlView: TPanel;
    Label5: TLabel;
    Label9: TLabel;
    txtName: TEdit;
    txtDescription: TMemo;
    Label2: TLabel;
    cboPKField: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FFrmDODFields: TfrmDODViewEditTableFieldList;
    FDODView: TDODLibEditView;
  protected
    property FrmDODFields: TfrmDODViewEditTableFieldList read FFrmDODFields;
    property DODView: TDODLibEditView read FDODView;
    procedure DataLoad(); override;
    procedure DataDisplay(); override;
    procedure DataStore(); override;
    procedure DataSave(); override;
  public
  end;

implementation

{$R *.dfm}

procedure TfrmDODViewEdit.FormCreate(Sender: TObject);
begin
  inherited;
  FFrmDODFields := TfrmDODViewEditTableFieldList.Create(Self);
  FrmDODFields.DisplayInContainer(pnlDODFields);
  FrmDODFields.Show();

  FDODView := TDODLibEditView.Create();

  cboPKField.Clear();
end;

procedure TfrmDODViewEdit.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FDODView);
end;

procedure TfrmDODViewEdit.DataLoad();
var
  LView: TDODLibEditView;
  LTblFieldList: TibeDODefTableFieldList;
  LViewField: TDODLibEditViewField;
begin
  if EditMode in [emBrowse, emEdit] then
  begin
    LView := TDODLibEditView(DODMain.Views.ItemByID[PKValue]);

    if Assigned(LView) then
    begin
      LTblFieldList := TibeDODefTableFieldList.Create(False);
      try
        DODMain.TableFields.AssignToDODTableFieldList(LTblFieldList);

        DODView.Assign(LView, LTblFieldList);
      finally
        FreeAndNil(LTblFieldList);
      end;
    end;
  end;

  cboPKField.Clear();
  for LViewField in DODView.Fields do
  begin
    cboPKField.Items.Add(LViewField.Data.FieldName);
  end;

  FrmDODFields.DODMain := DODMain;
  FrmDODFields.Fields := DODView.Fields;
end;

procedure TfrmDODViewEdit.DataDisplay();
begin
  txtName.Text := DODView.Name;
  txtDescription.Text := DODView.Description;
  cboPKField.ItemIndex := cboPKField.Items.IndexOf(DODView.ViewPKName);

  FrmDODFields.ActionRefresh();
end;

procedure TfrmDODViewEdit.DataStore();
begin
  DODView.Name := txtName.Text;
  DODView.Description := txtDescription.Text;
  DODView.ViewPKName := cboPKField.Text;
end;

procedure TfrmDODViewEdit.DataSave();
var
  LView: TDODLibEditView;
  LTblFieldList: TibeDODefTableFieldList;
begin
  PKValue := DODView.ID;

  DODMain.NotifyObservers(dosView, doeBeginBatchUpdate, DODView.ID);
  try
    FrmDODFields.ActionSave();

    if EditMode = emEdit then
    begin
      // mit der ID aus der Edit-Form nach der View im DOD suchen. Der gefunden
      // View den Inhalt der View aus der Edit-Form zuweisen
      LView := TDODLibEditView(DODMain.Views.ItemByID[DODView.ID]);

      if Assigned(LView) then
      begin
        LTblFieldList := TibeDODefTableFieldList.Create(False);
        try
          DODMain.TableFields.AssignToDODTableFieldList(LTblFieldList);

          LView.Assign(DODView, LTblFieldList);
        finally
          FreeAndNil(LTblFieldList);
        end;
      end;

      DODMain.NotifyObservers(dosView, doeObjectUpdate, DODView.ID);
    end else if EditMode = emNew then
    begin
      // ein neues Viewobjekt erzeugen und in das DOD einfgen
      LView := TDODLibEditView.Create();
      LView.Assign(DODView);
      DODMain.Views.Add(LView);

      DODMain.NotifyObservers(dosView, doeObjectAdd, DODView.ID);
    end;
  finally
    DODMain.NotifyObservers(dosView, doeEndBatchUpdate, DODView.ID);
  end;
end;

end.

