unit FDODViewEditTableFieldList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.ImageList, Vcl.ImgList,
  Vcl.Menus, System.Actions, Vcl.ActnList, Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ToolWin,
  Vcl.ExtCtrls,
  VirtualTrees, ibeVirtualTreeview,
  ibeDataObjectTypes, ibeDataObjectDefinition,
  ibeFCommonEdit,
  FDODTableFieldList,
  DODLibEditClasses;

type
  TfrmDODViewEditTableFieldList = class(TfrmDODTableFieldList)
    procedure FormCreate(Sender: TObject);
  private
    FFields: TDODLibEditViewFieldList;
    procedure DoSaveDeleteObsoleteReferences();
    procedure DoSaveCreateNewTableFields();
    procedure DoSaveUpdateExistingTableFields();
    procedure DoSaveDeleteUncheckedTableFields();
    procedure UpdateTableFieldProperties(ATableField: TDODLibEditTableField; AField: TDODLibEditViewField);
    procedure grdMainDrawText(ASender: TBaseVirtualTree; ATargetCanvas: TCanvas; ANode: PVirtualNode; AColumn: TColumnIndex; const AText: string; const ACellRect: TRect; var ADefaultDraw: Boolean);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionRefresh(); override;
    procedure InitNodeData(ANode: PVirtualNode; ANodeData: PDODTableFieldListNodeData); override;
    procedure InternalActionDisplay(); override;
    function RetrieveEditClass(): TibeFrmCommonEditClass; override;
    function RetieveNodeViewField(ANode: PVirtualNode): TDODLibEditViewField;
  public
    procedure ActionSave();
    property Fields: TDODLibEditViewFieldList read FFields write FFields;
  end;

implementation

{$R *.dfm}

procedure TfrmDODViewEditTableFieldList.FormCreate(Sender: TObject);
begin
  inherited;

  grdMain.OnDrawText := grdMainDrawText;
end;

procedure TfrmDODViewEditTableFieldList.ActionSave();
begin
  DODMain.NotifyObservers(dosViewField, doeBeginBatchUpdate, -1);
  try
    DoSaveDeleteObsoleteReferences();
    DoSaveCreateNewTableFields();
    DoSaveUpdateExistingTableFields();
    DoSaveDeleteUncheckedTableFields();
  finally
    DODMain.NotifyObservers(dosViewField, doeEndBatchUpdate, -1);
  end;
end;

procedure TfrmDODViewEditTableFieldList.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();

  // Checkboxen fr die Nodes einrichten
  grdMain.TreeOptions.MiscOptions := grdMain.TreeOptions.MiscOptions + [toCheckSupport];
end;

procedure TfrmDODViewEditTableFieldList.InternalActionRefresh();
begin
  inherited InternalActionRefresh();
end;

procedure TfrmDODViewEditTableFieldList.InitNodeData(ANode: PVirtualNode; ANodeData: PDODTableFieldListNodeData);
var
  LViewField: TDODLibEditViewField;
begin
  inherited InitNodeData(ANode, ANodeData);

  ANode.CheckType := ctCheckBox;

  for LViewField in Fields do
  begin
    if SameText(ANodeData.TableField.Data.Alias, LViewField.Data.FieldName) then
    begin
      ANodeData.ViewField := LViewField;
      ANode.CheckState := csCheckedNormal;
      Break;
    end;
  end;
end;

procedure TfrmDODViewEditTableFieldList.InternalActionDisplay();
begin
  inherited InternalActionDisplay();
end;

// Durchsucht die View nach Feldern die in den Tabellenfeldern des DO's nicht
// mehr vorkommen.
procedure TfrmDODViewEditTableFieldList.DoSaveDeleteObsoleteReferences();
var
  i: Integer;
  LFieldFound: Boolean;
  LNode: PVirtualNode;
  LViewField: TDODLibEditViewField;
  LTableField: TDODLibEditTableField;
begin
  // Schleife ber alle die Felder der View
  for i := 0 to Pred(Fields.Count) do
  begin
    LViewField := Fields[i];

    // prfen ob das View-Feld in den Tabellenfeldern vorkommt
    for LTableField in DODMain.TableFields do
    begin
      LFieldFound := SameText(LViewField.Data.TblFieldAlias, LTableField.Data.Alias);
      if LFieldFound then
      begin
        Break;
      end;
    end;

    // wenn fr das View-Feld kein Tabellenfeld gefunden wurde, wird das View-
    // Feld vor dem abspeichern gelscht
    if not LFieldFound then
    begin
      Fields.Delete(i);
    end;
  end;
end;

procedure TfrmDODViewEditTableFieldList.DoSaveCreateNewTableFields();
var
  LNode: PVirtualNode;
  LNodeData: PDODTableFieldListNodeData;
  LViewField: TDODLibEditViewField;
  LTableField: TDODLibEditTableField;
  LDODTableFieldList: TibeDODefTableFieldList;
begin
  LDODTableFieldList := TibeDODefTableFieldList.Create(False);
  try
    DODMain.TableFields.AssignToDODTableFieldList(LDODTableFieldList);

    LNode := grdMain.GetFirst();
    while Assigned(LNode) do
    begin
      if LNode.CheckState in [csCheckedNormal] then
      begin
        // der Node ist angehakt, das TableField soll in der View genutzt werden.
        // Prfen ob es bereits ein ViewField gibt. Falls nicht, muss das ViewField
        // neu erzeugt werden.
        LNodeData := grdMain.GetNodeData(LNode);

        if Assigned(LNodeData) then
        begin
          if not Assigned(LNodeData.ViewField) then
          begin
            // In der View gibt es das Feld noch nicht. Jetzt ein neues ViewField
            // erzeugen, dessen Eigenschaften initialisieren und das Feld dann
            // der View hinzufgen
            LTableField := LNodeData.TableField;
            LViewField := TDODLibEditViewField.Create();
            try
              LViewField.Data.InitFieldAttributes(LTableField.Data.Alias, doafNone, LDODTableFieldList);

              LViewField.Data.DataType := LTableField.Data.DataType;
              LViewField.Data.VisibleInList := True;
              //LViewField.Data.Caption := LTableField.Data.Caption;

              Fields.Add(LViewField);
            except
              FreeAndNil(LViewField);
              raise;
            end;
          end;
        end;
      end;

      LNode := grdMain.GetNext(LNode);
    end;
  finally
    FreeAndNil(LDODTableFieldList);
  end;
end;

procedure TfrmDODViewEditTableFieldList.DoSaveUpdateExistingTableFields();
//var
//  LNode: PVirtualNode;
//  LField: TDatabaseField;
begin
//  LNode := grdMain.GetFirst();
//  while Assigned(LNode) do
//  begin
//    if LNode.CheckState in [csCheckedNormal] then
//    begin
//      LField := RetieveNodeField(LNode);
//
//      if Assigned(LField) then
//      begin
//        if Assigned(LField.DODTableField) then
//        begin
//          UpdateTableFieldProperties(LField.DODTableField, LField);
//        end;
//      end;
//    end;
//
//    LNode := grdMain.GetNext(LNode);
//  end;
end;

procedure TfrmDODViewEditTableFieldList.DoSaveDeleteUncheckedTableFields();
var
  i: Integer;
  LNode: PVirtualNode;
  LNodeData: PDODTableFieldListNodeData;
begin
  LNode := grdMain.GetFirst();
  while Assigned(LNode) do
  begin
    // die nicht selektierten Nodes weiter untersuchen
    if LNode.CheckState in [csUncheckedNormal] then
    begin
      LNodeData := grdMain.GetNodeData(LNode);

      if Assigned(LNodeData) then
      begin
        // wenn dem Node ein View-Feld zugeordnet ist, dann muss das View-Feld
        // in der DOD-View gelscht werden
        if Assigned(LNodeData.ViewField) then
        begin
          // das zum Tabellenfeld der Liste gehrende Tabellenfeld im DOD suchen
          // und dort lschen
          for i := Pred(Fields.Count) downto 0 do
          begin
            if LNodeData.ViewField.ID = Fields[i].ID then
            begin
              Fields.Delete(i);
              Break;
            end;
          end;
        end;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

procedure TfrmDODViewEditTableFieldList.UpdateTableFieldProperties(ATableField: TDODLibEditTableField; AField: TDODLibEditViewField);
begin
//  ATableField.Data.TblAlias := TableAlias;
//  ATableField.Data.Name := AField.FieldName;
//  ATableField.Data.Alias := TableFieldPrefix + '_' + AField.FieldName;
//  ATableField.Data.DataType := DODefDataTypeFromDBFieldType(AField.DataType);
end;

procedure TfrmDODViewEditTableFieldList.grdMainDrawText(ASender: TBaseVirtualTree; ATargetCanvas: TCanvas; ANode: PVirtualNode; AColumn: TColumnIndex; const AText: string; const ACellRect: TRect; var ADefaultDraw: Boolean);
var
  LField: TDODLibEditViewField;
begin
  LField := RetieveNodeViewField(ANode);

  if Assigned(LField) then
  begin
    // Falls das Feld bereits im DOD referenziert wird die Darstellung hervorheben
    ATargetCanvas.Font.Style := ATargetCanvas.Font.Style + [fsBold];

    if not (ANode.CheckState in [csCheckedNormal, csCheckedPressed]) then
    begin
      if (ANode <> ASender.FocusedNode) and not (vsSelected in ANode.States) then
      begin
        // in normaler Darstellung Textfarbe auf Grau ndern
        ATargetCanvas.Font.Color := clGrayText;
      end;
      ATargetCanvas.Font.Style := ATargetCanvas.Font.Style + [fsItalic, fsStrikeOut];
    end;
  end else
  begin
    // neu selektierte Felder hervorheben
    if ANode.CheckState in [csCheckedNormal, csCheckedPressed] then
    begin
      if (ANode <> ASender.FocusedNode) and not (vsSelected in ANode.States) then
      begin
        // in normaler Darstellung Textfarbe auf dunkelblau und fett ndern
        ATargetCanvas.Font.Color := clBlue;
        ATargetCanvas.Font.Style := ATargetCanvas.Font.Style + [fsBold];
      end;
    end;
  end;
end;

function TfrmDODViewEditTableFieldList.RetieveNodeViewField(ANode: PVirtualNode): TDODLibEditViewField;
var
  LNodeData: PDODTableFieldListNodeData;
begin
  Result := nil;

  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      Result := LNodeData.ViewField;
    end;
  end;
end;

function TfrmDODViewEditTableFieldList.RetrieveEditClass(): TibeFrmCommonEditClass;
begin
  Result := nil;
end;

end.

