unit FDODViewFieldEdit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.Actions, Vcl.ActnList,
  Vcl.StdCtrls, Vcl.ExtCtrls,
  ibeFCommonEdit,
  ibeDataObjectDefinition, ibeDataObjectTypes,
  FDODLibCommonEdit,
  DODLibEditClasses;

type
  TfrmDODViewFieldEdit = class(TfrmDODLibCommonEdit)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    cboTableFields: TComboBox;
    cboAggFunc: TComboBox;
    cboDataType: TComboBox;
    txtCaption: TEdit;
    chkVisible: TCheckBox;
    txtCaptionTblField: TEdit;
    Label5: TLabel;
    txtColumnWidth: TEdit;
    Label6: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FDODView: TDODLibEditView;
    FDODViewField: TDODLibEditViewField;
  protected
    procedure DataLoad(); override;
    procedure DataDisplay(); override;
    procedure DataStore(); override;
    procedure DataSave(); override;
  public
    property DODView: TDODLibEditView read FDODView write FDODView;
    property DODViewField: TDODLibEditViewField read FDODViewField;
  end;

implementation

{$R *.dfm}

procedure TfrmDODViewFieldEdit.FormCreate(Sender: TObject);
var
  LDODDataType: TibeDODefDataType;
begin
  inherited;

  FDODViewField := TDODLibEditViewField.Create();

  cboDataType.Clear();
  for LDODDataType := Low(TibeDODefDataType) to High(TibeDODefDataType) do
  begin
    cboDataType.Items.Add(DODefDataTypeIDArray[LDODDataType]);
  end;
end;

procedure TfrmDODViewFieldEdit.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FDODViewField);
end;

procedure TfrmDODViewFieldEdit.DataLoad();
var
  i: Integer;
  LField: TDODLibEditViewField;
  LTblFieldList: TibeDODefTableFieldList;
  LAggFct: TibeDODefAggFunc;
begin
  LTblFieldList := TibeDODefTableFieldList.Create(False);
  try
    DODMain.TableFields.AssignToDODTableFieldList(LTblFieldList);

    if EditMode in [emBrowse, emEdit] then
    begin
      LField := TDODLibEditViewField(DODView.Fields.ItemByID[PKValue]);

      if Assigned(LField) then
      begin
        DODViewField.Assign(LField, LTblFieldList);
      end;
    end;

    cboTableFields.Clear();
    for i := 0 to Pred(DODMain.TableFields.Count) do
    begin
      cboTableFields.Items.AddObject(DODMain.TableFields[i].Data.Alias, DODMain.TableFields[i]);
    end;

    cboAggFunc.Clear();
    for LAggFct := Low(TibeDODefAggFunc) to High(TibeDODefAggFunc) do
    begin
      cboAggFunc.Items.Add(DODefAggFuncIDArray[LAggFct]);
    end;
  finally
    FreeAndNil(LTblFieldList);
  end;
end;

procedure TfrmDODViewFieldEdit.DataDisplay();
begin
  cboTableFields.ItemIndex := cboTableFields.Items.IndexOf(DODViewField.Data.TblFieldAlias);
  cboAggFunc.ItemIndex := cboAggFunc.Items.IndexOf(DODefAggFuncIDArray[DODViewField.Data.AggFunc]);
  cboDataType.ItemIndex := cboDataType.Items.IndexOf(DODefDataTypeIDArray[DODViewField.Data.DataType]);
  txtCaptionTblField.Text := DODViewField.Data.TblFieldCaption;
  txtCaption.Text := DODViewField.Data.Caption;
  txtColumnWidth.Text := IntToStr(DODViewField.Data.ColumnWidth);
  chkVisible.Checked := DODViewField.Data.VisibleInList;
end;

procedure TfrmDODViewFieldEdit.DataStore();
var
  LAggFunc: TibeDODefAggFunc;
  LDODTableFieldList: TibeDODefTableFieldList;
begin
  LDODTableFieldList := TibeDODefTableFieldList.Create(False);
  try
    LAggFunc := DODefAggFuncTypeFromString(cboAggFunc.Text);

    DODMain.TableFields.AssignToDODTableFieldList(LDODTableFieldList);

    DODViewField.Data.InitFieldAttributes(cboTableFields.Text, LAggFunc, LDODTableFieldList);

    DODViewField.Data.DataType := DODefDataTypeFromString(cboDataType.Text);
    DODViewField.Data.Caption := txtCaption.Text;
    DODViewField.Data.ColumnWidth := StrToInt(txtColumnWidth.Text);
    DODViewField.Data.VisibleInList := chkVisible.Checked;
  finally
    FreeAndNil(LDODTableFieldList);
  end;
end;

procedure TfrmDODViewFieldEdit.DataSave();
var
  LField: TDODLibEditViewField;
  LTblFieldList: TibeDODefTableFieldList;
begin
  PKValue := DODViewField.ID;

  LTblFieldList := TibeDODefTableFieldList.Create(False);
  try
    DODMain.TableFields.AssignToDODTableFieldList(LTblFieldList);

    if EditMode = emEdit then
    begin
      // mit der ID aus der Edit-Form nach dem Tabellenfeld im DOD suchen. Dem
      // gefunden Tabellenfeld den Inhalt des Felds aus der Edit-Form zuweisen
      LField := TDODLibEditViewField(DODView.Fields.ItemByID[DODViewField.ID]);

      if Assigned(LField) then
      begin
        LField.Assign(DODViewField, LTblFieldList);
      end;
    end else if EditMode = emNew then
    begin
      // ein neues Feldobjekt erzeugen und in das DOD einfgen
      LField := TDODLibEditViewField.Create();
      LField.Assign(DODViewField, LTblFieldList);
      DODView.Fields.Add(LField);
    end;
  finally
    FreeAndNil(LTblFieldList);
  end;
end;

end.

