unit FDODViewFieldList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, Vcl.Menus, System.Actions,
  Vcl.ActnList, Vcl.StdCtrls, Vcl.ExtCtrls, System.ImageList, System.StrUtils,
  Vcl.ImgList, Vcl.ToolWin, Vcl.ComCtrls,
  VirtualTrees, ibeVirtualTreeview,
  ibeDataObjectDefinition, ibeDataObjectTypes,
  ibeFCommonEdit,
  DODLibEditClasses,
  FDODLibCommonList,
  FDODViewOrderByList, FDODViewGroupByList;

type
  TfrmDODViewFieldList = class(TfrmDODLibCommonList)
    pnlViewData: TPanel;
    txtName: TEdit;
    Label5: TLabel;
    Label9: TLabel;
    txtDescription: TMemo;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    pnlFieldListBottom: TPanel;
    spFieldListBottomHeight: TSplitter;
    pnlOrderByList: TPanel;
    pnlGroupByList: TPanel;
    spOrderByWidth: TSplitter;
    actViewTest: TAction;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    procedure pnlFieldListBottomResize(Sender: TObject);
    procedure actViewTestExecute(Sender: TObject);
  private
    FFrmOrderBy: TfrmDODViewOrderByList;
    FFrmGroupBy: TfrmDODViewGroupByList;
    FDODView: TDODLibEditView;
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
    procedure SetDODView(ADODView: TDODLibEditView);
    procedure OnEndUpdateViewNotification(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionRefresh(); override;
    procedure InternalActionDisplay(); override;
    procedure InternalActionDelete(); override;
    procedure DoBeforeEdit(AEditForm: TibeFrmCommonEdit; AEditMode: TibeEditMode); override;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
    function RetrieveEditClass(): TibeFrmCommonEditClass; override;
    function RetieveNodeField(ANode: PVirtualNode): TDODLibEditViewField;
    procedure SetDODMain(ADOD: TDODLibDataObjectDefinition); override;
    property FrmOrderBy: TfrmDODViewOrderByList read FFrmOrderBy;
    property FrmGroupBy: TfrmDODViewGroupByList read FFrmGroupBy;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
    property DODView: TDODLibEditView read FDODView write SetDODView;
  end;

implementation

uses
  FDODViewFieldEdit, FDODViewTestContainer;

type

TDODViewFieldListNodeData = record
  Field: TDODLibEditViewField;
end;
PDODViewFieldListNodeData = ^TDODViewFieldListNodeData;

{$R *.dfm}

procedure TfrmDODViewFieldList.actViewTestExecute(Sender: TObject);
var
  LDODStream: TMemoryStream;
  LFrmTest: TfrmDODViewTestContainer;
begin
  inherited;
  LFrmTest := TfrmDODViewTestContainer.Create(Self);
  try
    LDODStream := TMemoryStream.Create();
    try
      DODMain.SaveToDODStream(LDODStream);
      LFrmTest.DODLoadFromStream(LDODStream, DODView.Name);

      LFrmTest.ActionRefresh();
      LFrmTest.Show();
    finally
      FreeAndNil(LDODStream);
    end;
  except
    FreeAndNil(LFrmTest);
    raise;
  end;
end;

constructor TfrmDODViewFieldList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  grdMain.OnGetText := grdMainGetText;
  grdMain.OnFreeNode := grdMainFreeNode;
  grdMain.NodeDataSize := SizeOf(TDODViewFieldListNodeData);

  FFrmOrderBy := TfrmDODViewOrderByList.Create(Self);
  FrmOrderBy.DisplayInContainer(pnlOrderByList);
  FrmOrderBy.Show();

  FFrmGroupBy := TfrmDODViewGroupByList.Create(Self);
  FrmGroupBy.DisplayInContainer(pnlGroupByList);
  FrmGroupBy.Show();
end;

destructor TfrmDODViewFieldList.Destroy();
begin
  inherited Destroy();

  DODMain.UnRegisterObserver(OnEndUpdateViewNotification);
end;

procedure TfrmDODViewFieldList.InternalActionRefresh();
begin
  inherited InternalActionRefresh();
end;

procedure TfrmDODViewFieldList.InternalActionDisplay();
var
  i: Integer;
  LNode: PVirtualNode;
  LData: PDODViewFieldListNodeData;
  LTblFieldList: TibeDODefTableFieldList;
begin
  inherited InternalActionDisplay();

  LTblFieldList := TibeDODefTableFieldList.Create(False);
  try
    DODMain.TableFields.AssignToDODTableFieldList(LTblFieldList);

    for i := 0 to DODView.Fields.Count - 1 do
    begin
      LNode := grdMain.AddChild(nil);
      LData := grdMain.GetNodeData(LNode);
      LData.Field := TDODLibEditViewField.Create();
      LData.Field.Assign(TDODLibEditViewField(DODView.Fields[i]), LTblFieldList);
    end;
  finally
    FreeAndNil(LTblFieldList);
  end;

  txtName.Text := DODView.Name;
  txtDescription.Text := DODView.Description;

  FrmOrderBy.DODView := DODView;
  FrmOrderBy.ActionRefresh();

  FrmGroupBy.DODView := DODView;
  FrmGroupBy.ActionRefresh();
end;

procedure TfrmDODViewFieldList.InternalActionDelete();
var
  LIndex: Integer;
  LViewField: TDODLibEditViewField;
begin
  inherited InternalActionDelete();

  LViewField := RetieveNodeField(grdMain.FocusedNode);

  if Assigned(LViewField) then
  begin
    LIndex := DODView.Fields.IndexByID[LViewField.ID];

    if LIndex >= 0 then
    begin
      DODView.Fields.Delete(LIndex);
    end;
  end;
end;

procedure TfrmDODViewFieldList.DoBeforeEdit(AEditForm: TibeFrmCommonEdit; AEditMode: TibeEditMode);
begin
  inherited DoBeforeEdit(AEditForm, AEditMode);

  TfrmDODViewFieldEdit(AEditForm).DODView := DODView;
end;

procedure TfrmDODViewFieldList.OnEndUpdateViewNotification(ASenderOfEvent: TDODObserverSenderOfEvent; ATypeOfEvent: TDODObserverTypeOfEvent; AObjectID: Integer);
begin
  ActionRefresh();
end;

procedure TfrmDODViewFieldList.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();

  AddGridMainColumn('Feld Name', 150);
  AddGridMainColumn('Beschriftung', 150);
  AddGridMainColumn('Spaltenbreite', 100);
  AddGridMainColumn('Datentyp', 100);
  AddGridMainColumn('Sichtbar', 150);
  AddGridMainColumn('Aggregatfunktion', 100);
  AddGridMainColumn('Tabellenfeld Name', 150);
end;

procedure TfrmDODViewFieldList.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LField: TDODLibEditViewField;
begin
  ACellText := '';

  LField := RetieveNodeField(ANode);

  if Assigned(LField) then
  begin
    case AColumn of
      0: ACellText := LField.Data.FieldName;
      1: ACellText := IfThen(LField.Data.Caption <> '', LField.Data.Caption, LField.Data.TblFieldCaption);
      2: ACellText := IntToStr(LField.Data.ColumnWidth);
      3: ACellText := DODefDataTypeIDArray[LField.Data.DataType];
      4: ACellText := IfThen(LField.Data.VisibleInList, 'ja', 'nein');
      5: ACellText := DODefAggFuncIDArray[LField.Data.AggFunc];
      6: ACellText := LField.Data.TblFieldAlias;
    end;
  end;
end;

procedure TfrmDODViewFieldList.grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
var
  LNodeData: PDODViewFieldListNodeData;
begin
  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      FreeAndNil(LNodeData.Field);
    end;
  end;
end;

function TfrmDODViewFieldList.GetPKValue(): Variant;
var
  LField: TDODLibEditViewField;
begin
  Result := Null;

  LField := RetieveNodeField(grdMain.FocusedNode);

  if Assigned(LField) then
  begin
    Result := LField.ID;
  end;
end;

procedure TfrmDODViewFieldList.SetPKValue(APKValue: Variant);
var
  LNode: PVirtualNode;
  LField: TDODLibEditViewField;
begin
  LNode := grdMain.GetFirst();

  while Assigned(LNode) do
  begin
    LField := RetieveNodeField(LNode);

    if Assigned(LField) then
    begin
      if APKValue = LField.ID then
      begin
        SelectNode(LNode);
        Break;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

function TfrmDODViewFieldList.RetrieveEditClass(): TibeFrmCommonEditClass;
begin
  Result := TfrmDODViewFieldEdit;
end;

function TfrmDODViewFieldList.RetieveNodeField(ANode: PVirtualNode): TDODLibEditViewField;
var
  LNodeData: PDODViewFieldListNodeData;
begin
  Result := nil;

  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      Result := LNodeData.Field;
    end;
  end;
end;

procedure TfrmDODViewFieldList.SetDODMain(ADOD: TDODLibDataObjectDefinition);
begin
  inherited SetDODMain(ADOD);

  FrmOrderBy.DODMain := ADOD;
  FrmGroupBy.DODMain := ADOD;
end;

procedure TfrmDODViewFieldList.SetDODView(ADODView: TDODLibEditView);
begin
  FDODView := ADODView;

  if Assigned(ADODView) then
  begin
    DODMain.RegisterObserver(dosView, doeEndBatchUpdate, DODView.ID, OnEndUpdateViewNotification);
  end;
end;

procedure TfrmDODViewFieldList.pnlFieldListBottomResize(Sender: TObject);
begin
  inherited;
  pnlOrderByList.Width := (pnlFieldListBottom.ClientWidth - spOrderByWidth.Width) div 2;
end;

end.

