unit FDODViewGroupByEdit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.Actions, Vcl.ActnList,
  Vcl.ExtCtrls, Vcl.StdCtrls,
  ibeFCommonEdit,
  ibeDataObjectDefinition, ibeDataObjectTypes,
  DODLibEditClasses,
  FDODLibCommonEdit;

type
  TfrmDODViewGroupByEdit = class(TfrmDODLibCommonEdit)
    Label1: TLabel;
    cboViewFields: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FDODView: TDODLibEditView;
    FDODViewGroupField: TDODLibEditViewGroupByField;
  protected
    procedure DataLoad(); override;
    procedure DataDisplay(); override;
    procedure DataStore(); override;
    procedure DataSave(); override;
  public
    property DODView: TDODLibEditView read FDODView write FDODView;
    property DODViewGroupField: TDODLibEditViewGroupByField read FDODViewGroupField;
  end;

implementation

{$R *.dfm}

procedure TfrmDODViewGroupByEdit.FormCreate(Sender: TObject);
begin
  inherited;

  FDODViewGroupField := TDODLibEditViewGroupByField.Create();
end;

procedure TfrmDODViewGroupByEdit.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FDODViewGroupField);
end;

procedure TfrmDODViewGroupByEdit.DataLoad();
var
  i: Integer;
  LField: TDODLibEditViewGroupByField;
begin

  if EditMode in [emBrowse, emEdit] then
  begin
    LField := TDODLibEditViewGroupByField(DODView.GroupByFields.ItemByID[PKValue]);

    if Assigned(LField) then
    begin
      DODViewGroupField.Assign(LField);
    end;
  end;

  cboViewFields.Clear();
  for i := 0 to Pred(DODView.Fields.Count) do
  begin
    cboViewFields.Items.AddObject(DODView.Fields[i].Data.FieldName, DODView.Fields[i]);
  end;
end;

procedure TfrmDODViewGroupByEdit.DataDisplay();
begin
  cboViewFields.ItemIndex := cboViewFields.Items.IndexOf(DODViewGroupField.Data.TblFieldAlias);
end;

procedure TfrmDODViewGroupByEdit.DataStore();
var
  LViewFieldList: TibeDODefViewFieldList;
begin
  LViewFieldList := TibeDODefViewFieldList.Create(False);
  try
    DODView.Fields.AssignToDODViewFieldList(LViewFieldList);

    DODViewGroupField.Data.InitFieldAttributes(cboViewFields.Text, LViewFieldList);
  finally
    FreeAndNil(LViewFieldList);
  end;
end;

procedure TfrmDODViewGroupByEdit.DataSave();
var
  LField: TDODLibEditViewGroupByField;
begin
  PKValue := DODViewGroupField.ID;

  if EditMode = emEdit then
  begin
    // mit der ID aus der Edit-Form nach dem Tabellenfeld im DOD suchen. Dem
    // gefunden Tabellenfeld den Inhalt des Felds aus der Edit-Form zuweisen
    LField := TDODLibEditViewGroupByField(DODView.GroupByFields.ItemByID[DODViewGroupField.ID]);

    if Assigned(LField) then
    begin
      LField.Assign(DODViewGroupField);
    end;
  end else if EditMode = emNew then
  begin
    // ein neues Feldobjekt erzeugen und in das DOD einfgen
    LField := TDODLibEditViewGroupByField.Create();
    LField.Assign(DODViewGroupField);
    DODView.GroupByFields.Add(LField);
  end;
end;

end.

