unit FDODViewGroupByList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.ImageList, Vcl.ImgList,
  Vcl.Menus, System.Actions, Vcl.ActnList, Vcl.StdCtrls, Vcl.ToolWin, Vcl.ComCtrls, Vcl.ExtCtrls,
  VirtualTrees, ibeVirtualTreeview,
  ibeDataObjectTypes,
  ibeFCommonEdit,
  DODLibEditClasses,
  FDODLibCommonList;

type
  TfrmDODViewGroupByList = class(TfrmDODLibCommonList)
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
  private
    FDODView: TDODLibEditView;
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionDisplay(); override;
    procedure InternalActionDelete(); override;
    procedure DoBeforeEdit(AEditForm: TibeFrmCommonEdit; AEditMode: TibeEditMode); override;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
    function RetrieveEditClass(): TibeFrmCommonEditClass; override;
    function RetieveNodeField(ANode: PVirtualNode): TDODLibEditViewGroupByField;
  public
    constructor Create(AOwner: TComponent); override;
    property DODView: TDODLibEditView read FDODView write FDODView;
  end;

implementation

uses
  FDODViewGroupByEdit;

type

TDODViewGroupByListNodeData = record
  Field: TDODLibEditViewGroupByField;
end;
PDODViewGroupByListNodeData = ^TDODViewGroupByListNodeData;

{$R *.dfm}

constructor TfrmDODViewGroupByList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  grdMain.OnGetText := grdMainGetText;
  grdMain.OnFreeNode := grdMainFreeNode;
  grdMain.NodeDataSize := SizeOf(TDODViewGroupByListNodeData);
end;

procedure TfrmDODViewGroupByList.InternalActionDisplay();
var
  i: Integer;
  LNode: PVirtualNode;
  LData: PDODViewGroupByListNodeData;
begin
  inherited InternalActionDisplay();

  for i := 0 to DODView.GroupByFields.Count - 1 do
  begin
    LNode := grdMain.AddChild(nil);
    LData := grdMain.GetNodeData(LNode);
    LData.Field := TDODLibEditViewGroupByField.Create();
    LData.Field.Assign(TDODLibEditViewGroupByField(DODView.GroupByFields[i]));
  end;
end;

procedure TfrmDODViewGroupByList.InternalActionDelete();
var
  LIndex: Integer;
  LGroupField: TDODLibEditViewGroupByField;
begin
  inherited InternalActionDelete();

  LGroupField := RetieveNodeField(grdMain.FocusedNode);

  if Assigned(LGroupField) then
  begin
    LIndex := DODView.GroupByFields.IndexByID[LGroupField.ID];

    if LIndex >= 0 then
    begin
      DODView.GroupByFields.Delete(LIndex);
    end;
  end;
end;

procedure TfrmDODViewGroupByList.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();

  AddGridMainColumn('Tabellenfeld Name', 150);
end;

procedure TfrmDODViewGroupByList.DoBeforeEdit(AEditForm: TibeFrmCommonEdit; AEditMode: TibeEditMode);
begin
  inherited DoBeforeEdit(AEditForm, AEditMode);

  TfrmDODViewGroupByEdit(AEditForm).DODView := DODView;
end;

procedure TfrmDODViewGroupByList.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LField: TDODLibEditViewGroupByField;
begin
  ACellText := '';

  LField := RetieveNodeField(ANode);

  if Assigned(LField) then
  begin
    case AColumn of
      0: ACellText := LField.Data.TblFieldAlias;
    end;
  end;
end;

procedure TfrmDODViewGroupByList.grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
var
  LNodeData: PDODViewGroupByListNodeData;
begin
  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      FreeAndNil(LNodeData.Field);
    end;
  end;
end;

function TfrmDODViewGroupByList.GetPKValue(): Variant;
var
  LField: TDODLibEditViewGroupByField;
begin
  Result := Null;

  LField := RetieveNodeField(grdMain.FocusedNode);

  if Assigned(LField) then
  begin
    Result := LField.ID;
  end;
end;

procedure TfrmDODViewGroupByList.SetPKValue(APKValue: Variant);
var
  LNode: PVirtualNode;
  LField: TDODLibEditViewGroupByField;
begin
  LNode := grdMain.GetFirst();

  while Assigned(LNode) do
  begin
    LField := RetieveNodeField(LNode);

    if Assigned(LField) then
    begin
      if APKValue = LField.ID then
      begin
        SelectNode(LNode);
        Break;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

function TfrmDODViewGroupByList.RetrieveEditClass(): TibeFrmCommonEditClass;
begin
  Result := TfrmDODViewGroupByEdit;
end;

function TfrmDODViewGroupByList.RetieveNodeField(ANode: PVirtualNode): TDODLibEditViewGroupByField;
var
  LNodeData: PDODViewGroupByListNodeData;
begin
  Result := nil;

  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      Result := LNodeData.Field;
    end;
  end;
end;

end.

