unit FDODViewOrderByEdit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.Actions, Vcl.ActnList,
  Vcl.ExtCtrls,
  ibeFCommonEdit,
  ibeDataObjectDefinition, ibeDataObjectTypes,
  DODLibEditClasses,
  FDODLibCommonEdit, Vcl.StdCtrls;

type
  TfrmDODViewOrderByEdit = class(TfrmDODLibCommonEdit)
    Label1: TLabel;
    cboViewFields: TComboBox;
    Label2: TLabel;
    cboDirection: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FDODView: TDODLibEditView;
    FDODViewOrderField: TDODLibEditViewOrderByField;
  protected
    procedure DataLoad(); override;
    procedure DataDisplay(); override;
    procedure DataStore(); override;
    procedure DataSave(); override;
  public
    property DODView: TDODLibEditView read FDODView write FDODView;
    property DODViewOrderField: TDODLibEditViewOrderByField read FDODViewOrderField;
  end;

implementation

{$R *.dfm}

procedure TfrmDODViewOrderByEdit.FormCreate(Sender: TObject);
var
  LDirection: TibeOrderByDirection;
begin
  inherited;

  FDODViewOrderField := TDODLibEditViewOrderByField.Create();

  cboDirection.Clear();
  for LDirection := Low(TibeOrderByDirection) to High(TibeOrderByDirection) do
  begin
    cboDirection.Items.Add(DODefOrderByIDArray[LDirection]);
  end;
end;

procedure TfrmDODViewOrderByEdit.FormDestroy(Sender: TObject);
begin
  inherited;

  FreeAndNil(FDODViewOrderField);
end;

procedure TfrmDODViewOrderByEdit.DataLoad();
var
  i: Integer;
  LField: TDODLibEditViewOrderByField;
begin

  if EditMode in [emBrowse, emEdit] then
  begin
    LField := TDODLibEditViewOrderByField(DODView.OrderByFields.ItemByID[PKValue]);

    if Assigned(LField) then
    begin
      DODViewOrderField.Assign(LField);
    end;
  end;

  cboViewFields.Clear();
  for i := 0 to Pred(DODView.Fields.Count) do
  begin
    cboViewFields.Items.AddObject(DODView.Fields[i].Data.FieldName, DODView.Fields[i]);
  end;
end;

procedure TfrmDODViewOrderByEdit.DataDisplay();
begin
  cboViewFields.ItemIndex := cboViewFields.Items.IndexOf(DODViewOrderField.Data.TblFieldAlias);
  cboDirection.ItemIndex := cboDirection.Items.IndexOf(DODefOrderByIDArray[DODViewOrderField.Data.Direction]);
end;

procedure TfrmDODViewOrderByEdit.DataStore();
var
  LDirection: TibeOrderByDirection;
  LViewFieldList: TibeDODefViewFieldList;
begin
  LViewFieldList := TibeDODefViewFieldList.Create(False);
  try
    DODView.Fields.AssignToDODViewFieldList(LViewFieldList);

    LDirection := DODefOrderByTypeFromString(cboDirection.Text);
    DODViewOrderField.Data.InitFieldAttributes(cboViewFields.Text, LViewFieldList, LDirection);
  finally
    FreeAndNil(LViewFieldList);
  end;
end;

procedure TfrmDODViewOrderByEdit.DataSave();
var
  LField: TDODLibEditViewOrderByField;
begin
  PKValue := DODViewOrderField.ID;

  if EditMode = emEdit then
  begin
    // mit der ID aus der Edit-Form nach dem Tabellenfeld im DOD suchen. Dem
    // gefunden Tabellenfeld den Inhalt des Felds aus der Edit-Form zuweisen
    LField := TDODLibEditViewOrderByField(DODView.OrderByFields.ItemByID[DODViewOrderField.ID]);

    if Assigned(LField) then
    begin
      LField.Assign(DODViewOrderField);
    end;
  end else if EditMode = emNew then
  begin
    // ein neues Feldobjekt erzeugen und in das DOD einfgen
    LField := TDODLibEditViewOrderByField.Create();
    LField.Assign(DODViewOrderField);
    DODView.OrderByFields.Add(LField);
  end;
end;

end.

