unit FDODViewOrderByList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.ImageList, Vcl.ImgList,
  Vcl.Menus, System.Actions, Vcl.ActnList, Vcl.StdCtrls, Vcl.ToolWin, Vcl.ComCtrls,
  Vcl.ExtCtrls,
  VirtualTrees, ibeVirtualTreeview,
  ibeDataObjectTypes,
  ibeFCommonEdit,
  DODLibEditClasses,
  FDODLibCommonList;

type
  TfrmDODViewOrderByList = class(TfrmDODLibCommonList)
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
  private
    FDODView: TDODLibEditView;
    procedure grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
    procedure grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionDisplay(); override;
    procedure InternalActionDelete(); override;
    procedure DoBeforeEdit(AEditForm: TibeFrmCommonEdit; AEditMode: TibeEditMode); override;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
    function RetrieveEditClass(): TibeFrmCommonEditClass; override;
    function RetieveNodeField(ANode: PVirtualNode): TDODLibEditViewOrderByField;
  public
    constructor Create(AOwner: TComponent); override;
    property DODView: TDODLibEditView read FDODView write FDODView;
  end;

implementation

uses
  FDODViewOrderByEdit;

type

TDODViewOrderByListNodeData = record
  Field: TDODLibEditViewOrderByField;
end;
PDODViewOrderByListNodeData = ^TDODViewOrderByListNodeData;

{$R *.dfm}

constructor TfrmDODViewOrderByList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  grdMain.OnGetText := grdMainGetText;
  grdMain.OnFreeNode := grdMainFreeNode;
  grdMain.NodeDataSize := SizeOf(TDODViewOrderByListNodeData);
end;

procedure TfrmDODViewOrderByList.InternalActionDisplay();
var
  i: Integer;
  LNode: PVirtualNode;
  LData: PDODViewOrderByListNodeData;
begin
  inherited InternalActionDisplay();

  for i := 0 to DODView.OrderByFields.Count - 1 do
  begin
    LNode := grdMain.AddChild(nil);
    LData := grdMain.GetNodeData(LNode);
    LData.Field := TDODLibEditViewOrderByField.Create();
    LData.Field.Assign(TDODLibEditViewOrderByField(DODView.OrderByFields[i]));
  end;
end;

procedure TfrmDODViewOrderByList.InternalActionDelete();
var
  LIndex: Integer;
  LOrderField: TDODLibEditViewOrderByField;
begin
  inherited InternalActionDelete();

  LOrderField := RetieveNodeField(grdMain.FocusedNode);

  if Assigned(LOrderField) then
  begin
    LIndex := DODView.OrderByFields.IndexByID[LOrderField.ID];

    if LIndex >= 0 then
    begin
      DODView.OrderByFields.Delete(LIndex);
    end;
  end;
end;

procedure TfrmDODViewOrderByList.DoBeforeEdit(AEditForm: TibeFrmCommonEdit; AEditMode: TibeEditMode);
begin
  inherited DoBeforeEdit(AEditForm, AEditMode);

  TfrmDODViewOrderByEdit(AEditForm).DODView := DODView;
end;

procedure TfrmDODViewOrderByList.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();

  AddGridMainColumn('Tabellenfeld Name', 150);
  AddGridMainColumn('Sortierung', 100);
end;

procedure TfrmDODViewOrderByList.grdMainGetText(ASender: TBaseVirtualTree; ANode: PVirtualNode; AColumn: TColumnIndex; ATextType: TVSTTextType; var ACellText: string);
var
  LField: TDODLibEditViewOrderByField;
begin
  ACellText := '';

  LField := RetieveNodeField(ANode);

  if Assigned(LField) then
  begin
    case AColumn of
      0: ACellText := LField.Data.TblFieldAlias;
      1: ACellText := DODefOrderByIDArray[LField.Data.Direction];
    end;
  end;
end;

procedure TfrmDODViewOrderByList.grdMainFreeNode(ASender: TBaseVirtualTree; ANode: PVirtualNode);
var
  LNodeData: PDODViewOrderByListNodeData;
begin
  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      FreeAndNil(LNodeData.Field);
    end;
  end;
end;

function TfrmDODViewOrderByList.GetPKValue(): Variant;
var
  LField: TDODLibEditViewOrderByField;
begin
  Result := Null;

  LField := RetieveNodeField(grdMain.FocusedNode);

  if Assigned(LField) then
  begin
    Result := LField.ID;
  end;
end;

procedure TfrmDODViewOrderByList.SetPKValue(APKValue: Variant);
var
  LNode: PVirtualNode;
  LField: TDODLibEditViewOrderByField;
begin
  LNode := grdMain.GetFirst();

  while Assigned(LNode) do
  begin
    LField := RetieveNodeField(LNode);

    if Assigned(LField) then
    begin
      if APKValue = LField.ID then
      begin
        SelectNode(LNode);
        Break;
      end;
    end;

    LNode := grdMain.GetNext(LNode);
  end;
end;

function TfrmDODViewOrderByList.RetrieveEditClass(): TibeFrmCommonEditClass;
begin
  Result := TfrmDODViewOrderByEdit;
end;

function TfrmDODViewOrderByList.RetieveNodeField(ANode: PVirtualNode): TDODLibEditViewOrderByField;
var
  LNodeData: PDODViewOrderByListNodeData;
begin
  Result := nil;

  if Assigned(ANode) then
  begin
    LNodeData := grdMain.GetNodeData(ANode);

    if Assigned(LNodeData) then
    begin
      Result := LNodeData.Field;
    end;
  end;
end;

end.

