unit FDODViewTestList;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, Vcl.Menus,
  System.Actions, Vcl.ActnList, Vcl.StdCtrls, Vcl.ExtCtrls,
  ibeFCommonDataList,
  ibeAppServiceProviderDB,
  DODEditGenericDataObject;

type

TSQLChangedNotification = procedure(const ASQL: string) of object;

  TfrmDODViewTestList = class(TibeFrmCommonDataList)
    Label1: TLabel;
    txtFilter: TEdit;
    chkVisibleFieldsOnly: TCheckBox;
  private
    FDOMain: TDODEditGenericDataObject;
    FOnSQLChanged: TSQLChangedNotification;
		function GetASP: TibeAppServiceProviderDB;
  protected
    procedure InternalActionInitGridMain(); override;
    procedure DoBeforeRefresh(); override;
    procedure InternalActionRefresh(); override;
    procedure FilterApply(); override;
    property DOMain: TDODEditGenericDataObject read FDOMain;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
    procedure DODLoadFromStream(ADODStream: TStream; const AActiveViewName: string);
    property OnSQLChanged: TSQLChangedNotification read FOnSQLChanged write FOnSQLChanged;
		property ASP: TibeAppServiceProviderDB read GetASP;
  end;

implementation

{$R *.dfm}

constructor TfrmDODViewTestList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  txtFilter.Text := '1 = 0';  // initiale Filtereinstellung: keine Datenstze laden

  FDOMain := TDODEditGenericDataObject.Create(ASP.Connection);
end;

destructor TfrmDODViewTestList.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FDOMain);
end;

procedure TfrmDODViewTestList.DODLoadFromStream(ADODStream: TStream; const AActiveViewName: string);
begin
  DOMain.LoadFromStream(ADODStream);
  DOMain.SelectActiveView(AActiveViewName);

  PKName := DOMain.PKName;
end;

procedure TfrmDODViewTestList.InternalActionInitGridMain();
var
  i: Integer;
begin
  inherited InternalActionInitGridMain();

  for i := 0 to DOMain.FieldCount - 1 do
  begin
    if DOMain.FieldByIndex[i].VisibleInList or not chkVisibleFieldsOnly.Checked then
    begin
      AddDataColumn(DOMain.FieldByIndex[i].FieldName, DOMain.FieldByIndex[i].Caption, DOMain.FieldByIndex[i].ColumnWidth);
    end;
  end;
end;

procedure TfrmDODViewTestList.DoBeforeRefresh();
begin
  inherited DoBeforeRefresh();
  // dafr sorgen, dass die Spalten neu aufgebaut werden
  grdMain.Header.Columns.Clear();
end;

procedure TfrmDODViewTestList.InternalActionRefresh();
begin
  inherited InternalActionRefresh();

  if Assigned(OnSQLChanged) then
  begin
    OnSQLChanged(DOMain.SQLText);
  end;

  DOMain.Refresh();
  DOMain.DataCopyToDataset(DatMain);
end;

procedure TfrmDODViewTestList.FilterApply();
begin
  DOMain.FilterClear();

  if Trim(txtFilter.Text) <> '' then
  begin
    DOMain.FilterAdd(Trim(txtFilter.Text));
  end;
end;

function TfrmDODViewTestList.GetASP(): TibeAppServiceProviderDB;
begin
	Result := TibeAppServiceProviderDB(inherited ASP);
end;

end.

