unit FCodeMeterMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, VirtualTrees, ExtCtrls, ActnList, Menus, System.Actions,
  ibeFCommonList,
  tolCodeMeterFile;

type
  TfrmCodeMeterMain = class(TibeFrmCommonList)
    dlgSelectRootFolder: TFileOpenDialog;
    txtRootFolder: TEdit;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FRootFolder: TCodeMeterFolder;
    procedure grdMainGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);
    procedure AddFolders(AFolders: TCodeMeterFolderList; AParentNode: PVirtualNode);
  protected
    procedure InternalActionInitGridMain(); override;
    procedure InternalActionRefresh(); override;
    procedure InternalActionDisplay(); override;
    function GetPKValue(): Variant; override;
    procedure SetPKValue(APKValue: Variant); override;
  public
  end;

var
  frmCodeMeterMain: TfrmCodeMeterMain;

implementation

type

TNodeData = record
  Folder: TCodeMeterFolder;
end;
PNodeData = ^TNodeData;

{$R *.dfm}

procedure TfrmCodeMeterMain.FormCreate(Sender: TObject);
var
  LTreeCol: TVirtualTreeColumn;
begin
  inherited;

  grdMain.NodeDataSize := SizeOf(TNodeData);
  grdMain.OnGetText := grdMainGetText;

  FRootFolder := TCodeMeterFolder.Create(nil, 'Dev');
end;

procedure TfrmCodeMeterMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FRootFolder);
end;

procedure TfrmCodeMeterMain.InternalActionInitGridMain();
begin
  inherited InternalActionInitGridMain();

  grdMain.Header.AutoSizeIndex := 0;
  grdMain.Header.Options := grdMain.Header.Options + [hoAutoResize];
  grdMain.DisplayAsTree := True;

  // Spalte fr Orndername
  AddGridMainColumn('Name', 200);
  // Total-Spalten (Code + Form)
  AddGridMainColumn('Units', 75, taRightJustify);
  AddGridMainColumn('Unit LOC', 75, taRightJustify);
  AddGridMainColumn('Unit Bytes', 75, taRightJustify);
  // Code-Spalten
  AddGridMainColumn('Code Units', 75, taRightJustify);
  AddGridMainColumn('Code LOC', 75, taRightJustify);
  AddGridMainColumn('Code Bytes', 75, taRightJustify);
  // Form-Spalten
  AddGridMainColumn('Forms', 75, taRightJustify);
  AddGridMainColumn('Form LOC', 75, taRightJustify);
  AddGridMainColumn('Form Bytes', 75, taRightJustify);
  AddGridMainColumn('Form DFM Bytes', 100, taRightJustify);
end;

procedure TfrmCodeMeterMain.InternalActionRefresh();
begin
  inherited InternalActionRefresh();

  FRootFolder.Clear();
  FRootFolder.ScanFolder(dlgSelectRootFolder.FileName, True);
end;

procedure TfrmCodeMeterMain.InternalActionDisplay();
begin
  inherited InternalActionDisplay();

  AddFolders(FRootFolder.Folders, nil);
end;

function TfrmCodeMeterMain.GetPKValue(): Variant;
var
  LData: PNodeData;
begin
  Result := Null;

  LData := grdMain.GetNodeData(grdMain.FocusedNode);

  if Assigned(LData) then
  begin
    if Assigned(LData.Folder) then
    begin
      Result := LData.Folder.FolderName;
    end;
  end;
end;

procedure TfrmCodeMeterMain.SetPKValue(APKValue: Variant);
begin

end;

procedure TfrmCodeMeterMain.Button1Click(Sender: TObject);
begin
  if dlgSelectRootFolder.Execute then
  begin
    txtRootFolder.Text := dlgSelectRootFolder.FileName;
    Application.ProcessMessages();

    ActionRefresh();
  end;
end;

procedure TfrmCodeMeterMain.AddFolders(AFolders: TCodeMeterFolderList; AParentNode: PVirtualNode);
var
  LNode: PVirtualNode;
  LData: PNodeData;
  LFolder: TCodeMeterFolder;
begin
  for LFolder in AFolders do
  begin
    LNode := grdMain.AddChild(AParentNode);
    LData := grdMain.GetNodeData(LNode);
    LData.Folder := LFolder;

    if LFolder.Folders.Count > 0 then
    begin
      AddFolders(LFolder.Folders, LNode);
    end;
  end;
end;

procedure TfrmCodeMeterMain.grdMainGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);
  function IntToStringWithThousandSep(AValue: Int64): string;
  var
    i: Integer;
    LStr: string;
  begin
    LStr := IntToStr(Abs(AValue));

    if FormatSettings.ThousandSeparator >= ' ' then
    begin
      i := Length(LStr) - 2;

      while i > 1 do
      begin
        Insert(FormatSettings.ThousandSeparator, LStr, i);
        Dec(i, 3);
      end;

      if AValue < 0 then 
      begin
        LStr := '-' + Result;
      end;
    end;

    Result := LStr;
  end;
 
var
  LData: PNodeData;
begin
  LData := grdMain.GetNodeData(Node);
  
  case Column of
     0: CellText := LData.Folder.FolderName;
     1: CellText := IntToStringWithThousandSep(LData.Folder.TotalUnitCount);
     2: CellText := IntToStringWithThousandSep(LData.Folder.TotalUnitLOC);
     3: CellText := IntToStringWithThousandSep(LData.Folder.TotalUnitSize);
     4: CellText := IntToStringWithThousandSep(LData.Folder.CodeCount);
     5: CellText := IntToStringWithThousandSep(LData.Folder.CodeUnitLOC);
     6: CellText := IntToStringWithThousandSep(LData.Folder.CodeUnitSize);
     7: CellText := IntToStringWithThousandSep(LData.Folder.FormCount);
     8: CellText := IntToStringWithThousandSep(LData.Folder.FormUnitLOC);
     9: CellText := IntToStringWithThousandSep(LData.Folder.FormUnitSize);
    10: CellText := IntToStringWithThousandSep(LData.Folder.FormDFMSize);
  end;
end;

end.

